/* Eirik's Miscellaneous Modified NetBeans Libraries.
   Written or modified (as noted) by Eirik Bakke 2013-2016, ebakke@alum.mit.edu. I elect to include
   this software in this distribution under the GPL Version 2 (with Classpath Exception) license. */

/* This file is based on parts of:
 *   org.openide.explorer.view.SheetCell
 *     (with listed author Jan Rojcek)
 * The complete license header for the original file can be found below.
 */
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2009 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package com.sieuferd.upstream.laf;

import com.sieuferd.uiutil.api.TableColors;
import java.awt.Color;
import javax.swing.UIManager;
import org.openide.util.lookup.ServiceProvider;

@ServiceProvider(service = TableColors.class)
public final class SheetCell extends TableColors {
  public SheetCell() { }

  private Color noFocusSelectionBackground = null;

  @Override
  public Color getNoFocusSelectionBackground() {
    if (noFocusSelectionBackground == null) {
      //allow theme/ui custom definition
      noFocusSelectionBackground =
          UIManager.getColor("nb.explorer.noFocusSelectionBackground"); //NOI18N
      if (noFocusSelectionBackground == null) {
        //try to get standard shadow color
        noFocusSelectionBackground = UIManager.getColor("controlShadow"); //NOI18N
        if (noFocusSelectionBackground == null) {
          //Okay, the look and feel doesn't suport it, punt
          noFocusSelectionBackground = Color.lightGray;
        }
        //Lighten it a bit because disabled text will use controlShadow/gray
        noFocusSelectionBackground = betterBrighter(noFocusSelectionBackground);
      }
    }
    return noFocusSelectionBackground;
  }

  private Color betterBrighter(Color c) {
    Color bb = c.brighter();
    if (Color.WHITE.equals(bb)) {
      bb = new Color((255 + c.getRed()) / 2, (255 + c.getGreen()) / 2, (255 + c.getBlue()) / 2);
    }
    return bb;
  }

  private Color noFocusSelectionForeground = null;

  @Override
  public Color getNoFocusSelectionForeground() {
    if (noFocusSelectionForeground == null) {
      //allow theme/ui custom definition
      noFocusSelectionForeground
          = UIManager.getColor("nb.explorer.noFocusSelectionForeground"); //NOI18N
      if (noFocusSelectionForeground == null) {
        //try to get standard shadow color
        noFocusSelectionForeground = UIManager.getColor("textText"); //NOI18N
        if (noFocusSelectionForeground == null) {
          //Okay, the look and feel doesn't suport it, punt
          noFocusSelectionForeground = Color.BLACK;
        }
      }
    }
    return noFocusSelectionForeground;
  }
}
