/* Eirik's Miscellaneous Modified NetBeans Libraries.
   Written or modified (as noted) by Eirik Bakke 2013-2016, ebakke@alum.mit.edu. I elect to include
   this software in this distribution under the GPL Version 2 (with Classpath Exception) license. */

/* This file is based on:
 *   org.netbeans.modules.editor.settings.storage.fontscolors.ColoringStorage.ColoringsWriter
 *     (with listed authors Jan Jancura, Vita Stejskal)
 * The complete license header for the original file can be found below.
 */
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package com.sieuferd.upstream.localeditor;

import java.awt.Color;
import java.util.Set;
import javax.swing.text.AttributeSet;
import javax.swing.text.StyleConstants;
import org.netbeans.api.editor.settings.EditorStyleConstants;
import org.openide.xml.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public final class ColoringsWriter {
  private static final String E_ROOT = "fontscolors"; //NOI18N
  private static final String E_FONTCOLOR = "fontcolor"; //NOI18N
  private static final String E_FONT = "font"; //NOI18N
  private static final String A_NAME = "name"; //NOI18N
  private static final String A_FOREGROUND = "foreColor"; //NOI18N
  private static final String A_BACKGROUND = "bgColor"; //NOI18N
  private static final String A_STRIKETHROUGH = "strikeThrough"; //NOI18N
  private static final String A_WAVEUNDERLINE = "waveUnderlined"; //NOI18N
  private static final String A_UNDERLINE = "underline"; //NOI18N
  private static final String A_DEFAULT = "default"; //NOI18N
  private static final String A_SIZE = "size"; //NOI18N
  private static final String A_STYLE = "style"; //NOI18N
  private static final String V_BOLD_ITALIC = "bold+italic"; //NOI18N
  private static final String V_BOLD = "bold"; //NOI18N
  private static final String V_ITALIC = "italic"; //NOI18N
  private static final String V_PLAIN = "plain"; //NOI18N

  private static final String PUBLIC_ID =
      "-//NetBeans//DTD Editor Fonts and Colors settings 1.1//EN"; //NOI18N
  private static final String SYSTEM_ID =
      "http://www.netbeans.org/dtds/EditorFontsColors-1_1.dtd"; //NOI18N

  public static final String FA_TYPE = "nbeditor-settings-ColoringType"; //NOI18N
  public static final String FAV_TOKEN = "token"; //NOI18N
  public static final String FAV_HIGHLIGHT = "highlight"; //NOI18N
  public static final String FAV_ANNOTATION = "annotation"; //NOI18N

  private ColoringsWriter() { }

  public static Document getDocument(Set<AttributeSet> categories) {
    final Document doc = XMLUtil.createDocument(E_ROOT, null, PUBLIC_ID, SYSTEM_ID);
    final Node root = doc.getElementsByTagName(E_ROOT).item(0);

    for (AttributeSet category : categories) {
      Element fontColor = doc.createElement(E_FONTCOLOR);
      root.appendChild(fontColor);
      fontColor.setAttribute(A_NAME, (String) category.getAttribute(StyleConstants.NameAttribute));

      if (category.isDefined(StyleConstants.Foreground)) {
        fontColor.setAttribute(
            A_FOREGROUND,
            colorToString((Color) category.getAttribute(StyleConstants.Foreground))
        );
      }
      if (category.isDefined(StyleConstants.Background)) {
        fontColor.setAttribute(
            A_BACKGROUND,
            colorToString((Color) category.getAttribute(StyleConstants.Background))
        );
      }
      if (category.isDefined(StyleConstants.StrikeThrough)) {
        fontColor.setAttribute(
            A_STRIKETHROUGH,
            colorToString((Color) category.getAttribute(StyleConstants.StrikeThrough))
        );
      }
      if (category.isDefined(EditorStyleConstants.WaveUnderlineColor)) {
        fontColor.setAttribute(
            A_WAVEUNDERLINE,
            colorToString((Color) category.getAttribute(EditorStyleConstants.WaveUnderlineColor))
        );
      }
      if (category.isDefined(StyleConstants.Underline)) {
        fontColor.setAttribute(
            A_UNDERLINE,
            colorToString((Color) category.getAttribute(StyleConstants.Underline))
        );
      }
      if (category.isDefined(EditorStyleConstants.Default)) {
        fontColor.setAttribute(
            A_DEFAULT,
            (String) category.getAttribute(EditorStyleConstants.Default)
        );
      }

      if (category.isDefined(StyleConstants.FontFamily)
          || category.isDefined(StyleConstants.FontSize)
          || category.isDefined(StyleConstants.Bold)
          || category.isDefined(StyleConstants.Italic))
      {
        final Element font = doc.createElement(E_FONT);
        fontColor.appendChild(font);

        if (category.isDefined(StyleConstants.FontFamily)) {
          font.setAttribute(
              A_NAME,
              (String) category.getAttribute(StyleConstants.FontFamily)
          );
        }
        if (category.isDefined(StyleConstants.FontSize)) {
          font.setAttribute(
              A_SIZE,
              ((Integer) category.getAttribute(StyleConstants.FontSize)).toString()
          );
        }
        if (category.isDefined(StyleConstants.Bold)
            || category.isDefined(StyleConstants.Italic))
        {
          Boolean bold = Boolean.FALSE, italic = Boolean.FALSE;
          if (category.isDefined(StyleConstants.Bold))
            bold = (Boolean) category.getAttribute(StyleConstants.Bold);
          if (category.isDefined(StyleConstants.Italic))
            italic = (Boolean) category.getAttribute(StyleConstants.Italic);

          font.setAttribute(A_STYLE, bold
              ? (italic ? V_BOLD_ITALIC : V_BOLD)
              : (italic ? V_ITALIC : V_PLAIN)
          );
        }
      }
    }
    return doc;
  }

  private static String colorToString(Color color) {
    return Integer.toHexString(color.getRGB());
  }
}
