/* Eirik's Miscellaneous Modified NetBeans Libraries.
   Written or modified (as noted) by Eirik Bakke 2013-2016, ebakke@alum.mit.edu. I elect to include
   this software in this distribution under the GPL Version 2 (with Classpath Exception) license. */

/* This file is based in part on:
 *   org.netbeans.modules.editor.NbEditorToolBar
 *   org.openide.awt.Actions
 * The complete license headers for the original files can be found below (identical for both
 * files).
 */
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package com.sieuferd.upstream.toolbar;

import com.google.common.base.Preconditions;
import com.sieuferd.uiutil.api.CommonUIUtil;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.AbstractButton;
import javax.swing.JToggleButton;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public final class ToolbarUtil {
  private ToolbarUtil() { }

  public static void setBorderOnHover(AbstractButton button) {
    BorderUpdater bu = new BorderUpdater(button);
    button.addMouseListener(bu);
    button.addFocusListener(bu);
    button.getModel().addChangeListener(bu);
    bu.update();
  }

  private static final class BorderUpdater
      extends MouseAdapter implements FocusListener, ChangeListener
  {
    private final AbstractButton button;
    private final boolean aqua = CommonUIUtil.isAqua();
    boolean mouseEntered;

    public BorderUpdater(AbstractButton button) {
      Preconditions.checkNotNull(button);
      this.button = button;
    }

    public @Override
    void mouseEntered(MouseEvent evt) {
      mouseEntered = true;
      update();
    }

    @Override
    public void mouseExited(MouseEvent evt) {
      mouseEntered = false;
      update();
    }

    @Override
    public void focusGained(FocusEvent e) {
      update();
    }

    @Override
    public void focusLost(FocusEvent e) {
      update();
    }

    @Override
    public void stateChanged(ChangeEvent e) {
      update();
    }

    void update() {
      setBorderVisible(button.isEnabled() && (mouseEntered || button.hasFocus()));
    }

    private void setBorderVisible(boolean visible) {
      if (visible) {
        /* The Aqua L&F will paint outside the button's visual boundary if contentAreaFilled is
        true, but on the other hand does not require contentAreaFilled to be true in order to
        paint the inside of the button. For these reasons, setting contentAreaFilled to false
        makes the buttons look better when drawn on top of other graphics, e.g. an ArrowWidget. */
        if (!aqua)
          button.setContentAreaFilled(true);
        button.setBorderPainted(true);
      } else {
        boolean canRemove = true;
        if (button instanceof JToggleButton)
          canRemove = !button.isSelected();
        if (canRemove) {
          button.setContentAreaFilled(false);
          button.setBorderPainted(false);
        }
      }
    }
  }

  /** @return may be empty, but never null */
  public static String getDisplayTextForAccelerator(KeyStroke accelerator) {
    Preconditions.checkNotNull(accelerator);
    // Based on org.openide.awt.Actions.findKey(Action)
    int modifiers = accelerator.getModifiers();
    String acceleratorText = ""; // NOI18N
    if (modifiers > 0) {
      acceleratorText = KeyEvent.getKeyModifiersText(modifiers);
      acceleratorText += "+"; // NOI18N
    } else if (accelerator.getKeyCode() == KeyEvent.VK_UNDEFINED) {
      return ""; // NOI18N
    }
    acceleratorText += KeyEvent.getKeyText(accelerator.getKeyCode());
    return acceleratorText;
  }
}
