/* Eirik's Miscellaneous Modified NetBeans Libraries.
   Written or modified (as noted) by Eirik Bakke 2013-2016, ebakke@alum.mit.edu. I elect to include
   this software in this distribution under the GPL Version 2 (with Classpath Exception) license. */

/* This file is based in part on:
 *   org.netbeans.modules.editor.NbEditorToolBar
 * The complete license header for the original file can be found below.
 */
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package com.sieuferd.upstream.toolbar;

import com.google.common.base.Preconditions;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeListener;
import java.lang.ref.Reference;
import javax.swing.Action;
import javax.swing.JComponent;

/**
 * Wraps an action to execute with a different component as a source.
 *
 * Originally based on org.netbeans.modules.editor.NbEditorToolBar.WrapperAction
 * (author Miloslav Metelka).
 */
public final class WrapperAction implements Action {
  private final Reference<JComponent> componentRef;
  private final Action delegate;

  public WrapperAction(Reference<JComponent> componentRef, Action delegate) {
    Preconditions.checkNotNull(componentRef);
    Preconditions.checkNotNull(delegate);
    this.componentRef = componentRef;
    this.delegate = delegate;
  }

  @Override
  public Object getValue(String key) {
    return delegate.getValue(key);
  }

  @Override
  public void putValue(String key, Object value) {
    delegate.putValue(key, value);
  }

  @Override
  public void setEnabled(boolean b) {
    delegate.setEnabled(b);
  }

  @Override
  public boolean isEnabled() {
    return delegate.isEnabled();
  }

  @Override
  public void addPropertyChangeListener(PropertyChangeListener listener) {
    delegate.addPropertyChangeListener(listener);
  }

  @Override
  public void removePropertyChangeListener(PropertyChangeListener listener) {
    delegate.removePropertyChangeListener(listener);
  }

  @Override
  public void actionPerformed(ActionEvent e) {
    final JComponent c = componentRef.get();
    if (c != null)
      e = new ActionEvent(c, e.getID(), e.getActionCommand());
    delegate.actionPerformed(e);
  }
}
