/* Eirik's Miscellaneous Swing Utilities.
   Written by Eirik Bakke 2013-2016, ebakke@alum.mit.edu. */

package com.sieuferd.uiutil.api;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.EventQueue;
import java.awt.Frame;
import java.awt.Window;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

public class CommonUIUtil {
  // Based on JOptionPane.getWindowForComponent()
  public static Window getWindowForComponent(Component component) {
    if (component == null)
      return JOptionPane.getRootFrame();
    if (component instanceof Frame || component instanceof Dialog)
      return (Window) component;
    return getWindowForComponent(component.getParent());
  }

  /**
   * Is Swing running with the Aqua L&F? Note that this can be false on MacOS if a different L&F is
   * used.
   */
  public static boolean isAqua() {
    return "Aqua".equals(UIManager.getLookAndFeel().getID());
  }

  public static void checkNotOnEDT() {
    if (EventQueue.isDispatchThread())
      throw new IllegalStateException("This method may not be called on the EDT");
  }

  public static void checkOnEDT() {
    if (!EventQueue.isDispatchThread())
      throw new IllegalStateException("This method may only be called on the EDT");
  }
}
