/* Eirik's Miscellaneous Swing Utilities.
   Written by Eirik Bakke 2013-2016, ebakke@alum.mit.edu. */

package com.sieuferd.uiutil.api;

import java.awt.Color;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import javax.annotation.Nullable;
import javax.swing.JTable;
import javax.swing.UIManager;
import org.openide.util.Lookup;

public abstract class TableColors {
  private static final TableColors DEFAULT = new TableColors() {
    @Override
    public Color getNoFocusSelectionBackground() {
      return Color.lightGray;
    }

    @Override
    public Color getNoFocusSelectionForeground() {
      return Color.black;
    }
  };

  public static TableColors getInstance() {
    TableColors ret = Lookup.getDefault().lookup(TableColors.class);
    return ret == null ? DEFAULT : ret;
  }

  public abstract Color getNoFocusSelectionBackground();
  public abstract Color getNoFocusSelectionForeground();

  /* On Aqua, JTable.getSelectionBackground/Foreground returns different colors depending on whether
  the JTable has focus or not. This might be OK, except the actual color returned is often out of
  sync with the actual focus state of the JTable (on JDK 1.8.0_25). For this reason, calculate all
  colors explicitly here. */

  public @Nullable Color getFocusSelectionBackground() {
    return UIManager.getColor("Table.selectionBackground");
  }

  public @Nullable Color getFocusSelectionForeground() {
    return UIManager.getColor("Table.selectionForeground");
  }

  private Color getFocusSelectionBackground(JTable table) {
    Color ret = getFocusSelectionBackground();
    return ret == null ? table.getSelectionBackground() : ret;
  }

  private Color getFocusSelectionForeground(JTable table) {
    Color ret = getFocusSelectionForeground();
    return ret == null ? table.getSelectionForeground() : ret;
  }

  public final Color getBackground(JTable table, boolean selected) {
    return selected
        ? (table.hasFocus() ? getFocusSelectionBackground(table) : getNoFocusSelectionBackground())
        : table.getBackground();
  }

  public final Color getForeground(JTable table, boolean selected) {
    return selected
        ? (table.hasFocus() ? getFocusSelectionForeground(table) : getNoFocusSelectionForeground())
        : table.getForeground();
  }

  public static void registerFocusRepaint(final JTable table) {
    table.addFocusListener(new FocusListener() {
      @Override
      public void focusGained(FocusEvent e) {
        table.repaint();
      }

      @Override
      public void focusLost(FocusEvent e) {
        table.repaint();
      }
    });
  }
}
