// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.builders;

import com.sieuferd.cell.support.Cell;
import com.sieuferd.cell.impl.BorderCell;
import com.sieuferd.cell.support.BiasedLocation;
import com.sieuferd.cell.support.Direction;
import com.google.common.base.Preconditions;
import java.awt.Color;
import com.sieuferd.cell.styles.BorderStyle;
import com.sieuferd.cell.styles.LineStyle;
import com.sieuferd.cell.support.Location;
import com.sieuferd.cell.support.RenderContext;
import com.sieuferd.datamodel.ViewProperty;
import com.sieuferd.datamodel.ViewPropertySource;
import com.sieuferd.uiutil.UIUtil;
import javax.annotation.Nullable;

public class BorderCellBuilder extends CellBuilder<BorderCellBuilder> {
  private final Cell childCell;
  private Color backgroundColor = UIUtil.COLOR_NONE;
  private LineStyle borderTop = LineStyle.NONE;
  private LineStyle borderRight = LineStyle.NONE;
  private LineStyle borderBottom = LineStyle.NONE;
  private LineStyle borderLeft = LineStyle.NONE;
  private @Nullable Location location = null;
  private Direction cursorBias = Direction.NONE;
  private boolean justified = false;

  public BorderCellBuilder(
      RenderContext renderContext, ViewPropertySource viewPropertySource, Cell childCell)
  {
    super(renderContext, viewPropertySource);
    this.childCell = childCell;
  }

  public BorderCellBuilder backgroundColor(@Nullable ViewProperty<Color> backgroundColor) {
    this.backgroundColor =
        backgroundColor == null ? UIUtil.COLOR_NONE : viewPropertySource.get(backgroundColor);
    return this;
  }

  public BorderCellBuilder backgroundColor(@Nullable Color backgroundColor) {
    this.backgroundColor =
        backgroundColor == null ? UIUtil.COLOR_NONE : backgroundColor;
    return this;
  }

  public BorderCellBuilder borders(LineStyle style) {
    Preconditions.checkNotNull(style);
    borderTop = style;
    borderRight = style;
    borderBottom = style;
    borderLeft = style;
    return this;
  }

  public BorderCellBuilder borders(ViewProperty<LineStyle> style) {
    Preconditions.checkNotNull(style);
    borderTop = viewPropertySource.get(style);
    borderRight = viewPropertySource.get(style);
    borderBottom = viewPropertySource.get(style);
    borderLeft = viewPropertySource.get(style);
    return this;
  }

  public BorderCellBuilder borderTop(LineStyle style) {
    Preconditions.checkNotNull(style);
    borderTop = style;
    return this;
  }

  public BorderCellBuilder borderRight(LineStyle style) {
    Preconditions.checkNotNull(style);
    borderRight = style;
    return this;
  }

  public BorderCellBuilder borderBottom(LineStyle style) {
    Preconditions.checkNotNull(style);
    borderBottom = style;
    return this;
  }

  public BorderCellBuilder borderLeft(LineStyle style) {
    Preconditions.checkNotNull(style);
    borderLeft = style;
    return this;
  }

  public BorderCellBuilder borderTop(ViewProperty<LineStyle> style) {
    Preconditions.checkNotNull(style);
    borderTop = viewPropertySource.get(style);
    return this;
  }

  public BorderCellBuilder borderRight(ViewProperty<LineStyle> style) {
    Preconditions.checkNotNull(style);
    borderRight = viewPropertySource.get(style);
    return this;
  }

  public BorderCellBuilder borderBottom(ViewProperty<LineStyle> style) {
    Preconditions.checkNotNull(style);
    borderBottom = viewPropertySource.get(style);
    return this;
  }

  public BorderCellBuilder borderLeft(ViewProperty<LineStyle> style) {
    Preconditions.checkNotNull(style);
    borderLeft = viewPropertySource.get(style);
    return this;
  }

  public BorderCellBuilder location(@Nullable Location location) {
    this.location = location;
    return this;
  }

  public BorderCellBuilder cursorBias(Direction cursorBias) {
    Preconditions.checkNotNull(cursorBias);
    this.cursorBias = cursorBias;
    return this;
  }

  public BorderCellBuilder justify() {
    justified = true;
    return this;
  }

  @Override
  public Cell build() {
    // TODO[cc]: Figure out who uses minimumDimensions and how.
    return new BorderCell(renderContext, minimumDimensions, childCell,
        new BorderStyle(borderTop, borderRight, borderBottom, borderLeft),
        backgroundColor, new BiasedLocation(cursorBias, location), justified);
  }

  @Override
  public BorderCellBuilder getThis() {
    return this;
  }
}
