// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.builders;

import com.sieuferd.cell.support.Cell;
import com.google.common.base.Preconditions;
import com.sieuferd.cell.support.Coordinates;
import com.sieuferd.cell.support.Pixels;
import com.sieuferd.cell.support.RenderContext;
import com.sieuferd.datamodel.ViewPropertySource;
import javax.annotation.Nullable;

// TODO[cc]: Consider removing the Data Model dependency from this module.
/** Clients can invoke builder methods in a chain so long as they call the methods from subclasses
first. The latter requirement arises from the fact that the methods in this class return a less
specific type of "this". Some generics magic could be used to avoid this requirement, but it would
be overkill. */
public abstract class CellBuilder<T extends CellBuilder<?>> {
  protected final RenderContext renderContext;
  protected final @Nullable ViewPropertySource viewPropertySource;
  protected Coordinates minimumDimensions = Coordinates.ZERO;

  public CellBuilder(RenderContext renderContext, @Nullable ViewPropertySource viewPropertySource) {
    Preconditions.checkNotNull(renderContext);
    this.renderContext = renderContext;
    this.viewPropertySource = viewPropertySource;
  }

  public T minimumDimensions(Coordinates minimumDimensions) {
    Preconditions.checkNotNull(minimumDimensions);
    this.minimumDimensions = this.minimumDimensions.max(minimumDimensions);
    return getThis();
  }

  public T minimumWidth(Pixels width) {
    return minimumDimensions(new Coordinates(width, Pixels.ZERO));
  }

  public T minimumHeight(Pixels height) {
    return minimumDimensions(new Coordinates(Pixels.ZERO, height));
  }

  public abstract Cell build();
  public abstract T getThis();
}
