// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.builders;

import com.sieuferd.cell.support.Cell;
import com.sieuferd.cell.impl.EmptyCell;
import com.sieuferd.cell.support.Direction;
import com.google.common.base.Preconditions;
import com.sieuferd.cell.impl.HorizontalBarCell;
import com.sieuferd.cell.support.Axis;
import com.sieuferd.cell.support.RenderContext;
import com.sieuferd.datamodel.ViewPropertySource;
import java.awt.Color;
import java.awt.Font;

public class EmptyCellBuilder extends CellBuilder<EmptyCellBuilder> {
  private Direction cursorBias = Direction.NONE;

  public EmptyCellBuilder(RenderContext renderContext, ViewPropertySource viewPropertySource) {
    super(renderContext, viewPropertySource);
  }

  public EmptyCellBuilder cursorBias(Direction cursorBias) {
    Preconditions.checkNotNull(cursorBias);
    this.cursorBias = cursorBias;
    return this;
  }

  @Override
  public Cell build() {
    return new EmptyCell(renderContext, minimumDimensions, cursorBias);
  }

  public Cell buildHorizontalBar(Font sizingFont, Color negativeColor, Color positiveColor,
      Axis axis, double value)
  {
    return new HorizontalBarCell(renderContext, minimumDimensions, cursorBias,
        sizingFont, negativeColor, positiveColor, axis, value);
  }

  @Override
  public EmptyCellBuilder getThis() {
    return this;
  }
}
