// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.builders;

import com.sieuferd.cell.support.Cell;
import com.sieuferd.cell.impl.HeaderedCell;
import com.google.common.base.Preconditions;
import com.sieuferd.cell.support.RenderContext;
import com.sieuferd.datamodel.ViewPropertySource;

/* This builder is unnecessary since it has no optional arguments and all mandatory arguments are
taken in the constructor. Keep it for consistency with other cell types. */
public class HeaderedCellBuilder extends CellBuilder<HeaderedCellBuilder> {
  private final boolean vertical;
  private final Cell headerCell;
  private final Cell contentCell;

  public HeaderedCellBuilder(RenderContext renderContext, ViewPropertySource viewPropertySource,
      boolean vertical, Cell headerCell, Cell contentCell)
  {
    super(renderContext, viewPropertySource);
    Preconditions.checkNotNull(headerCell);
    Preconditions.checkNotNull(contentCell);
    this.vertical = vertical;
    this.headerCell = headerCell;
    this.contentCell = contentCell;
  }

  @Override
  public Cell build() {
    return new HeaderedCell(renderContext, headerCell, contentCell, vertical);
  }

  @Override
  public HeaderedCellBuilder getThis() {
    return this;
  }
}
