// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.builders;

import com.sieuferd.cell.support.Cell;
import com.sieuferd.cell.impl.ListCell;
import com.google.common.collect.Lists;
import java.util.Arrays;
import java.util.List;
import com.sieuferd.cell.support.Coordinates;
import com.sieuferd.cell.support.RenderContext;
import com.sieuferd.datamodel.ViewPropertySource;

public class ListCellBuilder extends CellBuilder<ListCellBuilder> {
  private final boolean vertical;
  private final List<Cell> childCells = Lists.newArrayList();

  public ListCellBuilder(
      RenderContext renderContext, ViewPropertySource viewPropertySource, boolean vertical)
  {
    super(renderContext, viewPropertySource);
    this.vertical = vertical;
  }

  public ListCellBuilder addCell(Cell cell) {
    childCells.add(cell);
    return this;
  }

  public ListCellBuilder addCells(Cell... cells) {
    childCells.addAll(Arrays.asList(cells));
    return this;
  }

  @Override
  public ListCellBuilder minimumDimensions(Coordinates minimumDimensions) {
    super.minimumDimensions(minimumDimensions);
    return this;
  }

  @Override
  public Cell build() {
    if (minimumDimensions.isEqualTo(Coordinates.ZERO) && childCells.size() == 1)
      return childCells.get(0);
    return new ListCell(renderContext, minimumDimensions, vertical, childCells);
  }

  @Override
  public ListCellBuilder getThis() {
    return this;
  }
}
