// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.impl;

import com.sieuferd.cell.support.BiasedLocation;
import com.sieuferd.cell.support.CellParam;
import com.sieuferd.cell.support.Direction;
import com.google.common.base.Preconditions;
import com.sieuferd.cell.support.Cell;
import com.sieuferd.cell.support.Coordinates;
import com.sieuferd.cell.support.HitParameters;
import com.sieuferd.cell.support.Pixels;
import com.sieuferd.cell.support.RenderContext;
import javax.annotation.Nullable;

/**
 * An empty cell.
 *
 * May be subclassed.
 */
public class EmptyCell extends Cell {
  private final Coordinates dimensions;
  private final Direction localCursorBias;

  /* The localCursorBias is provided as part of EmptyCell as a convenience; alternatively, an
  EmptyCell can be wrapped in a BorderCell. */

  public EmptyCell(RenderContext renderContext, Coordinates dimensions, Direction localCursorBias) {
    super(renderContext);
    Preconditions.checkNotNull(dimensions);
    Preconditions.checkNotNull(localCursorBias);
    this.dimensions = dimensions;
    this.localCursorBias = localCursorBias;
  }

  @Override
  public final Coordinates getDimensions() {
    return dimensions;
  }

  @Override
  public Coordinates getBottomRightOverprintLimit(RenderContext renderContext) {
    return Coordinates.ZERO;
  }

  /**
   * Overriding implementations should call the superclass method first to handle hitParameters.
   */
  @Override
  public void traverse(
      CellParam param, Coordinates position, Coordinates justifiedDimensions,
      @Nullable Pixels baselineOffset, Coordinates headerPoint, boolean overlayMode,
      @Nullable HitParameters hitParameters)
  {
    if (hitParameters != null)
      hitParameters.setResult(new BiasedLocation(localCursorBias, null));
  }

  @Override
  public Pixels getPaginatedHeight(RenderContext renderContext,
      Pixels startOffset, Pixels preferredMax, Pixels paperHeight) {
    checkRenderContext(renderContext);
    return getDimensions().getY().minus(startOffset).min(preferredMax);
  }

  @Override
  public final boolean containsBiasedLocation() {
    return false;
  }

  @Override
  public String toString() {
    return ".";
  }

  @Override
  public Pixels getMinimumBaselineOffset(RenderContext renderContext) {
    return Pixels.ZERO;
  }
}
