// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.styles;

import com.google.common.base.Preconditions;
import com.sieuferd.cell.support.Canvas;
import com.sieuferd.cell.support.Coordinates;
import com.sieuferd.cell.support.Pixels;
import com.sieuferd.cell.support.RenderContext;

public class BorderStyle {
  public static final BorderStyle NONE = new BorderStyle(
      LineStyle.NONE, LineStyle.NONE, LineStyle.NONE, LineStyle.NONE);
  // Keep the CSS order: clockwise, starting from "noon".
  private final LineStyle borderTop, borderRight, borderBottom, borderLeft;

  public BorderStyle(LineStyle borderTop, LineStyle borderRight,
      LineStyle borderBottom, LineStyle borderLeft)
  {
    Preconditions.checkNotNull(borderTop);
    Preconditions.checkNotNull(borderRight);
    Preconditions.checkNotNull(borderBottom);
    Preconditions.checkNotNull(borderLeft);
    this.borderTop = borderTop;
    this.borderRight = borderRight;
    this.borderBottom = borderBottom;
    this.borderLeft = borderLeft;
  }

  @Override
  public String toString() {
    return "BorderStyle(" +
        borderTop + ", " + borderRight + ", " + borderBottom + ", " + borderLeft + ")";
  }

  public Coordinates topLeftBorderDimensions(RenderContext renderContext) {
    return new Coordinates(
        borderLeft.measure(renderContext, true),
        borderTop.measure(renderContext, false));
  }

  public Coordinates bottomRightBorderDimensions(RenderContext renderContext) {
    return new Coordinates(
        borderRight.measure(renderContext, true),
        borderBottom.measure(renderContext, false));
  }

  /** Borders are painted hanging down bottom-right from the rectangle specified by the position and
  dimensions. Border stacking policy:
    <pre>
    |-----|
    |     |
    |     |
    |-----|
    </pre>
  */
  public void paintBorder(Canvas canvas, Coordinates position, Coordinates dimensions) {
    final Coordinates topLeft = topLeftBorderDimensions(canvas.getRenderContext());
    final Coordinates bottomRight = bottomRightBorderDimensions(canvas.getRenderContext());

    /* TODO[pd]: Draw this as a rectangle Shape instead, and see if that fixes overshooting lines in
                 rendering of PDFs generated by printing to Adobe Acrobat's PDF printer. */
    borderTop.paint(canvas, false, position.plus(
        new Coordinates(topLeft.getX(), Pixels.ZERO)),
        dimensions.getX().minus(topLeft.getX()));
    borderBottom.paint(canvas, false, position.plus(
        new Coordinates(topLeft.getX(), dimensions.getY())),
        dimensions.getX().minus(topLeft.getX()));
    borderLeft.paint(canvas, true, position.plus(
        new Coordinates(Pixels.ZERO, Pixels.ZERO)),
        dimensions.getY().plus(bottomRight.getY()));
    borderRight.paint(canvas, true, position.plus(
        new Coordinates(dimensions.getX(), Pixels.ZERO)),
        dimensions.getY().plus(bottomRight.getY()));
  }
}
