// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.styles;

import com.google.common.base.Preconditions;
import java.text.BreakIterator;
import java.util.Locale;
import javax.annotation.concurrent.Immutable;

// TODO[cc]: Consider renaming.
@Immutable
public final class LineBreakPolicy {
  public static final LineBreakPolicy DEFAULT_PARAGRAPH = createParagraphPolicy(Locale.US);
  public static final LineBreakPolicy CHARACTER_POLICY =
      new LineBreakPolicy("CP", BreakIterator.getCharacterInstance(Locale.US));
  private final String identifier;
  /** Must not be modified; only provide clones of this object. */
  private final BreakIterator breakIteratorPrototype;

  private LineBreakPolicy(String identifier, BreakIterator breakIteratorPrototype) {
    Preconditions.checkNotNull(identifier);
    Preconditions.checkNotNull(breakIteratorPrototype);
    this.identifier = identifier;
    this.breakIteratorPrototype = breakIteratorPrototype;
  }

  public static LineBreakPolicy createParagraphPolicy(Locale locale) {
    return new LineBreakPolicy(
        "PP:" + locale.toLanguageTag(), BreakIterator.getLineInstance(locale));
  }

  public BreakIterator createBreakIterator() {
    // The BreakIterator.getXInstance methods all rely on cloning, so I can assume this is safe.
    return (BreakIterator) breakIteratorPrototype.clone();
  }

  @Override
  public int hashCode() {
    return identifier.hashCode();
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof LineBreakPolicy))
      return false;
    LineBreakPolicy other = (LineBreakPolicy) obj;
    return other.identifier.equals(identifier);
  }
}
