// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.styles;

import com.google.common.collect.ImmutableMap;
import com.sieuferd.cell.support.Location;
import com.sieuferd.datamodel.Path;
import java.awt.Color;
import java.util.Map;
import java.util.Objects;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class PathHighlights {
  public static final PathHighlights NONE = new PathHighlights(ImmutableMap.of(), null);
  private final ImmutableMap<Path,Color> highlights;
  private final @Nullable Location editLocation;

  public PathHighlights(Map<Path,Color> highlights, @Nullable Location editLocation) {
    this.highlights = ImmutableMap.copyOf(highlights);
    this.editLocation = editLocation;
  }

  public Map<Path,Color> getHighlights() {
    return highlights;
  }

  public @Nullable Location getEditLocation() {
    return editLocation;
  }

  @Override
  public int hashCode() {
    return Objects.hash(highlights, editLocation);
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof PathHighlights))
      return false;
    PathHighlights other = (PathHighlights) obj;
    return Objects.equals(this.getHighlights(), other.getHighlights())
        && Objects.equals(this.getEditLocation(), other.getEditLocation());
  }
}
