// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.styles;

import com.google.common.base.Preconditions;
import com.sieuferd.cell.support.RenderContext;
import java.awt.font.TextAttribute;
import java.text.AttributedString;
import java.util.Map;
import javax.annotation.concurrent.Immutable;

/**
 * A styled string. Implementations should be immutable and provide appropriate implementations of
 * {@code equals}/{@code hashCode}.
 */
@Immutable
public abstract class Text {
  public static final Text EMPTY = new SimpleText("");

  public abstract String getString();

  /**
   * Get a fresh {@code AttributedString} over the string returned by {@link #getString()}. May not
   * be called if said string is empty.
   *
   * @param originalAttributes should include {@link TextAttribute#SIZE}
   */
  public AttributedString createAttributedString(
      Map<TextAttribute,?> originalAttributes, RenderContext rcx)
  {
    Preconditions.checkNotNull(originalAttributes);
    Preconditions.checkNotNull(rcx);
    Preconditions.checkArgument(originalAttributes.containsKey(TextAttribute.SIZE));
    Preconditions.checkState(!getString().isEmpty());
    return new AttributedString(getString(), originalAttributes);
  }

  @Override
  public String toString() {
    return getString();
  }

  public static final class SimpleText extends Text {
    private final String string;

    public SimpleText(String string) {
      Preconditions.checkNotNull(string);
      this.string = string;
    }

    @Override
    public String getString() {
      return string;
    }

    @Override
    public int hashCode() {
      return string.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
      if (obj == this)
        return true;
      if (!(obj instanceof SimpleText))
        return false;
      SimpleText other = (SimpleText) obj;
      return this.string.equals(other.string);
    }
  }
}
