// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.support;

import com.google.common.base.Preconditions;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class Axis {
  public static final Axis INVALID = new Axis(0.0, 0.0);
  public final double start;
  public final double end;

  private Axis(double start, double end) {
    Preconditions.checkArgument(Double.isFinite(start));
    Preconditions.checkArgument(Double.isFinite(end));
    Preconditions.checkArgument(start <= end);
    this.start = start;
    this.end = end;
  }

  public static final Axis create(double start, double end) {
    return Double.isFinite(start) && Double.isFinite(end) && start < end
      ? new Axis(start, end)
      : INVALID;
  }

  /**
   * @param value any floating-point value, including NaN or infinity
   * @return the {@code value} scaled and translated such that values on the axis lie between 0.0
   *         and 1.0, and values not on the axis lie correspondingly outside the 0.0 and 1.0 range.
   *         May be NaN or positive/negative infinity.
   */
  public double getRelativePosition(double value) {
    final double axisWidth = end - start;
    if (axisWidth == 0.0)
      return Double.NaN;
    return (value - start) / axisWidth;
  }
}
