// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.support;

import com.google.common.base.Preconditions;
import java.util.Objects;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

// TODO[cc]: See if I can make this class private.
@Immutable
public final class BiasedLocation {
  public static final BiasedLocation NONE = new BiasedLocation(Direction.NONE, null);
  private final Direction cursorBias;
  private final @Nullable Location location;

  public BiasedLocation(Direction cursorBias, @Nullable Location location) {
    Preconditions.checkNotNull(cursorBias);
    this.cursorBias = cursorBias;
    this.location = location;
  }

  // Introduced for debugging purposes only.
  public boolean equals(BiasedLocation other) {
    return cursorBias.equals(other.cursorBias)
        && Objects.equals(location, other.location);
  }

  public boolean isNone() {
    return cursorBias == Direction.NONE && getCursorLocation() == null;
  }

  public Direction getCursorBias() {
    return cursorBias;
  }

  public @Nullable Location getCursorLocation() {
    return (cursorBias == Direction.NONE) ? location : null;
  }

  public @Nullable Location getLocation() {
    return location;
  }

  @Override
  public String toString() {
    return cursorBias + "/" + location;
  }
}
