// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.support;

import com.google.common.base.Preconditions;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import javax.annotation.Nonnull;

public class Canvas {
  private static final PixelRectangle NO_CLIP_RECTANGLE =
      new PixelRectangle(new Rectangle(0, 0, Integer.MAX_VALUE, Integer.MAX_VALUE));
  private static final PixelRectangle EMPTY_RECTANGLE =
      new PixelRectangle(Coordinates.ZERO, Coordinates.ZERO);
  private static final boolean PERMIT_MISMATCHED_FRC = false;
  private final Graphics2D graphics;
  private final RenderContext renderContext;

  public Canvas(Graphics2D graphics, RenderContext renderContext) {
    Preconditions.checkNotNull(graphics);
    Preconditions.checkNotNull(renderContext);
    if (!PERMIT_MISMATCHED_FRC &&
        !graphics.getFontRenderContext().equals(renderContext.getFontRenderContext()))
    {
      throw new IllegalArgumentException("Mismatched FontRenderContext");
    }
    this.graphics = graphics;
    this.renderContext = renderContext;
  }

  public final @Nonnull PixelRectangle getClipBounds() {
    Rectangle ret = graphics.getClipBounds();
    if (ret == null) {
      return NO_CLIP_RECTANGLE;
    } else if (ret.width <= 0 || ret.height <= 0) {
      /* Note that Graphics2D.clipRect can produce negative clipping areas, hence the
      less-than-or-equals in the test above. For example, the following results in a clipping area
      of Rectangle[x=0,y=0,width=150,height=-50]:

        g.clipRect(0, 0, 800, 600);
        g.clipRect(0, -200, 150, 150);
      */
      return EMPTY_RECTANGLE;
    } else {
      return new PixelRectangle(ret);
    }
  }

  public Graphics2D getGraphics() {
    return graphics;
  }

  public RenderContext getRenderContext() {
    return renderContext;
  }
}
