// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.support;

import com.google.common.base.Preconditions;
import com.sieuferd.cell.support.Cell.FocusMode;
import java.util.EnumSet;
import java.util.Set;

// TODO[cc]: See if I can move this to a private package.
public final class CellPaintParameters {
  private final Canvas canvas;
  private final FocusMode focusMode;
  private final Set<Layer> layers;

  public CellPaintParameters(
      Canvas canvas, FocusMode focusMode, Set<Layer> layers)
  {
    Preconditions.checkNotNull(canvas);
    Preconditions.checkNotNull(focusMode);
    Preconditions.checkNotNull(layers);
    this.canvas = canvas;
    this.focusMode = focusMode;
    this.layers = EnumSet.copyOf(layers);
  }

  public Canvas getCanvas() {
    return canvas;
  }

  public FocusMode getFocusMode() {
    return focusMode;
  }

  public boolean includesLayer(Layer layer) {
    Preconditions.checkNotNull(layer);
    return layers.contains(layer);
  }

  public static enum Layer {
    /**
     * Cell backgrounds and borders, excluding those of frozen headers, but including those of
     * frozen values. Also includes shadows from frozen headers, painted on top of HeaderedCell
     * contents.
     */
    BACKGROUND,
    /**
     * Backgrounds and borders of frozen headers. Painted in a separate layer in order to appear
     * on top of frozen header shadows.
     */
    FROZEN_HEADER_BACKGROUND,
    /**
     * The cursor (not including its contents).
     */
    CURSOR,
    /**
     * Text and other graphics that should appear on top of all other elements, including the
     * cursor.
     */
    TOP
  }
}
