// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.support;

import com.google.common.base.Preconditions;
import com.sieuferd.cell.styles.PathHighlights;
import com.sieuferd.cell.support.Cell.FocusMode;
import com.sieuferd.cell.support.CellPaintParameters.Layer;
import java.util.Set;
import javax.annotation.Nullable;
import javax.annotation.concurrent.NotThreadSafe;

// TODO[cc]: Move this to a private package.
@NotThreadSafe
public final class CellParam {
  private final PixelRectangle clipBounds;
  private final RenderContext renderContext;
  private final Selection selection;
  private final PathHighlights pathHighlights;
  private final @Nullable CellRectangles cellRectangles;
  private final @Nullable CellPaintParameters paintParameters;
  private final @Nullable Location avoidHit;

  private CellParam(PixelRectangle clipBounds, RenderContext renderContext,
      Selection selection, PathHighlights pathHighlights, boolean findRectangles,
      @Nullable CellPaintParameters paintParameters, @Nullable Location avoidHit)
  {
    Preconditions.checkNotNull(clipBounds);
    Preconditions.checkNotNull(renderContext);
    Preconditions.checkNotNull(selection);
    Preconditions.checkNotNull(pathHighlights);
    this.clipBounds = clipBounds;
    this.renderContext = renderContext;
    this.selection = selection;
    this.pathHighlights = pathHighlights;
    this.cellRectangles = findRectangles ? new CellRectangles() : null;
    this.paintParameters = paintParameters;
    this.avoidHit = avoidHit;
  }

  public static CellParam createFindRectangles(PixelRectangle clipBounds,
      RenderContext renderContext, Selection selection, PathHighlights pathHighlights)
  {
    return new CellParam(clipBounds, renderContext, selection, pathHighlights,
        true, null, null);
  }

  public static CellParam createPaint(Canvas canvas, Selection selection,
      PathHighlights pathHighlights, FocusMode focusMode, Set<Layer> layers)
  {
    return new CellParam(canvas.getClipBounds(), canvas.getRenderContext(), selection,
        pathHighlights, false,
        new CellPaintParameters(canvas, focusMode, layers), null);
  }

  /**
   * Prepare for a hit-testing cell traversal.
   *
   * @param avoid a {@code CursorLocation} to skip past (typically the currently selected location,
   *        for keyboard-based physical cursor movements)
   */
  public static CellParam createHit(PixelRectangle clipBounds,
      RenderContext renderContext, @Nullable Location avoid)
  {
    return new CellParam(clipBounds, renderContext, Selection.NONE, PathHighlights.NONE,
        false, null, avoid);
  }

  public PixelRectangle getClipBounds() {
    return clipBounds;
  }

  public RenderContext getRenderContext() {
    return renderContext;
  }

  public Selection getSelection() {
    return selection;
  }

  public PathHighlights getPathHighlights() {
    return pathHighlights;
  }

  /**
   * @return if not null, the returned structure should be populated during the traversal
   */
  public @Nullable CellRectangles getCellRectangles() {
    return cellRectangles;
  }

  public @Nullable CellPaintParameters getPaintParameters() {
    return paintParameters;
  }

  public Location getAvoidHit() {
    return avoidHit;
  }
}
