// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.support;

import com.google.common.base.Objects;
import com.google.common.base.Preconditions;
import java.awt.Dimension;
import java.awt.Point;

/* TODO[cc]: Consider splitting this into one class for positions and one for offsets, like Swing's
             Point vs. Dimension. */
public final class Coordinates {
  public static final Coordinates ZERO = new Coordinates(Pixels.ZERO, Pixels.ZERO);

  private final Pixels x, y;

  public Coordinates(Pixels x, Pixels y) {
    Preconditions.checkNotNull(x);
    Preconditions.checkNotNull(y);
    this.x = x;
    this.y = y;
  }

  public Coordinates(int x, int y) {
    this.x = Pixels.create(x);
    this.y = Pixels.create(y);
  }

  public Coordinates(Point point) {
    this(Pixels.create(point.x), Pixels.create(point.y));
  }

  public Coordinates(Dimension dimension) {
    this(Pixels.create(dimension.width), Pixels.create(dimension.height));
  }

  public Point getPoint() {
    return new Point(x.getInteger(), y.getInteger());
  }

  public Dimension getDimension() {
    return new Dimension(x.getInteger(), y.getInteger());
  }

  public Pixels getX() {
    return x;
  }

  public Pixels getY() {
    return y;
  }

  // TODO[cc]: Rename to equals().
  public boolean isEqualTo(Coordinates other) {
    return getX().isEqualTo(other.getX()) && getY().isEqualTo(other.getY());
  }

  public Coordinates max(Coordinates other) {
    return new Coordinates(getX().max(other.getX()), getY().max(other.getY()));
  }

  public Coordinates min(Coordinates other) {
    return new Coordinates(getX().min(other.getX()), getY().min(other.getY()));
  }

  public Coordinates plus(Coordinates other) {
    return new Coordinates(getX().plus(other.getX()), getY().plus(other.getY()));
  }

  public Coordinates minus(Coordinates other) {
    return new Coordinates(getX().minus(other.getX()), getY().minus(other.getY()));
  }

  @Override
  public String toString() {
    return "(" + x.toString() + ", " + y.toString() + ")";
  }

  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof Coordinates))
      return false;
    if (obj == this)
      return true;
    return isEqualTo((Coordinates) obj);
  }

  @Override
  public int hashCode() {
    return Objects.hashCode(x, y);
  }
}
