// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.support;

import com.google.common.base.Preconditions;
import com.sieuferd.cell.styles.BorderStyle;
import com.sieuferd.cell.styles.LineStyle;
import java.awt.Color;

public final class CursorStyle {
  private final BorderStyle borderStyle;
  // TODO[cc]: Separate out a BorderDimensions class for this one.
  private final BorderStyle overlapStyle;

  public CursorStyle(BorderStyle borderStyle, BorderStyle overlapStyle)
  {
    Preconditions.checkNotNull(borderStyle);
    Preconditions.checkNotNull(overlapStyle);
    this.borderStyle = borderStyle;
    this.overlapStyle = overlapStyle;
  }

  public BorderStyle getBorderStyle() {
    return borderStyle;
  }

  public BorderStyle getOverlapStyle() {
    return overlapStyle;
  }

  // TODO[cc]: Move these constants elsewhere.
  public static final Color NEGATIVE_BAR_VISUALIZATION_COLOR = new Color(244, 66, 66, 255);
  public static final Color BLUE_CURSOR_COLOR = new Color(66, 133, 244, 255);
  // Yields the same light blue color as used in Excel when overlaid on white with 25% opacity.
  public static final Color SELECTION_COLOR = new Color(150, 200, 252, 255);
  // Grayscale version of the above.
  public static final Color SELECTION_COLOR_NOFOCUS = new Color(194, 194, 194, 255);
  public static final Color WARNING_COLOR = new Color(255, 255, 0, 255);
  // The orange color from the bottom right of the inside of the warning icon triangle.
  //public static final Color WARNING_COLOR = new Color(255, 206, 67, 255);
  // Fully saturated version of the above.
  //public static final Color WARNING_COLOR = new Color(255, 189, 0, 255);

  /* Using 2.2pt cursor thickness instead of 2.0pt guarantees that the cursor covers external
  borders of 1.0pt even in certain roundoff error cases (e.g. Zoom 8 times on Windows in
  WidgetMain). It makes the cursor stick out a little bit on very high zoom levels, which is
  fine. The thickness still comes out right at 100% on Windows, MacOS, and MacOS Retina zoom
  levels (it comes out wrong at 2.25 or above). */
  private static final Points CURSOR_THICKNESS = new Points(2.2);

  private static final LineStyle OVERLAP_LINE = new LineStyle(null, new Points(1.0));
  private static final BorderStyle OVERLAP_BORDER = new BorderStyle(
      OVERLAP_LINE, OVERLAP_LINE, OVERLAP_LINE, OVERLAP_LINE);
  private static final LineStyle HIGHLIGHT_LINE =
      new LineStyle(new Color(255, 255, 255, 128), new Points(1.0));
  private static final BorderStyle HIGHLIGHT_BORDER = new BorderStyle(
      HIGHLIGHT_LINE, HIGHLIGHT_LINE, HIGHLIGHT_LINE, HIGHLIGHT_LINE);

  private static final LineStyle LINE_BLUE =
      new LineStyle(BLUE_CURSOR_COLOR, CURSOR_THICKNESS);
  private static final LineStyle LINE_GRAY =
      new LineStyle(new Color(136, 136, 136, 255), CURSOR_THICKNESS);
  private static final LineStyle LINE_GRAY_TR =
      new LineStyle(new Color(0, 0, 0, 100), CURSOR_THICKNESS);
  private static final LineStyle LINE_BLACK =
      new LineStyle(new Color(0, 0, 0, 255), CURSOR_THICKNESS);

  private static final BorderStyle BORDER_BLUE =
      new BorderStyle(LINE_BLUE, LINE_BLUE, LINE_BLUE, LINE_BLUE);
  private static final BorderStyle BORDER_GRAY =
      new BorderStyle(LINE_GRAY, LINE_GRAY, LINE_GRAY, LINE_GRAY);
  /**
   * Translucent black that yields approximately the same level of gray as BORDER_GRAY when overlaid
   * on white or the default separator lines.
   */
  private static final BorderStyle BORDER_GRAY_TR =
      new BorderStyle(LINE_GRAY_TR, LINE_GRAY_TR, LINE_GRAY_TR, LINE_GRAY_TR);
  private static final BorderStyle BORDER_BLACK =
      new BorderStyle(LINE_BLACK, LINE_BLACK, LINE_BLACK, LINE_BLACK);

  public static final CursorStyle CURSOR_BLUE =
      new CursorStyle(BORDER_BLUE, OVERLAP_BORDER);
  public static final CursorStyle CURSOR_GRAY =
      new CursorStyle(BORDER_GRAY, OVERLAP_BORDER);
  public static final CursorStyle CURSOR_GRAY_TRANSLUCENT =
      new CursorStyle(BORDER_GRAY_TR, OVERLAP_BORDER);
  public static final CursorStyle CURSOR_BLACK =
      new CursorStyle(BORDER_BLACK, OVERLAP_BORDER);

  // TODO[cc]: Get rid of this.
  public static final BorderStyle CURSOR_DIMENSIONS =
      new BorderStyle(LINE_BLACK, LINE_BLACK, LINE_BLACK, LINE_BLACK);

  public static float BACKGROUND_TOKEN_HIGHLIGHT_OPACITY = 0.25f;
  public static float STANDARD_HIGHLIGHT_OPACITY = 0.25f;
  public static float SOFT_HIGHLIGHT_OPACITY = 0.10f;
  public static float MEDIUM_HIGHLIGHT_OPACITY = 0.60f;
  public static float HARD_HIGHLIGHT_OPACITY = 1.00f;
  public static final Color LABEL_BADGE_COLOR = new Color(0, 0, 0, 128);
}
