// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.support;

import com.google.common.base.Preconditions;
import java.util.Objects;
import javax.annotation.concurrent.Immutable;

/** A fractional distance measured in typographical points (1/72nds of an inch). */
@Immutable
public final class Points {
  public static final Points ZERO = new Points(0.0);
  private final double value;

  public Points(double value) {
    Preconditions.checkArgument(Double.isFinite(value));
    this.value = value;
  }

  /** Returns a finite double. */
  public double getValue() {
    return value;
  }

  public boolean isZero() {
    return value == 0.0;
  }

  @Override
  public String toString() {
    return value + "pt";
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof Points))
      return false;
    Points other = (Points) obj;
    return value == other.value;
  }

  @Override
  public int hashCode() {
    return Objects.hash(value);
  }
}
