// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.support;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableSet;
import com.sieuferd.datamodel.Path;
import java.util.Objects;
import java.util.Set;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class Selection {
  public static final Selection NONE = new Selection(null, ImmutableSet.of());
  private final @Nullable Location cursorLocation;
  private final ImmutableSet<Path> selectedPaths;

  public Selection(@Nullable Location cursorLocation, Set<Path> selectedPaths) {
    Preconditions.checkNotNull(selectedPaths);
    this.cursorLocation = cursorLocation;
    this.selectedPaths = ImmutableSet.copyOf(selectedPaths);
  }

  public @Nullable Location getCursorLocation() {
    return cursorLocation;
  }

  public Set<Path> getSelectedPaths() {
    return selectedPaths;
  }

  @Override
  public int hashCode() {
    return Objects.hash(cursorLocation, selectedPaths);
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof Selection))
      return false;
    Selection other = (Selection) obj;
    return Objects.equals(this.cursorLocation, other.cursorLocation)
        && Objects.equals(this.selectedPaths, other.selectedPaths);
  }

  @Override
  public String toString() {
    return "Selection(" + cursorLocation + ", " + selectedPaths + ")";
  }
}
