// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.support;

import com.google.common.base.Preconditions;
import com.sieuferd.datamodel.Path;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Set;
import javax.annotation.Nullable;

/** Mutable. Intended to be used only from the EDT. */
public class SelectionModel {
  public static final String PROP_SELECTION = "selection";
  private final PropertyChangeSupport pcs = new PropertyChangeSupport(this);

  private Selection selection = Selection.NONE;

  public SelectionModel() {
  }

  public Selection getSelection() {
    return selection;
  }

  public void setSelection(Selection selection, boolean scrollToCursor) {
    Preconditions.checkNotNull(selection);
    Selection oldValue = this.selection;
    this.selection = selection;
    pcs.firePropertyChange(
        new SelectionModelPropertyChangeEvent(this, oldValue, selection, scrollToCursor));
  }

  public void setCursorLocation(
      @Nullable Location cursorLocation, boolean scrollToCursor)
  {
    setSelection(
        new Selection(cursorLocation, getSelection().getSelectedPaths()), scrollToCursor);
  }

  public @Nullable Location getCursorLocation() {
    return getSelection().getCursorLocation();
  }

  public void setSelectedPaths(Set<Path> selectedPaths) {
    setSelection(new Selection(getCursorLocation(), selectedPaths), false);
  }

  public void addPropertyChangeListener(PropertyChangeListener listener) {
    pcs.addPropertyChangeListener(listener);
  }

  public void removePropertyChangeListener(PropertyChangeListener listener) {
    pcs.removePropertyChangeListener(listener);
  }

  @Override
  public String toString() {
    return "SelectionModel/" + getSelection().toString();
  }

  public static class SelectionModelPropertyChangeEvent extends PropertyChangeEvent {
    private static final long serialVersionUID = 1L;
    private final boolean scrollToCursor;

    public SelectionModelPropertyChangeEvent(Object source,
        Selection oldValue, Selection newValue, boolean scrollToCursor)
    {
      super(source, PROP_SELECTION, oldValue, newValue);
      this.scrollToCursor = scrollToCursor;
    }

    public boolean getScrollToCursor() {
      return scrollToCursor;
    }
  }
}
