// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.cell.support;

import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

public class PixelsTest {
  private static RenderContext createFractionalRenderContext() {
    return new RenderContext(
        new FontRenderContext(AffineTransform.getScaleInstance(1.0, 1.0), false, false),
        1.0, null, true, true);
  }
  @Test
  public void testBasic() {
    RenderContext rcx = createFractionalRenderContext();
    Pixels DIV2 = Pixels.create(rcx, 2.0);
    assertThat(Pixels.create(rcx, 1.0).getDouble(), is(1.0)); // 512
    assertThat(Pixels.create(rcx, 1.0).round(DIV2).getDouble(), is(2.0)); // 1024
    assertThat(Pixels.create(rcx, 1.4).round(DIV2).getDouble(), is(2.0)); // 1024
    assertThat(Pixels.create(rcx, 1.6).round(DIV2).getDouble(), is(2.0)); // 1024
    assertThat(Pixels.create(rcx, 0.6).round(DIV2).getDouble(), is(0.0)); // 0
    assertThat(Pixels.create(rcx, 0.4).round(DIV2).getDouble(), is(0.0)); // 0
    assertThat(Pixels.create(rcx, 2.1).round(DIV2).getDouble(), is(2.0)); // 1024
  }

  @Test
  public void testNegative() {
    RenderContext rcx = createFractionalRenderContext();
    Pixels DIV1 = Pixels.create(rcx, 1.0);
    Pixels DIV2 = Pixels.create(rcx, 2.0);
    assertThat(Pixels.create(rcx, -1.0).getDouble(), is(-1.0));
    assertThat(Pixels.create(rcx, -1.0).round(DIV2).getDouble(), is(-2.0));
    assertThat(Pixels.create(rcx, -1.4).round(DIV2).getDouble(), is(-2.0));
    assertThat(Pixels.create(rcx, -1.6).round(DIV2).getDouble(), is(-2.0));
    assertThat(Pixels.create(rcx, -0.6).round(DIV2).getDouble(), is(0.0));
    assertThat(Pixels.create(rcx, -0.4).round(DIV2).getDouble(), is(0.0));
    assertThat(Pixels.create(rcx, -2.1).round(DIV2).getDouble(), is(-2.0));
    assertThat(Pixels.create(rcx, -2.8).round(DIV2).getDouble(), is(-2.0));
    assertThat(Pixels.create(rcx, -3.1).round(DIV2).getDouble(), is(-4.0));
    assertThat(Pixels.create(rcx, -0.55).round(DIV2).getDouble(), is(0.0));
    assertThat(Pixels.create(rcx, -0.45).round(DIV2).getDouble(), is(0.0));
    assertThat(Pixels.create(rcx, -1.05).round(DIV2).getDouble(), is(-2.0));
    assertThat(Pixels.create(rcx, 0.55).round(DIV1).getDouble(), is(1.0));
    assertThat(Pixels.create(rcx, 0.45).round(DIV1).getDouble(), is(0.0));
    assertThat(Pixels.create(rcx, -0.55).round(DIV1).getDouble(), is(-1.0));
    assertThat(Pixels.create(rcx, -0.45).round(DIV1).getDouble(), is(0.0));
    assertThat(Pixels.create(rcx, -1.55).round(DIV1).getDouble(), is(-2.0));
    assertThat(Pixels.create(rcx, -1.45).round(DIV1).getDouble(), is(-1.0));
  }

  @Test
  public void testRoundBiased() {
    RenderContext rcx = createFractionalRenderContext();
    Pixels DIV1 = Pixels.create(rcx, 1.0);
    assertThat(Pixels.create(rcx, 0.55).round(DIV1).getDouble(), is(1.0));
    assertThat(Pixels.create(rcx, 0.45).round(DIV1).getDouble(), is(0.0));
    assertThat(Pixels.create(rcx, 1.0).round(DIV1).getDouble(), is(1.0));
    assertThat(Pixels.create(rcx, 1.4).round(DIV1).getDouble(), is(1.0));
    assertThat(Pixels.create(rcx, 1.6).round(DIV1).getDouble(), is(2.0));

    assertThat(Pixels.create(rcx, 0.55).roundBiased(DIV1).getDouble(), is(1.0));
    assertThat(Pixels.create(rcx, 0.45).roundBiased(DIV1).getDouble(), is(1.0));
    assertThat(Pixels.create(rcx, -0.55).roundBiased(DIV1).getDouble(), is(-1.0));
    assertThat(Pixels.create(rcx, -0.45).roundBiased(DIV1).getDouble(), is(-1.0));
    assertThat(Pixels.create(rcx, 0.0).roundBiased(DIV1).getDouble(), is(0.0));
    assertThat(Pixels.create(rcx, 1.0).roundBiased(DIV1).getDouble(), is(1.0));
    assertThat(Pixels.create(rcx, 1.4).roundBiased(DIV1).getDouble(), is(1.0));
    assertThat(Pixels.create(rcx, 1.6).roundBiased(DIV1).getDouble(), is(2.0));
  }
}
