// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import org.openide.util.NbBundle;

/** Common properties. */
public final class CProp extends ViewPropertySet {
  public static final CProp INSTANCE = new CProp();

  @NbBundle.Messages("TIT_Common=Data Model")
  private CProp() {
    super("sieuferd_common", Bundle.TIT_Common(), null, 100);
  }

  private static <T> ViewProperty<T> create(
      String storageName, AppliesTo appliesTo, ViewPropertyType<T> type)
  {
    return addProperty(INSTANCE, storageName, appliesTo, type, false);
  }

  private static <T> ViewProperty<T> createRO(
      String storageName, AppliesTo appliesTo, ViewPropertyType<T> type)
  {
    return addProperty(INSTANCE, storageName, appliesTo, type, true);
  }

  /* TODO[cc]: Consider using Boolean.TYPE instead of Boolean.class and similar for other primitive
               types. See the Javadoc for org.openide.nodes.Node.Property.getValueType(). */
  @NbBundle.Messages("TIT_ViewProperty.VISIBLE=Visible")
  public static final ViewProperty<Boolean> VISIBLE = create("VISIBLE", AppliesTo.ANY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.LABEL=Label")
  public static final ViewProperty<String> LABEL = create("LABEL", AppliesTo.ANY, ViewPropertyTypes.STRING);
  @NbBundle.Messages("TIT_ViewProperty.CANONICAL_LABEL=Canonical Label")
  public static final ViewProperty<String> CANONICAL_LABEL = createRO("CANONICAL_LABEL", AppliesTo.ANY, ViewPropertyTypes.STRING);
  @NbBundle.Messages("TIT_ViewProperty.HEADING=Heading")
  public static final ViewProperty<Boolean> HEADING = create("HEADING", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.PLURAL=Plural")
  public static final ViewProperty<Boolean> PLURAL = createRO("PLURAL", AppliesTo.RELATION_ONLY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.PROJECTED=Projected")
  public static final ViewProperty<Boolean> PROJECTED = createRO("PROJECTED", AppliesTo.ANY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.IN_TUPLE_KEY=In Tuple Key")
  public static final ViewProperty<Boolean> IN_TUPLE_KEY = createRO("IN_TUPLE_KEY", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.AFFORDANCE=Affordance")
  public static final ViewProperty<Boolean> AFFORDANCE = create("AFFORDANCE", AppliesTo.ANY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.EXPANDED=Expanded")
  public static final ViewProperty<Boolean> EXPANDED = createRO("EXPANDED", AppliesTo.RELATION_ONLY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.INHERIT_STYLES=Inherit Styles from Primary")
  public static final ViewProperty<Boolean> INHERIT_STYLES = create("INHERIT_STYLES", AppliesTo.ANY, ViewPropertyTypes.BOOLEAN);

  public static final ViewDefaults DEFAULTS = ViewDefaults.builder()
      .set(CProp.LABEL, "")
      .set(CProp.CANONICAL_LABEL, "")
      .set(CProp.HEADING, false)
      .set(CProp.VISIBLE, true)
      .set(CProp.PLURAL, true)
      .set(CProp.PROJECTED, false)
      .set(CProp.IN_TUPLE_KEY, true)
      .set(CProp.AFFORDANCE, false)
      .set(CProp.EXPANDED, false)
      .set(CProp.INHERIT_STYLES, true)
      .build();
}
