// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.sieuferd.datamodel.Path.PrimitivePath;
import com.sieuferd.datamodel.Path.RelationPath;

// TODO[cc]: Consider renaming to "Result".
public abstract class Data {
  private volatile boolean frozen;

  public abstract TupleInterval getRelationTuples(RelationPath path, Tuple parentTuple);

  // TODO[cc]: Consider removing this.
  public abstract boolean isIncomplete();

  // TODO[cc]: Get rid of this, since it can be derived outside of Data implementations.
  public abstract int getCompleteTupleCount(RelationPath path);

  public abstract Tuple getParentTuple(RelationPath childPath, Tuple childTuple);

  public abstract PrimitiveValue getTupleField(Tuple tuple, PrimitivePath path);

  public abstract boolean isProjected(Path path);

  /** Disallow further mutation on this object. */
  public final void freeze() {
    frozen = true;
  }

  public final boolean isFrozen() {
    return frozen;
  }

  /** Return a Data object suitable for use in comparisons using equals()/hashCode(). Only for use
  in assertion checking. */
  protected Data getCanonicalData() {
    return this;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof Data))
      return false;
    Data other = (Data) obj;
    return getCanonicalData() == other.getCanonicalData();
  }

  @Override
  public int hashCode() {
    return getCanonicalData() == this ? super.hashCode() : getCanonicalData().hashCode();
  }
}
