// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.sieuferd.datamodel.Path.PrimitivePath;
import com.sieuferd.datamodel.Path.RelationPath;
import javax.annotation.concurrent.Immutable;

@Immutable
public class ImmutableData extends Data {
  private final Data frozenDelegate;

  private ImmutableData(Data frozenDelegate) {
    if (!frozenDelegate.isFrozen())
      throw new IllegalArgumentException("Cannot make ImmutableData from non-frozen delegate");
    /* Safely publish frozenDelegate, which is an the Effectively Immutable Object due to having
    been frozen, to other threads through initialization of a final field (see Java Concurrency in
    Practice, section 3.5.4). */
    this.frozenDelegate = frozenDelegate;
    freeze();
  }

  @Override
  protected Data getCanonicalData() {
    return frozenDelegate;
  }

  /** @param delegateToFreeze will be frozen. */
  public static ImmutableData create(Data delegateToFreeze) {
    delegateToFreeze.freeze();
    if (delegateToFreeze instanceof ImmutableData)
      return (ImmutableData) delegateToFreeze;
    return new ImmutableData(delegateToFreeze);
  }

  @Override
  public TupleInterval getRelationTuples(RelationPath path, Tuple parentTuple) {
    return frozenDelegate.getRelationTuples(path, parentTuple);
  }

  @Override
  public boolean isIncomplete() {
    return frozenDelegate.isIncomplete();
  }

  @Override
  public int getCompleteTupleCount(RelationPath path) {
    return frozenDelegate.getCompleteTupleCount(path);
  }

  @Override
  public Tuple getParentTuple(RelationPath childPath, Tuple childTuple) {
    return frozenDelegate.getParentTuple(childPath, childTuple);
  }

  @Override
  public PrimitiveValue getTupleField(Tuple tuple, PrimitivePath path) {
    return frozenDelegate.getTupleField(tuple, path);
  }

  @Override
  public boolean isProjected(Path path) {
    return frozenDelegate.isProjected(path);
  }
}
