// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import com.sieuferd.datamodel.internal.BaseView;
import java.util.Set;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class ImmutableView extends View {
  private final View delegate;

  private ImmutableView(
      View source, @Nullable Set<ViewProperty<?>> properties, @Nullable Set<Path> paths)
  {
    MutableView toDelegate = new BaseView(ViewDefaults.EMPTY);
    toDelegate.copyFrom(source, Path.ROOT,
        properties == null ? null : Sets.union(properties, ImmutableSet.of(SProp.CHILD_PATHS)),
        paths);
    /* Safely publish a new Effectively Immutable Object to other threads through initialization of
    a final field (see Java Concurrency in Practice, section 3.5.4). */
    delegate = toDelegate;
  }

  public static ImmutableView copyOf(View source) {
    return copyOf(source, null, null);
  }

  /**
   * @param properties the properties to copy, or null to copy all properties
   * @param paths the paths to copy, or null to copy all paths
   */
  public static ImmutableView copyOf(
      View source, @Nullable Set<ViewProperty<?>> properties, @Nullable Set<Path> paths)
  {
    if (source instanceof ImmutableView) {
      /* TODO[pd]: Consider providing optimizations for other cases as well, e.g. by wrapping the
                   existing ImmutableView to hide excluded paths and properties. MutableView could
                   then cache a single ImmutableView for fast re-use. */
      if (properties == null && paths == null)
        return (ImmutableView) source;
    }
    return new ImmutableView(source, properties, paths);
  }

  @Override
  public void performReadTransaction(Runnable runnable) {
    runnable.run();
  }

  @Override
  public boolean exists(Path path) {
    return delegate.exists(path);
  }

  @Override
  public <T> T get(ViewProperty<T> property, Path path) {
    return delegate.get(property, path);
  }

  @Override
  public Set<ViewProperty<?>> getDefinedProperties(Path path) {
    return delegate.getDefinedProperties(path);
  }

  @Override
  public boolean defines(ViewProperty<?> property, Path path) {
    return delegate.defines(property, path);
  }

  @Override
  public String toString() {
    return delegate.toString();
  }
}
