// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import javax.annotation.Nullable;

public abstract class ObservableView extends View {
  /* The restriction that each listener may only be added once ensures symmetrical usage with
  removeViewListener(), which needs to take only a ViewListener as an argument in order to work with
  org.openide.util.WeakListeners. */
  /**
   * Each listener may only be added once. Note that the client is responsible for removing the
   * listeners even if pathToListenTo is removed from the view, since there exists a possibility
   * that the same path may be reintroduced at a later time. For the same reason, it is legal to
   * listen to a path that does not currently exist in the View.
   *
   * @param pathToListenTo can be null to listen to all paths.
   */
  public abstract void addViewListener(@Nullable Path pathToListenTo, ViewListener listener);
  public abstract void removeViewListener(ViewListener listener);
  // TODO[cc]: Consider getting rid of this.
  public abstract ViewDefaults getDefaults();

  public <T> T getOrDefault(final ViewProperty<T> property, final Path path) {
    T ret = getOrNull(property, path);
    return (ret == null) ? getDefaults().get(property) : ret;
  }
}
