// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import com.sieuferd.util.MapUtil;
import java.util.Iterator;
import java.util.Set;
import javax.annotation.concurrent.Immutable;

/** Paths in order; no path may be present more than once or be null. */
@Immutable
public final class PathList implements Iterable<Path> {
  private final ImmutableList<Path> paths;
  private final int hashCode;

  public PathList(Iterable<Path> paths) {
    this.paths = ImmutableList.copyOf(paths);
    Preconditions.checkArgument(
        this.paths.equals(ImmutableList.copyOf(ImmutableSet.copyOf(this.paths))),
        "paths may not contain duplicate items");
    this.hashCode = this.paths.hashCode();
  }

  // TODO[pd]: Make this operation constant-time by making PathList a persistent data structure.
  public PathList createAdd(Path path) {
    Set<Path> newPaths = Sets.newLinkedHashSet(paths);
    MapUtil.addChecked(newPaths, path);
    return new PathList(newPaths);
  }

  public PathList createRemove(Path path) {
    Set<Path> newPaths = Sets.newLinkedHashSet(paths);
    MapUtil.removeChecked(newPaths, path);
    return new PathList(newPaths);
  }

  public boolean isEmpty() {
    return paths.isEmpty();
  }

  public Path get(int index) {
    return paths.get(index);
  }

  public int size() {
    return paths.size();
  }

  public boolean equalsUnordered(PathList other) {
    return ImmutableSet.copyOf(paths).equals(ImmutableSet.copyOf(other.paths));
  }

  @Override
  public int hashCode() {
    return hashCode;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof PathList))
      return false;
    PathList other = (PathList) obj;
    return (
        this.hashCode == other.hashCode &&
        this.paths.equals(other.paths));
  }

  @Override
  public Iterator<Path> iterator() {
    return paths.iterator();
  }

  @Override
  public String toString() {
    return paths.toString();
  }
}
