// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableBiMap;
import com.google.common.collect.ImmutableMap;
import java.util.Map;
import java.util.Map.Entry;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class PathMap {
  public static final PathMap EMPTY = new PathMap(ImmutableMap.of());

  private final ImmutableBiMap<Path,Path> map;

  /** @param map keys are old paths, values are new paths */
  public <P extends Path> PathMap(Map<P,P> map) {
    this.map = ImmutableBiMap.<Path,Path>copyOf(map);
    for (Entry<Path,Path> entry : this.map.entrySet())
      Preconditions.checkArgument(entry.getKey().isRelation() == entry.getValue().isRelation());
  }

  @SuppressWarnings("unchecked")
  public @Nullable <P extends Path> P getNewPathIfMapped(P oldPath) {
    Preconditions.checkNotNull(oldPath);
    return (P) map.get(oldPath);
  }

  @SuppressWarnings("unchecked")
  public @Nullable <P extends Path> P getOldPathIfMapped(P newPath) {
    Preconditions.checkNotNull(newPath);
    return (P) map.inverse().get(newPath);
  }

  public <P extends Path> P getNewPath(P oldPath) {
    P ret = getNewPathIfMapped(oldPath);
    return ret == null ? oldPath : ret;
  }

  public <P extends Path> P getOldPath(P newPath) {
    P ret = getOldPathIfMapped(newPath);
    return ret == null ? newPath : ret;
  }
}
