// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

public final class PrettyPrinter {
  private PrettyPrinter() { }

  // TODO[cc]: Consider taking a ViewResult instead of separate View and Data objects.
  private static void prettyPrintInternal(
      View view, Data data, StringBuilder ret, Path path, Tuple parentTuple, String indentation)
  {
    if (!data.isProjected(path)) {
      ret.append("NP");
      return;
    }
    if (path.isRelation()) {
      if (!view.get(CProp.PLURAL, path.getRelation()))
        ret.append("!");
      ret.append("[\n");
      final String childIndentation = "  ";
      TupleInterval childTuples = data.getRelationTuples(path.getRelation(), parentTuple);
      for (Tuple childTuple : childTuples) {
        boolean first = true;
        ret.append(indentation).append(childIndentation + "(");

        for (Path childPath : view.getChildPaths(path.getRelation())) {
          if (!first) {
            ret.append(", ");
          } else {
            first = false;
          }
          ret.append(view.get(CProp.LABEL, childPath));
          ret.append("=");
          prettyPrintInternal(
              view, data, ret, childPath, childTuple, indentation + childIndentation);
        }
        ret.append(")\n");
      }
      if (!childTuples.isComplete())
        ret.append(indentation).append(childIndentation + "...\n");
      ret.append(indentation).append("]");
    } else {
      ret.append(data.getTupleField(parentTuple, path.getPrimitive()).toString());
    }
  }

  public static String prettyPrint(View view, Data data) {
    StringBuilder ret = new StringBuilder();
    prettyPrintInternal(view, data, ret, Path.ROOT, Tuple.createRootParent(data), "");
    return ret.toString();
  }
}
