// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.sieuferd.util.StringUtil;
import java.util.Objects;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class PrimitiveValue {
  private final @Nullable Object content;

  /** @param content should be immutable if possible, and may not be subsequently modified by the
             client */
  public PrimitiveValue(@Nullable Object content) {
    this.content = content;
  }

  /**
   * Get the underlying value object. Clients are forbidden from modifying any mutable objects that
   * may be returned by this method.
   */
  public @Nullable Object getContent() {
    return content;
  }

  public boolean isNull() {
    return content == null;
  }

  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof PrimitiveValue))
      return false;
    if (obj == this)
      return true;
    PrimitiveValue rhs = (PrimitiveValue) obj;
    /* Doing simple object equality here is appropriate for the short-term purposes of matching rows
    between different queries in the same transaction. This way, more data types can be supported
    than would be possible if using an explicit canonicalization routine like that in
    ConstantExpression.fromObject(). The exact kinds of objects involved may be tailored for
    specific database backends through BackendDialect.fetchResultValue. */
    return Objects.equals(getContent(), rhs.getContent());
  }

  @Override
  public int hashCode() {
    return Objects.hashCode(getContent());
  }

  @Override
  public String toString() {
    if (content == null) {
      return "null";
    } else if (content instanceof String) {
      return StringUtil.debugQuote((String) content);
    } else {
      return content.toString();
    }
  }
}
