// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.google.common.base.Preconditions;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

// TODO[cc]: Consider moving this out of the datamodel module.
// TODO[cc]: Consider renaming.
/** A limit defined for a specific query. */
@Immutable
public final class ResultLimit {
  public static final ResultLimit UNDEFINED = new ResultLimit(null, 0);

  // TODO[pd]: Consider using a weak reference.
  private final @Nullable ImmutableView querySnapshot;
  private final int limit;

  private ResultLimit(@Nullable ImmutableView querySnapshot, int limit) {
    this.limit = limit;
    this.querySnapshot = querySnapshot;
  }

  public static ResultLimit create(ImmutableView querySnapshot, int limit) {
    Preconditions.checkNotNull(querySnapshot);
    return new ResultLimit(querySnapshot, limit);
  }

  public boolean appliesTo(ImmutableView querySnapshot) {
    Preconditions.checkNotNull(querySnapshot);
    return this.querySnapshot == querySnapshot;
  }

  public int getLimit() {
    return limit;
  }

  public static ResultLimit getIncreasedLimit(ResultLimit previousLimit, ResultLimit displayedLimit,
      double increaseFactor, int minimumLimit)
  {
    Preconditions.checkNotNull(previousLimit);
    Preconditions.checkNotNull(displayedLimit);
    Preconditions.checkArgument(increaseFactor > 1.0);

    final ImmutableView displayedQS = displayedLimit.querySnapshot;
    if (displayedQS == null)
      return previousLimit;

    final int newLimitCount = Math.max(minimumLimit, Math.max(displayedLimit.getLimit() + 1,
        (int) Math.min(Integer.MAX_VALUE,
        Math.round(displayedLimit.getLimit() * increaseFactor))));
    if (!previousLimit.appliesTo(displayedQS) || newLimitCount > previousLimit.getLimit()) {
      return ResultLimit.create(displayedQS, newLimitCount);
    } else {
      return previousLimit;
    }
  }
}
