// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.google.common.base.Preconditions;

/** Support for a simplified tree traversal idiom which is used instead of the visitor pattern. */
public abstract class TreeStructure<T extends TreeStructure<T,R,P>,R extends T,P extends T> {
  protected abstract Class<R> getRelationClass();
  protected abstract Class<P> getPrimitiveClass();

  @SuppressWarnings("OverridableMethodCallInConstructor")
  protected TreeStructure() {
    Preconditions.checkNotNull(getRelationClass());
    Preconditions.checkNotNull(getPrimitiveClass());
    if (getRelationClass() == getPrimitiveClass())
      throw new IllegalArgumentException();
    final Class<?> thisClass = getClass();
    if (thisClass != getRelationClass() && thisClass != getPrimitiveClass())
      throw new IllegalArgumentException();
  }

  public final boolean isRelation() {
    return getClass() == getRelationClass();
  }

  @SuppressWarnings("unchecked")
  public final R getRelation() throws NodeTypeException {
    if (!isRelation())
      throw new NodeTypeException("Not a relation");
    return (R) this;
  }

  @SuppressWarnings("unchecked")
  public final P getPrimitive() throws NodeTypeException {
    if (isRelation())
      throw new NodeTypeException("Not a primitive");
    return (P) this;
  }
}
