// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.sieuferd.datamodel.Path.RelationPath;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;

public abstract class View {
  public abstract boolean exists(Path path);
  public abstract @Nonnull <T> T get(ViewProperty<T> property, Path path);
  public abstract Set<ViewProperty<?>> getDefinedProperties(Path path);
  // TODO[cc]: Get rid of uses of this.
  public abstract boolean defines(ViewProperty<?> property, Path path);
  /** Run the supplied Runnable synchonously with read-only transactional access to the View. The
  View is guaranteed not to change during the transaction, and implementations permitting
  modifications to the View must not permit them from within a read-only transaction. During the
  transaction, a lock may be held on the View, so in order to avoid deadlock, the client's
  transaction logic should avoid waiting for other threads that may simultaneously access the View.
  The {@code Runnable} is guaranteed to be run on the same thread as the caller. Transactions are
  reentrant. Behavior is undefined if any exceptions are thrown from the {@code Runnable}. */
  public abstract void performReadTransaction(Runnable runnable);

  // TODO[cc]: Consider having a getRelationChildPaths method as a shorthand.
  public final PathList getChildPaths(RelationPath parent) {
    return get(SProp.CHILD_PATHS, parent);
  }

  public final void checkPathExists(final Path path) {
    if (!exists(path))
      throw new IllegalArgumentException("Non-existent path " + path);
  }

  public @Nullable <T> T getOrNull(final ViewProperty<T> property, final Path path) {
    final List<T> ret = new ArrayList<>(1);
    performReadTransaction(() -> {
      if (exists(path) && defines(property, path))
        ret.add(get(property, path));
    });
    return ret.isEmpty() ? null : ret.get(0);
  }

  public final ViewPropertySource getViewPropertySource(final Path path) {
    checkPathExists(path);
    return new ViewPropertySource() {
      @Override
      public <T> T get(ViewProperty<T> property) {
        return View.this.get(property, path);
      }
    };
  }
}
