// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableSet;
import com.sieuferd.datamodel.Path.RelationPath;
import com.sieuferd.flow.ModelAdapterFlow;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;
import javax.annotation.Nullable;

public final class ViewFlow extends ModelAdapterFlow<ImmutableView> {
  private final ObservableView view;
  private final @Nullable ImmutableSet<ViewProperty<?>> properties;
  private final @Nullable ImmutableSet<Path> paths;
  private final @Nullable ImmutableSet<RelationPath> strictAncestorPaths;

  private final ViewListener viewListener = new ViewListener() {
    private final AtomicBoolean markAtTransactionCompleted = new AtomicBoolean(false);

    @Override
    public <T> void propertyChanged(ViewPropertyChangedEvent<T> evt) {
      final boolean applicablePropertyChange =
          (paths == null || paths.contains(evt.getPath())) &&
          (properties == null || properties.contains(evt.getProperty()));
      final boolean applicableStructuralChange = (evt.getProperty() == SProp.CHILD_PATHS) &&
          (strictAncestorPaths == null ||
          strictAncestorPaths.contains(evt.getPath().getRelation()));
      if (applicablePropertyChange || applicableStructuralChange) {
        /* Don't permit out-of-date views to be exposed alongside updated ones based on the same
        underlying MutableView. */
        setInternalIfDifferent(null);
        markAtTransactionCompleted.set(true);
      }
    }

    @Override
    public void transactionCompleted(ViewTransactionCompletedEvent evt) {
      if (markAtTransactionCompleted.getAndSet(false))
        markModelUpdated();
    }
  };

  public ViewFlow(
      ObservableView view, @Nullable Set<ViewProperty<?>> properties, @Nullable Set<Path> paths)
  {
    super("View");
    Preconditions.checkNotNull(view);
    this.view = view;
    this.properties = (properties == null) ? null : ImmutableSet.copyOf(properties);
    this.paths = (paths == null) ? null : ImmutableSet.copyOf(paths);
    strictAncestorPaths = (paths == null) ? null :
        ImmutableSet.copyOf(Views.getStrictAncestors(paths));
  }

  @Override
  protected void addModelListener() {
    view.addViewListener(null, viewListener);
  }

  @Override
  protected void removeModelListener() {
    view.removeViewListener(viewListener);
  }

  @Override
  protected ImmutableView getModelValue() {
    return ImmutableView.copyOf(view, properties, paths);
  }
}
