// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.google.common.base.Preconditions;
import java.util.EventObject;
import javax.annotation.Nullable;

/** Shallowly immutable. */
public final class ViewPropertyChangedEvent<T> extends EventObject {
  private static final long serialVersionUID = 1L;
  private final ViewProperty<T> property;
  private final Path path;
  private final @Nullable T oldValue;
  private final @Nullable T newValue;

  /** @param oldValue is null iff the property was previously undefined */
  public ViewPropertyChangedEvent(
      ObservableView source, ViewProperty<T> property, Path path, @Nullable T oldValue, T newValue)
  {
    super(source);
    Preconditions.checkNotNull(source);
    Preconditions.checkNotNull(property);
    Preconditions.checkNotNull(path);
    Preconditions.checkNotNull(newValue);
    this.source = source;
    this.property = property;
    this.path = path;
    this.oldValue = oldValue;
    this.newValue = newValue;
  }

  @Override
  public ObservableView getSource() {
    return (ObservableView) super.getSource();
  }

  public ViewProperty<T> getProperty() {
    return property;
  }

  public Path getPath() {
    return path;
  }

  public @Nullable T getOldValue() {
    return oldValue;
  }

  public T getNewValue() {
    return newValue;
  }

  @Override
  public String toString() {
    return path.toString(getSource()) + ": " + getProperty() + " = " + getNewValue();
  }
}
