// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.google.common.base.Preconditions;
import javax.annotation.Nullable;

public abstract class ViewPropertyType<T> {
  private final Class<T> valueClass;

  public ViewPropertyType(Class<T> valueClass) {
    Preconditions.checkNotNull(valueClass);
    this.valueClass = valueClass;
  }

  public final Class<T> getValueClass() {
    return valueClass;
  }

  @Override
  public String toString() {
    return valueClass.getName();
  }

  /** @return null indicates a value that cannot be represented by a string */
  public @Nullable String valueToString(Strepco strepco, T value) {
    return null;
  }

  // TODO[cc]: Consider allowing null to be returned instead.
  /**
   * @throws IllegalArgumentException to indicate a malformed string
   * @throws UnsupportedOperationException if the operation is not supported
   */
  public T stringToValue(Strepco strepco, String str) throws IllegalArgumentException {
    throw new UnsupportedOperationException();
  }

  public T rewritePathReferences(PathMap pathMap, T input) {
    return input;
  }
}
