// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Multimap;
import com.sieuferd.datamodel.proptypes.Warning;
import javax.annotation.concurrent.Immutable;

@Immutable
public class ViewResult {
  private final ResultLimit limit;
  private final ImmutableView view;
  private final ImmutableData data;
  private final ImmutableMultimap<Path,Warning> warnings;

  public ViewResult(ResultLimit limit, ImmutableView view, ImmutableData data,
      Multimap<Path,Warning> warnings)
  {
    Preconditions.checkNotNull(limit);
    Preconditions.checkNotNull(view);
    Preconditions.checkNotNull(data);
    Preconditions.checkNotNull(warnings);
    this.limit = limit;
    this.view = view;
    this.data = data;
    this.warnings = ImmutableMultimap.copyOf(warnings);
  }

  public ResultLimit getLimit() {
    return limit;
  }

  public ImmutableView getView() {
    return view;
  }

  public ImmutableData getData() {
    return data;
  }

  public ImmutableMultimap<Path, Warning> getWarnings() {
    return warnings;
  }
}
