// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel.expressions;

import com.sieuferd.datamodel.expressions.parser.ANTLR_ExpressionLexer;
import javax.annotation.Nullable;

/* Make sure to update usage sites when adding items to the list. The syntax highlighting system in
particular has a unit test in place to check this (ExpressionHighlightingTest). */
public enum ExpressionTokenType {
  IDENTIFIER(ANTLR_ExpressionLexer.Identifier),
  PATH_REFERENCE(ANTLR_ExpressionLexer.PathReference),
  ARGUMENT_SEPARATOR(ANTLR_ExpressionLexer.SEP_COMMA),
  PARENTHESIS_LEFT(ANTLR_ExpressionLexer.SEP_LPAREN),
  PARENTHESIS_RIGHT(ANTLR_ExpressionLexer.SEP_RPAREN),
  LITERAL_NUMBER(ANTLR_ExpressionLexer.NumberLiteral),
  LITERAL_STRING(ANTLR_ExpressionLexer.StringLiteral),
  LITERAL_TEMPORAL(ANTLR_ExpressionLexer.TemporalLiteral),
  LITERAL_BOOLEAN(ANTLR_ExpressionLexer.BooleanLiteral),
  LITERAL_ERROR(ANTLR_ExpressionLexer.ErrorLiteral),
  LITERAL_NULL(ANTLR_ExpressionLexer.NullLiteral),
  OPERATOR(null),
  WHITESPACE(ANTLR_ExpressionLexer.WS),
  UNTERMINATED_PATH_REFERENCE(ANTLR_ExpressionLexer.UnterminatedPathReference),
  UNTERMINATED_LITERAL_STRING(ANTLR_ExpressionLexer.UnterminatedStringLiteral),
  UNTERMINATED_LITERAL_TEMPORAL(ANTLR_ExpressionLexer.UnterminatedTemporalLiteral),
  OTHER(null);

  private final @Nullable Integer antlrTokenType;

  /** @param antlrTokenType may be null if there is more than one corresponding ANTLR token */
  private ExpressionTokenType(@Nullable Integer antlrTokenType) {
    this.antlrTokenType = antlrTokenType;
  }

  @Nullable Integer getANTLRTokenType() {
    return antlrTokenType;
  }
}
