// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel.expressions;

import com.sieuferd.datamodel.expressions.Expression.FunctionExpression;
import com.sieuferd.datamodel.expressions.Expression.ConstantExpression;
import com.sieuferd.datamodel.expressions.Expression.PathExpression;
import com.sieuferd.util.UnitObject;
import javax.annotation.Nonnull;

public interface ExpressionVisitor<R,E extends Exception> {
  public @Nonnull R visit(ConstantExpression exp) throws E;
  public @Nonnull R visit(FunctionExpression exp) throws E;
  public @Nonnull R visit(PathExpression exp) throws E;

  public abstract class VoidExpressionVisitor<E extends Exception>
      implements ExpressionVisitor<UnitObject,E>
  {
    public abstract void voidVisit(ConstantExpression exp) throws E;
    public abstract void voidVisit(FunctionExpression exp) throws E;
    public abstract void voidVisit(PathExpression exp) throws E;

    @Override
    public final UnitObject visit(ConstantExpression exp) throws E {
      voidVisit(exp);
      return UnitObject.INSTANCE;
    }

    @Override
    public final UnitObject visit(FunctionExpression exp) throws E {
      voidVisit(exp);
      return UnitObject.INSTANCE;
    }

    @Override
    public final UnitObject visit(PathExpression exp) throws E {
      voidVisit(exp);
      return UnitObject.INSTANCE;
    }
  }

  public abstract class ExpressionVisitorAdapter<E extends Exception>
      extends VoidExpressionVisitor<E>
  {
    @Override
    public void voidVisit(ConstantExpression exp) throws E { }

    @Override
    public void voidVisit(FunctionExpression exp) throws E {
      for (Expression argumentExp : exp.getArguments())
        argumentExp.accept(this);
    }

    @Override
    public void voidVisit(PathExpression exp) throws E { }
  }
}
