// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel.expressions;

import com.sieuferd.datamodel.Strepco;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.Path.PrimitivePath;
import com.sieuferd.datamodel.Path.RelationPath;
import com.sieuferd.datamodel.View;
import com.sieuferd.datamodel.expressions.PathReference.Segment;
import com.sieuferd.datamodel.expressions.PathReference.Segment.LabelSegment;
import com.sieuferd.datamodel.expressions.PathReference.Segment.OrdinalSegment;

public abstract class PathReferenceException extends Exception {
  PathReferenceException(View view, Path path, Object identifier) {
    super(identifier + " at path " + path.toString(view));
  }

  PathReferenceException(Strepco strepco, Object identifier) {
    this(strepco.getView(), strepco.getPath(), identifier);
  }

  static final class LabelNotInScopeException extends PathReferenceException {
    LabelNotInScopeException(Strepco strepco, String label) {
      super(strepco, label);
    }
  }

  static final class PrimitivePathDereferencedException extends PathReferenceException {
    PrimitivePathDereferencedException(View view, PrimitivePath path, Segment segment) {
      super(view, path, segment);
    }
  };

  static final class LabelNotAtPathException extends PathReferenceException {
    LabelNotAtPathException(View view, RelationPath path, LabelSegment segment) {
      super(view, path, segment);
    }
  }

  static final class LabelNotUniqueException extends PathReferenceException {
    LabelNotUniqueException(View view, RelationPath path, LabelSegment segment) {
      super(view, path, segment);
    }
  }

  static final class OrdinalOutOfRangeException extends PathReferenceException {
    OrdinalOutOfRangeException(View view, RelationPath path, OrdinalSegment segment) {
      super(view, path, segment);
    }
  }
}
