// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel.expressions;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableMap;
import com.sieuferd.datamodel.Path.PrimitivePath;
import com.sieuferd.datamodel.expressions.Expression.ConstantExpression;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class TupleConstant {
  public static final TupleConstant UNIT = new TupleConstant(ImmutableMap.of());

  // TODO[pd]: Consider storing this in a more compact fashion.
  private final ImmutableMap<PrimitivePath,ConstantExpression> map;

  public TupleConstant(Map<PrimitivePath,ConstantExpression> map) {
    this.map = ImmutableMap.copyOf(map);
  }

  public ConstantExpression getValue(PrimitivePath path) {
    Preconditions.checkNotNull(path);
    ConstantExpression ret = map.get(path);
    if (ret == null)
      throw new IllegalArgumentException("Path does not exist in tuple");
    return ret;
  }

  public void checkSamePathSet(Set<PrimitivePath> paths) {
    Preconditions.checkArgument(map.keySet().equals(paths));
  }

  @Override
  public String toString() {
    return "TupleConstant(" + map.toString() + ")";
  }

  @Override
  public int hashCode() {
    return map.hashCode();
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof TupleConstant))
      return false;
    TupleConstant other = (TupleConstant) obj;
    return Objects.equals(this.map, other.map);
  }
}
