// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel.internal;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Maps;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.PathList;
import com.sieuferd.datamodel.SProp;
import com.sieuferd.datamodel.ViewProperty;
import java.util.Map;
import javax.annotation.Nullable;
import javax.annotation.concurrent.NotThreadSafe;

@NotThreadSafe
class ViewBranch {
  private final Map<ViewProperty<?>,Object> properties = Maps.newLinkedHashMap();

  ViewBranch(boolean relation) {
    if (relation)
      properties.put(SProp.CHILD_PATHS, new PathList(ImmutableSet.of()));
  }

  /** @return null if the property does not exist. */
  @SuppressWarnings("unchecked")
  public @Nullable <T> T getProperty(ViewProperty<T> property) {
    return (T) properties.get(property);
  }

  public boolean definesProperty(ViewProperty<?> property) {
    return properties.containsKey(property);
  }

  public <T> void setProperty(ViewProperty<T> property, T value) {
    Preconditions.checkNotNull(value);
    properties.put(property, value);
  }

  public ImmutableSet<ViewProperty<?>> getDefinedProperties() {
    return ImmutableSet.copyOf(properties.keySet());
  }

  public void addChildPath(Path path) {
    Preconditions.checkNotNull(path);
    setProperty(SProp.CHILD_PATHS, getProperty(SProp.CHILD_PATHS).createAdd(path));
  }

  public void removeChildPath(Path path) {
    Preconditions.checkNotNull(path);
    setProperty(SProp.CHILD_PATHS, getProperty(SProp.CHILD_PATHS).createRemove(path));
  }
}
