// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel.proptypes;

import com.google.common.base.Preconditions;
import java.math.BigDecimal;
import java.util.Objects;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class OptionalBigDecimal {
  private static final OptionalBigDecimal ABSENT = new OptionalBigDecimal(null);
  private final @Nullable BigDecimal value;

  public static OptionalBigDecimal absent() {
    return ABSENT;
  }

  public BigDecimal max(BigDecimal other) {
    Preconditions.checkNotNull(other);
    return value == null ? other : value.max(other);
  }

  public BigDecimal min(BigDecimal other) {
    Preconditions.checkNotNull(other);
    return value == null ? other : value.min(other);
  }

  public OptionalBigDecimal max(OptionalBigDecimal other) {
    Preconditions.checkNotNull(other);
    return value == null ? other : OptionalBigDecimal.of(other.max(value));
  }

  public OptionalBigDecimal min(OptionalBigDecimal other) {
    Preconditions.checkNotNull(other);
    return value == null ? other : OptionalBigDecimal.of(other.min(value));
  }

  public static OptionalBigDecimal of(BigDecimal value) {
    Preconditions.checkNotNull(value);
    return new OptionalBigDecimal(value);
  }

  public static OptionalBigDecimal ofNullable(@Nullable BigDecimal value) {
    return value == null ? absent() : of(value);
  }

  private OptionalBigDecimal(@Nullable BigDecimal value) {
    this.value = value;
  }

  public boolean isPresent() {
    return value != null;
  }

  public BigDecimal get() {
    if (value == null)
      throw new IllegalStateException("Optional BigDecimal not set");
    return value;
  }

  @Override
  public String toString() {
    return value == null ? "" : Objects.toString(value);
  }

  @Override
  public int hashCode() {
    return value == null ? Integer.MIN_VALUE : value.hashCode();
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof OptionalBigDecimal))
      return false;
    OptionalBigDecimal other = (OptionalBigDecimal) obj;
    return Objects.equals(other.value, this.value);
  }
}
