// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel.proptypes;

import com.google.common.base.Preconditions;
import java.util.Objects;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

/* TODO[cc]: Consider making a common superclass for optional view properties, to get rid of all
             this boilerplate. It might even be possible to make use of existing property editors in
             a general way. */
// TODO[cc]: Consider using java.util.OptionalInt instead.
@Immutable
public final class OptionalInteger {
  private static final OptionalInteger ABSENT = new OptionalInteger(null);
  private final @Nullable Integer value;

  public static OptionalInteger absent() {
    return ABSENT;
  }

  public static OptionalInteger of(int value) {
    Preconditions.checkNotNull(value);
    return new OptionalInteger(value);
  }

  private OptionalInteger(@Nullable Integer value) {
    this.value = value;
  }

  public boolean isPresent() {
    return value != null;
  }

  public int get() {
    if (value == null)
      throw new IllegalStateException("Optional integer not set");
    return value;
  }

  @Override
  public String toString() {
    return value == null ? "" : Objects.toString(value);
  }

  @Override
  public int hashCode() {
    return value == null ? Integer.MIN_VALUE : value;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof OptionalInteger))
      return false;
    OptionalInteger other = (OptionalInteger) obj;
    return Objects.equals(other.value, this.value);
  }
}
