// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.datamodel.proptypes;

import java.math.BigDecimal;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

// TODO[cc]: Consider just using OptionalInteger.
/* TODO[cc]: This is yet another type of the "Optional" kind. Eliminate duplication with
             OptionalInteger and OptionalPrimitivePath. */
@Immutable
public class Position implements Comparable<Position> {
  public static final Position NONE = new Position(null);
  /** Null to indicate not included or at the end of list. */
  private final @Nullable BigDecimal position;

  private Position(@Nullable BigDecimal position) {
    this.position = position;
  }

  public static Position create(@Nullable BigDecimal position) {
    if (position == null)
      return NONE;
    return new Position(position);
  }

  public @Nullable BigDecimal getPosition() {
    return position;
  }

  @Override
  public int compareTo(Position other) {
    if (getPosition() == null || other.getPosition() == null)
      throw new NullPointerException("Don't compare Position objects with null positions");
    return getPosition().compareTo(other.getPosition());
  }

  @Override
  public String toString() {
    return position == null ? "(none)" : position.toString();
  }
}
