// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.layout;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableSet;
import com.sieuferd.datamodel.Path.PrimitivePath;
import com.sieuferd.datamodel.expressions.Expression;
import com.sieuferd.datamodel.expressions.RelationConstant;
import com.sieuferd.datamodel.expressions.TupleConstant;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import javax.annotation.concurrent.Immutable;

/**
 * An ordered set of constant tuples to identify crosstab columns. Tuple order is significant for
 * comparison with {@code equals}, though path order is not.
 */
@Immutable
public final class CrosstabColumns {
  public static final CrosstabColumns UNIT = new CrosstabColumns(new RelationConstant(
      ImmutableSet.of(), ImmutableSet.of(TupleConstant.UNIT)));
  private final RelationConstant relationConstant;
  private final ImmutableList<TupleConstant> tuples;

  public CrosstabColumns(RelationConstant relationConstant) {
    Preconditions.checkNotNull(relationConstant);
    this.relationConstant = relationConstant;
    this.tuples = ImmutableList.copyOf(relationConstant.getTuples());
  }

  public Expression toExpression() {
    return relationConstant.toExpression();
  }

  public static CrosstabColumns fromExpression(Expression exp) throws IllegalArgumentException {
    return new CrosstabColumns(RelationConstant.fromExpression(exp));
  }

  /** @return will be empty if the filter contains no tuples */
  public Set<PrimitivePath> getPaths() {
    return relationConstant.getPaths();
  }

  public List<TupleConstant> getTuples() {
    return tuples;
  }

  @Override
  public int hashCode() {
    return Objects.hash(getPaths().hashCode(), getTuples().hashCode());
  }

  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof CrosstabColumns))
      return false;
    if (obj == this)
      return true;
    CrosstabColumns other = (CrosstabColumns) obj;
    return this.getPaths().equals(other.getPaths())
        && this.getTuples().equals(other.getTuples());
  }
}
