// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.layout;

import com.sieuferd.datamodel.ViewDefaults;
import com.sieuferd.datamodel.ViewProperty;
import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import com.sieuferd.datamodel.ViewPropertySet;
import com.sieuferd.datamodel.ViewPropertyType;
import org.openide.util.NbBundle;

public final class LAProp extends ViewPropertySet {
  public static final LAProp INSTANCE = new LAProp();

  @NbBundle.Messages("TIT_LayoutAuto=Auto-Styleables")
  private LAProp() {
    super("sieuferd_layout_auto", Bundle.TIT_LayoutAuto(), null, 440);
  }

  private static <T> ViewProperty<T> create(
      String storageName, AppliesTo appliesTo, ViewPropertyType<T> type)
  {
    return addProperty(INSTANCE, storageName, appliesTo, type, false);
  }

  @NbBundle.Messages("TIT_ViewProperty.OUTLINE_COLUMN_EXCLUDE=Exclude from Outline Columns")
  public static final ViewProperty<YesNoAuto> OUTLINE_COLUMN_EXCLUDE = create("OUTLINE_COLUMN_EXCLUDE", AppliesTo.ANY, LayoutPropertyTypes.YES_NO_AUTO);
  @NbBundle.Messages("TIT_ViewProperty.LABEL_VISIBLE=Label Visible")
  public static final ViewProperty<YesNoAuto> LABEL_VISIBLE = create("LABEL_VISIBLE", AppliesTo.ANY, LayoutPropertyTypes.YES_NO_AUTO);
  @NbBundle.Messages("TIT_ViewProperty.START_NEW_COLUMN=Start New Column")
  public static final ViewProperty<YesNoAuto> START_NEW_COLUMN = create("START_NEW_COLUMN", AppliesTo.ANY, LayoutPropertyTypes.YES_NO_AUTO);
  @NbBundle.Messages("TIT_ViewProperty.START_NEW_COLUMN_SET=Start New Column Set")
  public static final ViewProperty<YesNoAuto> START_NEW_COLUMN_SET = create("START_NEW_COLUMN_SET", AppliesTo.ANY, LayoutPropertyTypes.YES_NO_AUTO);
  @NbBundle.Messages("TIT_ViewProperty.ALTERNATE_ROW_COLORS=Alternating Row Colors")
  public static final ViewProperty<YesNoAuto> ALTERNATE_ROW_COLORS = create("ALTERNATE_ROW_COLORS", AppliesTo.RELATION_ONLY, LayoutPropertyTypes.YES_NO_AUTO);
  @NbBundle.Messages("TIT_ViewProperty.USE_TABLE=Tabular Layout")
  public static final ViewProperty<YesNoAuto> USE_TABLE = create("USE_TABLE", AppliesTo.RELATION_ONLY, LayoutPropertyTypes.YES_NO_AUTO);
  @NbBundle.Messages("TIT_ViewProperty.USE_VERTICAL_TABLE_HEADER=Vertical Table Header")
  public static final ViewProperty<YesNoAuto> USE_VERTICAL_TABLE_HEADER = create("USE_VERTICAL_TABLE_HEADER", AppliesTo.ANY, LayoutPropertyTypes.YES_NO_AUTO);

  public static final ViewDefaults DEFAULTS = ViewDefaults.builder()
      .set(OUTLINE_COLUMN_EXCLUDE, YesNoAuto.AUTO)
      .set(LABEL_VISIBLE, YesNoAuto.AUTO)
      .set(START_NEW_COLUMN, YesNoAuto.AUTO)
      .set(START_NEW_COLUMN_SET, YesNoAuto.AUTO)
      .set(ALTERNATE_ROW_COLORS, YesNoAuto.AUTO)
      .set(USE_TABLE, YesNoAuto.AUTO)
      .set(USE_VERTICAL_TABLE_HEADER, YesNoAuto.AUTO)
      .build();
}
