// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.layout;

import com.sieuferd.cell.styles.HorizontalAlignment;
import com.sieuferd.cell.styles.LineStyle;
import com.sieuferd.cell.support.Points;
import com.sieuferd.datamodel.ViewProperty;
import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import com.sieuferd.datamodel.ViewPropertySet;
import com.sieuferd.datamodel.ViewPropertyType;
import com.sieuferd.datamodel.ViewPropertyTypes;
import java.awt.Color;
import java.awt.Font;
import org.openide.util.NbBundle;

public final class LBProp extends ViewPropertySet {
  public static final LBProp INSTANCE = new LBProp();

  @NbBundle.Messages("TIT_LayoutBasic=Styling Constants")
  private LBProp() {
    super("sieuferd_layout_basic", Bundle.TIT_LayoutBasic(), null, 410);
  }

  private static <T> ViewProperty<T> create(
      String storageName, AppliesTo appliesTo, ViewPropertyType<T> type)
  {
    return addProperty(INSTANCE, storageName, appliesTo, type, false);
  }

  private static final ViewPropertyType<Color> COLOR_TYPE = ViewPropertyTypes.createNonpersistableType(Color.class);
  private static final ViewPropertyType<Font> FONT_TYPE = ViewPropertyTypes.createNonpersistableType(Font.class);
  private static final ViewPropertyType<LineStyle> LINE_STYLE_TYPE = ViewPropertyTypes.createNonpersistableType(LineStyle.class);

  @NbBundle.Messages("TIT_ViewProperty.OUTLINE_COLUMNS=Outline Columns")
  public static final ViewProperty<Boolean> OUTLINE_COLUMNS = create("OUTLINE_COLUMNS", AppliesTo.RELATION_ONLY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.TABLE_JUSTIFY=Justify Table")
  public static final ViewProperty<Boolean> TABLE_JUSTIFY = create("TABLE_JUSTIFY", AppliesTo.RELATION_ONLY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.INTERNAL_TABLE_HEADER=Internal Table Header")
  public static final ViewProperty<Boolean> INTERNAL_TABLE_HEADER = create("INTERNAL_TABLE_HEADER", AppliesTo.RELATION_ONLY, ViewPropertyTypes.BOOLEAN);

  @NbBundle.Messages("TIT_ViewProperty.LABEL_TEXT_COLOR=Label Text Color")
  public static final ViewProperty<Color> LABEL_TEXT_COLOR = create("LABEL_TEXT_COLOR", AppliesTo.ANY, COLOR_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.PRIMITIVE_LABEL_FONT=Primitive Label Font")
  public static final ViewProperty<Font> PRIMITIVE_LABEL_FONT = create("PRIMITIVE_LABEL_FONT", AppliesTo.PRIMITIVE_ONLY, FONT_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.RELATION_LABEL_FONT=Relation Label Font")
  public static final ViewProperty<Font> RELATION_LABEL_FONT = create("RELATION_LABEL_FONT", AppliesTo.RELATION_ONLY, FONT_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.CROSSTAB_HEADING_FONT=Crosstab Heading Font")
  public static final ViewProperty<Font> CROSSTAB_HEADING_FONT = create("CROSSTAB_HEADING_FONT", AppliesTo.ANY, FONT_TYPE);

  @NbBundle.Messages("TIT_ViewProperty.VALUE_TEXT_COLOR=Value Text Color")
  public static final ViewProperty<Color> VALUE_TEXT_COLOR = create("VALUE_TEXT_COLOR", AppliesTo.PRIMITIVE_ONLY, COLOR_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.VALUE_FONT=Value Text Font")
  public static final ViewProperty<Font> VALUE_FONT = create("VALUE_FONT", AppliesTo.PRIMITIVE_ONLY, FONT_TYPE);
  /* This is regarded as a constant styling property rather than a measured property. For numerical
  formatting, LMProp.DECIMAL_PLACES is automatically set and right alignment is forced, effectively
  overriding whatever is set in LBProp.HORIZONTAL_ALIGNMENT. */
  @NbBundle.Messages("TIT_ViewProperty.HORIZONTAL_ALIGNMENT=Horizontal Alignment")
  public static final ViewProperty<HorizontalAlignment> HORIZONTAL_ALIGNMENT = create("HORIZONTAL_ALIGNMENT", AppliesTo.ANY, LayoutPropertyTypes.HORIZONTAL_ALIGNMENT);
  @NbBundle.Messages("TIT_ViewProperty.VALUE_JUSTIFIED=Justify Value")
  public static final ViewProperty<Boolean> VALUE_JUSTIFIED = create("VALUE_JUSTIFIED", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.BOOLEAN);

  @NbBundle.Messages("TIT_ViewProperty.BULLET_COLOR=Bullet Color")
  public static final ViewProperty<Color> BULLET_COLOR = create("BULLET_COLOR", AppliesTo.RELATION_ONLY, COLOR_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.BULLET_FONT=Bullet Font")
  public static final ViewProperty<Font> BULLET_FONT = create("BULLET_FONT", AppliesTo.RELATION_ONLY, FONT_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.BULLET_STRING=Bullet String")
  public static final ViewProperty<String> BULLET_STRING = create("BULLET_STRING", AppliesTo.RELATION_ONLY, ViewPropertyTypes.STRING);
  @NbBundle.Messages("TIT_ViewProperty.ELLIPSIS_COLOR=Ellipsis Color")
  public static final ViewProperty<Color> ELLIPSIS_COLOR = create("ELLIPSIS_COLOR", AppliesTo.ANY, COLOR_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.ELLIPSIS_FONT=Ellipsis Font")
  public static final ViewProperty<Font> ELLIPSIS_FONT = create("ELLIPSIS_FONT", AppliesTo.ANY, FONT_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.ELLIPSIS_STRING=Ellipsis String")
  public static final ViewProperty<String> ELLIPSIS_STRING = create("ELLIPSIS_STRING", AppliesTo.ANY, ViewPropertyTypes.STRING);

  @NbBundle.Messages("TIT_ViewProperty.COLOR_LABEL=Label Color")
  public static final ViewProperty<Color> COLOR_LABEL = create("COLOR_LABEL", AppliesTo.ANY, COLOR_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.COLOR_VALUE=Value Color")
  public static final ViewProperty<Color> COLOR_VALUE = create("COLOR_VALUE", AppliesTo.ANY, COLOR_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.COLOR_VALUE_ALTERNATIVE=Value Color Alternating Rows")
  public static final ViewProperty<Color> COLOR_VALUE_ALTERNATIVE = create("COLOR_VALUE_ALTERNATIVE", AppliesTo.ANY, COLOR_TYPE);

  @NbBundle.Messages("TIT_ViewProperty.OUTLINE_INDENT_WIDTH=Outline Indentation Width")
  public static final ViewProperty<Points> OUTLINE_INDENT_WIDTH = create("OUTLINE_INDENT_WIDTH", AppliesTo.RELATION_ONLY, LayoutPropertyTypes.POINTS);
  @NbBundle.Messages("TIT_ViewProperty.OUTLINE_TUPLE_SPACE=Vertical Outline Tuple Spacing")
  public static final ViewProperty<Points> OUTLINE_TUPLE_SPACE = create("OUTLINE_TUPLE_SPACE", AppliesTo.RELATION_ONLY, LayoutPropertyTypes.POINTS);
  @NbBundle.Messages("TIT_ViewProperty.OUTLINE_EMPTY_TABLE_SPACE=Empty Table Height")
  public static final ViewProperty<Points> OUTLINE_EMPTY_TABLE_SPACE = create("OUTLINE_EMPTY_TABLE_SPACE", AppliesTo.RELATION_ONLY, LayoutPropertyTypes.POINTS);
  @NbBundle.Messages("TIT_ViewProperty.TABLE_NEST_SPACE_SIDE=Nested Table Margin")
  public static final ViewProperty<Points> TABLE_NEST_SPACE_SIDE = create("TABLE_NEST_SPACE_SIDE", AppliesTo.RELATION_ONLY, LayoutPropertyTypes.POINTS);

  @NbBundle.Messages("TIT_ViewProperty.VALUE_BORDER=Value Border")
  public static final ViewProperty<LineStyle> VALUE_BORDER = create("VALUE_BORDER", AppliesTo.ANY, LINE_STYLE_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.OUTLINE_LABEL_BORDER=Outline Label Border")
  public static final ViewProperty<LineStyle> OUTLINE_LABEL_BORDER = create("OUTLINE_LABEL_BORDER", AppliesTo.ANY, LINE_STYLE_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.SEP_STYLE_TABLE_FIELD=Table Field Separator")
  public static final ViewProperty<LineStyle> SEP_STYLE_TABLE_FIELD = create("SEP_STYLE_TABLE_FIELD", AppliesTo.RELATION_ONLY, LINE_STYLE_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.SEP_STYLE_TABLE_ROW=Table Row Separator")
  public static final ViewProperty<LineStyle> SEP_STYLE_TABLE_ROW = create("SEP_STYLE_TABLE_ROW", AppliesTo.RELATION_ONLY, LINE_STYLE_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.SEP_STYLE_TABLE_ROW_MINOR=Table Minor Row Separator")
  public static final ViewProperty<LineStyle> SEP_STYLE_TABLE_ROW_MINOR = create("SEP_STYLE_TABLE_ROW_MINOR", AppliesTo.RELATION_ONLY, LINE_STYLE_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.SEP_STYLE_TABLE_HEADER=Table Header Separator")
  public static final ViewProperty<LineStyle> SEP_STYLE_TABLE_HEADER = create("SEP_STYLE_TABLE_HEADER", AppliesTo.RELATION_ONLY, LINE_STYLE_TYPE);

  @NbBundle.Messages("TIT_ViewProperty.MAX_INCOMPLETE_TUPLES=Incomplete Tuple Limit")
  public static final ViewProperty<Integer> MAX_INCOMPLETE_TUPLES = create("MAX_INCOMPLETE_TUPLES", AppliesTo.ROOT_ONLY, ViewPropertyTypes.INTEGER);

  @NbBundle.Messages("TIT_ViewProperty.TEMPORAL_STYLE=Date/Time Style")
  public static final ViewProperty<TemporalStyle> TEMPORAL_STYLE = create("TEMPORAL_STYLE", AppliesTo.PRIMITIVE_ONLY, LayoutPropertyTypes.TEMPORAL_STYLE);
  /** Display locale, specified by an IETF BCP 47 language tag. An empty string means the default locale. */
  @NbBundle.Messages("TIT_ViewProperty.DISPLAY_LOCALE=Display Locale")
  public static final ViewProperty<String> DISPLAY_LOCALE = create("DISPLAY_LOCALE", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.STRING);
  /** Time zone to use for displaying timestamps, in {@link java.time.ZoneId} format. An empty string means the system time zone. */
  @NbBundle.Messages("TIT_ViewProperty.DISPLAY_TIME_ZONE=Display Time Zone")
  public static final ViewProperty<String> DISPLAY_TIME_ZONE = create("DISPLAY_TIME_ZONE", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.STRING);
  @NbBundle.Messages("TIT_ViewProperty.CROSSTAB=Crosstab")
  public static final ViewProperty<Boolean> CROSSTAB = create("CROSSTAB", AppliesTo.RELATION_ONLY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.VISUALIZATION=Visualization")
  public static final ViewProperty<VisualizationType> VISUALIZATION = create("VISUALIZATION", AppliesTo.PRIMITIVE_ONLY, LayoutPropertyTypes.VISUALIZATION_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.VISUALIZATION_WIDTH=Visualization Width")
  public static final ViewProperty<Points> VISUALIZATION_WIDTH = create("VISUALIZATION_WIDTH", AppliesTo.PRIMITIVE_ONLY, LayoutPropertyTypes.POINTS);
}
