// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.layout;

import com.sieuferd.cell.support.Points;
import com.sieuferd.datamodel.ViewProperty;
import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import com.sieuferd.datamodel.ViewPropertySet;
import com.sieuferd.datamodel.ViewPropertyType;
import com.sieuferd.datamodel.ViewPropertyTypes;
import com.sieuferd.datamodel.proptypes.OptionalBigDecimal;
import com.sieuferd.datamodel.proptypes.OptionalInteger;
import org.openide.util.NbBundle;

public final class LMProp extends ViewPropertySet {
  public static final LMProp INSTANCE = new LMProp();

  @NbBundle.Messages("TIT_LayoutMeasure=Auto-Style Measurements")
  private LMProp() {
    super("sieuferd_layout_measure", Bundle.TIT_LayoutMeasure(), null, 430);
  }

  private static <T> ViewProperty<T> create(
      String storageName, AppliesTo appliesTo, ViewPropertyType<T> type)
  {
    return addProperty(INSTANCE, storageName, appliesTo, type, false);
  }

  @NbBundle.Messages("TIT_ViewProperty.OVERRIDE_MEASUREMENTS=Override Measurements")
  public static final ViewProperty<Boolean> OVERRIDE_MEASUREMENTS = create("OVERRIDE_MEASUREMENTS", AppliesTo.ANY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.SAMPLE_SIZE=Sample Size")
  public static final ViewProperty<Integer> SAMPLE_SIZE = create("SAMPLE_SIZE", AppliesTo.ANY, ViewPropertyTypes.INTEGER);
  /* Note: Even values that always have the same length may be considered variable width if their
  table columns should be allowed to expand during justification (e.g. fields that always have the
  same number of characters, but that are too long to fit within TABLE_MAX_PRIMITIVE_WIDTH). */
  @NbBundle.Messages("TIT_ViewProperty.VARIABLE_WIDTH=Variable Width")
  public static final ViewProperty<Boolean> VARIABLE_WIDTH = create("VARIABLE_WIDTH", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.AVERAGE_CARDINALITY=Average Cardinality")
  public static final ViewProperty<Double> AVERAGE_CARDINALITY = create("AVERAGE_CARDINALITY", AppliesTo.RELATION_ONLY, ViewPropertyTypes.DOUBLE);
  @NbBundle.Messages("TIT_ViewProperty.VALUE_WIDTH=Value Width")
  public static final ViewProperty<Points> VALUE_WIDTH = create("VALUE_WIDTH", AppliesTo.PRIMITIVE_ONLY, LayoutPropertyTypes.POINTS);
  @NbBundle.Messages("TIT_ViewProperty.CONFIDENCE_WIDTH=Value Confidence Width")
  public static final ViewProperty<Points> CONFIDENCE_WIDTH = create("CONFIDENCE_WIDTH", AppliesTo.PRIMITIVE_ONLY, LayoutPropertyTypes.POINTS);
  @NbBundle.Messages("TIT_ViewProperty.DECIMAL_PLACES=Decimal Places")
  public static final ViewProperty<OptionalInteger> DECIMAL_PLACES = create("DECIMAL_PLACES", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.OPTIONAL_INTEGER);
  @NbBundle.Messages("TIT_ViewProperty.GROUP_DIGITS=Group Digits")
  public static final ViewProperty<Boolean> GROUP_DIGITS = create("GROUP_DIGITS", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.TEMPORAL_PRECISION=Date/Time Precision")
  public static final ViewProperty<TemporalPrecision> TEMPORAL_PRECISION = create("TEMPORAL_PRECISION", AppliesTo.PRIMITIVE_ONLY, LayoutPropertyTypes.TEMPORAL_PRECISION);

  @NbBundle.Messages("TIT_ViewProperty.NUMERIC_MAX=Numeric Maximum")
  public static final ViewProperty<OptionalBigDecimal> NUMERIC_MAX = create("NUMERIC_MAX", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.OPTIONAL_BIG_DECIMAL);
  @NbBundle.Messages("TIT_ViewProperty.NUMERIC_MIN=Numeric Minimum")
  public static final ViewProperty<OptionalBigDecimal> NUMERIC_MIN = create("NUMERIC_MIN", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.OPTIONAL_BIG_DECIMAL);

  @NbBundle.Messages("TIT_ViewProperty.CROSSTAB_COLUMNS=Crosstab Columns")
  public static final ViewProperty<CrosstabColumns> CROSSTAB_COLUMNS = create("CROSSTAB_COLUMNS", AppliesTo.RELATION_ONLY, LayoutPropertyTypes.CROSSTAB_COLUMNS);
}
