// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.layout;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableSet;
import com.sieuferd.cell.styles.Text;
import com.sieuferd.cell.styles.Text.SimpleText;
import com.sieuferd.datamodel.CProp;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.View;
import com.sieuferd.datamodel.ViewProperty;
import java.util.Set;

public abstract class LabelFormatter {
  private final ImmutableSet<ViewProperty<?>> badgeProperties;

  public static final LabelFormatter SIMPLE = new LabelFormatter(ImmutableSet.of()) {
    @Override
    public Text getStyledLabel(View view, Path path, boolean rightAligned, String label) {
      return new SimpleText(label);
    }
  };

  public LabelFormatter(Set<ViewProperty<?>> badgeProperties) {
    Preconditions.checkNotNull(badgeProperties);
    this.badgeProperties = ImmutableSet.copyOf(badgeProperties);
  }

  // TODO[cc]: Get rid of this, or rename.
  public final Set<ViewProperty<?>> getBadgeProperties() {
    return badgeProperties;
  }

  public final Text getStyledLabel(View view, Path path, boolean rightAligned) {
    return getStyledLabel(view, path, rightAligned, view.get(CProp.LABEL, path));
  }

  /**
   * @param label the string to use as the label; custom strings may be used for instance in
   *        crosstab headings
   */
  public abstract Text getStyledLabel(
      View view, Path path, boolean rightAligned, String label);
}
