// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.layout;

import com.google.common.base.Preconditions;
import com.sieuferd.datamodel.ViewPropertyTypes.EnumMapping;
import javax.annotation.concurrent.Immutable;
import org.openide.util.NbBundle;

@NbBundle.Messages({
  "ENU_TemporalPrecision_DATE=Date",
  "ENU_TemporalPrecision_MINUTE=Minute",
  "ENU_TemporalPrecision_SECOND=Second",
  "ENU_TemporalPrecision_TENTH_SECOND=Tenth Second",
  "ENU_TemporalPrecision_HUNDREDTH_SECOND=Hundredth Second",
  "ENU_TemporalPrecision_MILLISECOND=Millisecond",
  "ENU_TemporalPrecision_MICROSECOND=Microsecond",
  "ENU_TemporalPrecision_NANOSECOND=Nanosecond"
})
@Immutable
public enum TemporalPrecision implements EnumMapping {
  DATE("date", Bundle.ENU_TemporalPrecision_DATE(), false, 0),
  MINUTE("minute", Bundle.ENU_TemporalPrecision_MINUTE(), false, 0),
  SECOND("second", Bundle.ENU_TemporalPrecision_SECOND(), true, 0),
  TENTH_SECOND("tenth_second", Bundle.ENU_TemporalPrecision_TENTH_SECOND(), true, 1),
  HUNDREDTH_SECOND("hundredth_second", Bundle.ENU_TemporalPrecision_HUNDREDTH_SECOND(), true, 2),
  MILLISECOND("millisecond", Bundle.ENU_TemporalPrecision_MILLISECOND(), true, 3),
  MICROSECOND("microsecond", Bundle.ENU_TemporalPrecision_MICROSECOND(), true, 6),
  NANOSECOND("nanosecond", Bundle.ENU_TemporalPrecision_NANOSECOND(), true, 9);

  private final String storageName;
  private final String presentationName;
  private final boolean includingSeconds;
  private final int secondFractionDigits;

  private TemporalPrecision(String storageName, String presentationName,
      boolean includingSeconds, int secondFractionDigits)
  {
    Preconditions.checkNotNull(storageName);
    Preconditions.checkNotNull(presentationName);
    Preconditions.checkArgument(secondFractionDigits == 0 || includingSeconds);
    this.storageName = storageName;
    this.presentationName = presentationName;
    this.includingSeconds = includingSeconds;
    this.secondFractionDigits = secondFractionDigits;
  }

  @Override
  public String getStorageName() {
    return storageName;
  }

  @Override
  public String toString() {
    return presentationName;
  }

  public boolean isIncludingSeconds() {
    return includingSeconds;
  }

  public int getSecondFractionDigits() {
    return secondFractionDigits;
  }
}
