// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.layout;

import com.google.common.base.Preconditions;
import com.sieuferd.datamodel.ViewPropertyTypes.EnumMapping;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.Locale;
import javax.annotation.Nullable;
import org.openide.util.NbBundle;

/**
 * Categories for date formatting styles.
 */
@NbBundle.Messages({
  "ENU_TemporalStyle_ISO=International (ISO 8601)",
  "ENU_TemporalStyle_LOCALIZED_SHORT=Localized Short",
  "ENU_TemporalStyle_LOCALIZED_MEDIUM=Localized Medium",
  "ENU_TemporalStyle_LOCALIZED_LONG=Localized Long",
  "ENU_TemporalStyle_LOCALIZED_FULL=Localized Full"
})
public enum TemporalStyle implements EnumMapping {
  ISO("iso", Bundle.ENU_TemporalStyle_ISO(), null),
  LOCALIZED_SHORT("localized_short", Bundle.ENU_TemporalStyle_LOCALIZED_SHORT(), FormatStyle.SHORT),
  LOCALIZED_MEDIUM("localized_medium", Bundle.ENU_TemporalStyle_LOCALIZED_MEDIUM(), FormatStyle.MEDIUM),
  LOCALIZED_LONG("localized_long", Bundle.ENU_TemporalStyle_LOCALIZED_LONG(), FormatStyle.LONG),
  LOCALIZED_FULL("localized_full", Bundle.ENU_TemporalStyle_LOCALIZED_FULL(), FormatStyle.FULL);

  private final String presentationString;
  private final String storageName;
  private final @Nullable FormatStyle localizedStyle;

  private TemporalStyle(
      String storageName, String presentationString, @Nullable FormatStyle localizedStyle)
  {
    Preconditions.checkNotNull(storageName);
    Preconditions.checkNotNull(presentationString);
    this.storageName = storageName;
    this.presentationString = presentationString;
    this.localizedStyle = localizedStyle;
  }

  @Override
  public String getStorageName() {
    return storageName;
  }

  @Override
  public String toString() {
    return presentationString;
  }

  DateTimeFormatter getFormatter(
      TemporalFormatterProvider type, Locale locale, TemporalPrecision precision)
  {
    return localizedStyle == null
        ? type.getISOFormatter(precision)
        : type.getLocalizedFormatter(localizedStyle, locale, precision);
  }
}