// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.layout.internal;

import com.sieuferd.cell.support.Coordinates;
import com.sieuferd.cell.support.Pixels;
import com.google.common.collect.Lists;
import java.util.List;
import com.sieuferd.cell.support.Cell;
import com.sieuferd.cell.builders.ListCellBuilder;
import javax.annotation.Nullable;

public class Column {
  private final LayoutContext parentContext;
  private final Pixels columnWidth;
  private final List<OutlineField> fields = Lists.newArrayList();
  private @Nullable Pixels unpaddedHeight;

  public Column(LayoutContext parentContext, Pixels columnWidth) {
    this.columnWidth = columnWidth;
    this.parentContext = parentContext;
  }

  public void addField(OutlineField field) {
    fields.add(field);
    unpaddedHeight = null;
  }

  private Cell getCellInternal(Pixels requiredPaddingHeight) {
    ListCellBuilder builder = parentContext.verticalListCellBuilder()
          .minimumDimensions(new Coordinates(columnWidth, Pixels.ZERO));
    for (int i = 0; i < fields.size(); i++) {
      OutlineField field = fields.get(i);
      builder.addCell(field.getCell(i == fields.size() - 1 ? requiredPaddingHeight : Pixels.ZERO));
    }
    return builder.build();
  }

  public Cell getPaddedCell(Pixels padToColumnHeight) {
    return getCellInternal(padToColumnHeight.minus(getUnpaddedHeight()));
  }

  public Pixels getUnpaddedHeight() {
    if (unpaddedHeight == null)
      unpaddedHeight = getCellInternal(Pixels.ZERO).getDimensions().getY();
    return unpaddedHeight;
  }
}
