// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.layout.internal;

import com.google.common.base.Preconditions;
import com.sieuferd.cell.support.Coordinates;
import com.sieuferd.cell.support.Pixels;
import com.sieuferd.cell.support.Cell;
import com.sieuferd.cell.support.Direction;
import com.sieuferd.layout.LBProp;
import com.sieuferd.layout.LEProp;

public class OutlineField {
  private static final boolean PAD_INVISIBLE_LABELS = false;
  private final LayoutContext context;
  private final Cell contentCell;
  private final Pixels labelWidth;
  private final boolean labelAdjacent;

  /**
   * @param labelWidth the width of the label as if it was visible, not including border
   */
  public OutlineField(
      LayoutContext context, Cell contentCell, Pixels labelWidth, boolean labelAdjacent)
  {
    Preconditions.checkNotNull(context);
    Preconditions.checkNotNull(contentCell);
    Preconditions.checkNotNull(labelWidth);
    this.context = context;
    this.contentCell = contentCell;
    this.labelWidth = labelWidth;
    this.labelAdjacent = labelAdjacent;
  }

  public Cell getCell(Pixels labelPadding) {
    Pixels labelHeightForPadding;
    if (labelAdjacent && labelPadding.isGreaterThan(Pixels.ZERO)) {
      labelHeightForPadding = getCell(Pixels.ZERO).getDimensions().getY().plus(labelPadding);
    } else {
      labelHeightForPadding = Pixels.ZERO;
    }
    if (context.getP(LEProp.EFF_LABEL_VISIBLE)) {
      Cell labelCell = context.textCellBuilder(
          LBProp.LABEL_TEXT_COLOR, context.getLabelFontProperty(), labelWidth, true)
          .content(context.getStyledLabel(false))
          .build();
      final Cell borderedLabelCell =
          context.placeholderTraversal().selectableBorderCellBuilder(labelCell)
          .borders(LBProp.OUTLINE_LABEL_BORDER)
          .backgroundColor(LBProp.COLOR_LABEL)
          /* Justify the label cell to ensure that the maximum amount of space possible is available
          for baseline alignment. */
          .justify()
          .minimumDimensions(new Coordinates(Pixels.ZERO, labelAdjacent ?
              contentCell.getDimensions().getY().max(labelHeightForPadding) : Pixels.ZERO))
          .build();
      if (labelAdjacent) {
        return context.horizontalListCellBuilder()
            .addCell(borderedLabelCell)
            .addCell(contentCell)
            .build();
        /* I tried expanding primitive value cells vertically in cases where an adjacent label cell
        is taller, but this looked ugly. Primitive cells look best when they are all one text line
        tall. */
      } else {
        return context.headeredCellBuilder(true, borderedLabelCell, contentCell).build();
      }
    } else {
      if (labelAdjacent && PAD_INVISIBLE_LABELS) {
        Cell paddingCell = context.selectableBorderCellBuilder(context.emptyCellBuilder().build())
            .cursorBias(Direction.UP)
            .minimumDimensions(new Coordinates(labelWidth, Pixels.ZERO))
            .build();
        final Cell borderedPaddingCell =
            context.borderCellBuilder(paddingCell)
            .borders(LBProp.OUTLINE_LABEL_BORDER)
            .backgroundColor(LBProp.COLOR_LABEL)
            .minimumDimensions(new Coordinates(Pixels.ZERO, labelPadding))
            .build();
        if (borderedPaddingCell.getDimensions().getY().isGreaterThan(labelPadding)) {
          return contentCell;
        } else {
          return context.verticalListCellBuilder()
              .addCell(contentCell)
              .addCell(borderedPaddingCell)
              .build();
        }
      } else {
        return contentCell;
      }
    }
  }
}
