// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.query;

import com.google.common.base.Preconditions;
import com.sieuferd.datamodel.Path.PrimitivePath;
import java.util.Objects;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class OptionalPrimitivePath {
  private static final OptionalPrimitivePath ABSENT = new OptionalPrimitivePath(null);
  private final @Nullable PrimitivePath value;

  public static OptionalPrimitivePath absent() {
    return ABSENT;
  }

  public static OptionalPrimitivePath of(PrimitivePath value) {
    Preconditions.checkNotNull(value);
    return new OptionalPrimitivePath(value);
  }

  private OptionalPrimitivePath(@Nullable PrimitivePath value) {
    this.value = value;
  }

  public boolean isPresent() {
    return value != null;
  }

  public PrimitivePath getPath() {
    if (value == null)
      throw new IllegalStateException("Optional path not set");
    return value;
  }

  public @Nullable PrimitivePath getNullable() {
    return value;
  }

  @Override
  public int hashCode() {
    return Objects.hash(value);
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof OptionalPrimitivePath))
      return false;
    OptionalPrimitivePath other = (OptionalPrimitivePath) obj;
    return Objects.equals(other.value, this.value);
  }
}
