// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.query;

import com.sieuferd.datamodel.proptypes.Position;
import com.sieuferd.datamodel.ViewDefaults;
import com.sieuferd.datamodel.ViewProperty;
import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import com.sieuferd.datamodel.ViewPropertySet;
import com.sieuferd.datamodel.ViewPropertyType;
import com.sieuferd.datamodel.ViewPropertyTypes;
import com.sieuferd.datamodel.proptypes.OptionalInteger;
import com.sieuferd.datamodel.expressions.Expression;
import com.sieuferd.datamodel.expressions.Expression.ConstantExpression;
import org.openide.util.NbBundle;

/** Query-related properties. */
public final class QProp extends ViewPropertySet {
  public static final QProp INSTANCE = new QProp();

  @NbBundle.Messages("TIT_Query=Query")
  private QProp() {
    super("sieuferd_query", Bundle.TIT_Query(), null, 200);
  }

  private static <T> ViewProperty<T> create(
      String storageName, AppliesTo appliesTo, ViewPropertyType<T> type)
  {
    return addProperty(INSTANCE, storageName, appliesTo, type, false);
  }

  private static <T> ViewProperty<T> createRO(
      String storageName, AppliesTo appliesTo, ViewPropertyType<T> type)
  {
    return addProperty(INSTANCE, storageName, appliesTo, type, true);
  }

  @NbBundle.Messages("TIT_ViewProperty.PRIMARY=Primary View")
  public static final ViewProperty<Boolean> PRIMARY = createRO("PRIMARY", AppliesTo.ROOT_ONLY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.IN_PRIMARY_KEY=In Primary Key")
  public static final ViewProperty<Boolean> IN_PRIMARY_KEY = createRO("IN_PRIMARY_KEY", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.ORDER_RANK=Sort Rank")
  public static final ViewProperty<Position> ORDER_RANK = create("ORDER_RANK", AppliesTo.ANY, ViewPropertyTypes.POSITION);
  // Only matters when ORDER_RANK is not Position.NONE.
  @NbBundle.Messages("TIT_ViewProperty.ORDER_DESCENDING=Descending Sort")
  public static final ViewProperty<Boolean> ORDER_DESCENDING = create("ORDER_DESCENDING", AppliesTo.ANY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.ORDER_NULLS_FIRST=Nulls First")
  public static final ViewProperty<Boolean> ORDER_NULLS_FIRST = create("ORDER_NULLS_FIRST", AppliesTo.ANY, ViewPropertyTypes.BOOLEAN);
  @NbBundle.Messages("TIT_ViewProperty.INSTANTIATED_TABLE=Instantiated SQL Table")
  public static final ViewProperty<InstanceDefinition> INSTANTIATED_TABLE = create("INSTANTIATED_TABLE", AppliesTo.RELATION_ONLY, QueryPropertyTypes.INSTANCE_DEFINITION_TYPE);
  // Only matters when INSTANTIATED_TABLE encloses a non-null value.
  @NbBundle.Messages("TIT_ViewProperty.INCLUDE_PRIMARY_KEY=Include Primary Key")
  public static final ViewProperty<Boolean> INCLUDE_PRIMARY_KEY = create("INCLUDE_PRIMARY_KEY", AppliesTo.RELATION_ONLY, ViewPropertyTypes.BOOLEAN);
  // Only matters when SQL_COLUMN_NAME is empty.
  @NbBundle.Messages("TIT_ViewProperty.FORMULA_EXPRESSION=Formula Expression")
  public static final ViewProperty<Expression> FORMULA_EXPRESSION = create("FORMULA_EXPRESSION", AppliesTo.PRIMITIVE_ONLY, QueryPropertyTypes.EXPRESSION_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.FILTER=Filter")
  public static final ViewProperty<Filter> FILTER = create("FILTER", AppliesTo.ANY, QueryPropertyTypes.FILTER_TYPE);
  @NbBundle.Messages("TIT_ViewProperty.EXISTENTIAL_FILTER=Existential Filter")
  public static final ViewProperty<Boolean> EXISTENTIAL_FILTER = create("EXISTENTIAL_FILTER", AppliesTo.RELATION_ONLY, ViewPropertyTypes.BOOLEAN);
  /** The target path must be enclosed by the parent relation of the enclosing relation of the path
  for which the property is defined. */
  @NbBundle.Messages("TIT_ViewProperty.JOINED_ON=Joined On")
  public static final ViewProperty<OptionalPrimitivePath> JOINED_ON = create("JOINED_ON", AppliesTo.PRIMITIVE_ONLY, QueryPropertyTypes.OPTIONAL_PRIMITIVE_PATH_TYPE);

  // If empty, use FORMULA_EXPRESSION instead. Must be empty if INSTANTIATED_TABLE encloses a null value.
  @NbBundle.Messages("TIT_ViewProperty.SQL_COLUMN_NAME=SQL Column Name")
  public static final ViewProperty<String> SQL_COLUMN_NAME = create("SQL_COLUMN_NAME", AppliesTo.PRIMITIVE_ONLY, ViewPropertyTypes.STRING);
  @NbBundle.Messages("TIT_ViewProperty.TUPLE_RESULT_LIMIT=Result Limit")
  public static final ViewProperty<OptionalInteger> TUPLE_RESULT_LIMIT = create("TUPLE_RESULT_LIMIT", AppliesTo.RELATION_ONLY, ViewPropertyTypes.OPTIONAL_INTEGER);

  public static final ViewDefaults DEFAULTS = ViewDefaults.builder()
      .set(PRIMARY, false)
      .set(IN_PRIMARY_KEY, true)
      .set(ORDER_RANK, Position.NONE)
      .set(ORDER_DESCENDING, false)
      .set(ORDER_NULLS_FIRST, false)
      .set(INSTANTIATED_TABLE, new InstanceDefinition(null))
      .set(INCLUDE_PRIMARY_KEY, true)
      .set(FORMULA_EXPRESSION, ConstantExpression.NULL)
      .set(FILTER, Filter.UNCONSTRAINED)
      .set(EXISTENTIAL_FILTER, false)
      .set(JOINED_ON, OptionalPrimitivePath.absent())
      .set(SQL_COLUMN_NAME, "")
      .set(TUPLE_RESULT_LIMIT, OptionalInteger.absent())
      .build();
}
