// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.query;

import com.google.common.base.Objects;
import com.google.common.base.Preconditions;
import com.sieuferd.util.StringUtil;
import javax.annotation.concurrent.Immutable;

// TODO[cc]: Move this back into the Database module.
@Immutable
public final class SQLDatabase {
  /** May be empty, but not null. */
  private final String primaryNamePart;
  /** May be empty iff primaryNamePart is empty, may not be null. */
  private final String secondaryNamePart;

  /**
   * @param primaryNamePart must be trimmed, may be empty, may not be null
   * @param secondaryNamePart must be trimmed, may be empty iff primaryNamePart is empty, may not be
   *                          null
   */
  public SQLDatabase(String primaryNamePart, String secondaryNamePart) {
    StringUtil.checkArgumentTrimmed(primaryNamePart);
    StringUtil.checkArgumentTrimmed(secondaryNamePart);
    Preconditions.checkArgument(!(primaryNamePart.isEmpty() && !secondaryNamePart.isEmpty()));
    this.primaryNamePart = primaryNamePart;
    this.secondaryNamePart = secondaryNamePart;
  }

  /** @return may be empty, but not null. */
  public String getPrimaryNamePart() {
    return primaryNamePart;
  }

  /** @return may be empty, but not null. */
  public String getSecondaryNamePart() {
    return secondaryNamePart;
  }

  public boolean isEmpty() {
    return primaryNamePart.isEmpty() && secondaryNamePart.isEmpty();
  }

  @Override
  public int hashCode() {
    return Objects.hashCode(primaryNamePart, secondaryNamePart);
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null || !(obj instanceof SQLDatabase))
      return false;
    if (obj == this)
      return true;
    SQLDatabase rhs = (SQLDatabase) obj;
    return Objects.equal(primaryNamePart  , rhs.primaryNamePart)
        && Objects.equal(secondaryNamePart, rhs.secondaryNamePart);
  }

  @Override
  public String toString() {
    return isEmpty() ? "(empty)" :
        (primaryNamePart + (secondaryNamePart.isEmpty() ? "" : ("." + secondaryNamePart)));
  }
}
