// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.flow;

import javax.annotation.Nullable;

/* TODO[cc]: Consider making this package-private. At least discourage its direct use and recommend
             the ProcessorTemplate classes instead. */
/**
 * A factory for objects constructed from a predetermined number of non-null {@code Object}
 * arguments of predetermined types. Implementations must be thread-safe; this is easy to achieve by
 * simply refraining from accessing any internal or external state.
 */
public abstract class CompositeFactory<T> {
  // TODO[cc]: Avoid exposing this.
  // Used to implement ValueProcessor.clearState() in the synchronous case.
  void clearState() {
  }

  /* Allowing a null return value here makes CompositeFactory more useful as a building block for
  other operations, e.g. to implement synchronous value processors. */
  /**
   * Construct the composite object.
   *
   * @param args the input arguments, each guaranteed to be non-null
   */
  protected abstract @Nullable T construct(Object args[]);
}
