// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.flow;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableSet;
import java.util.Set;
import javax.annotation.Nullable;

final class DefaultFlow<T> extends Flow<T> {
  private final Flow<T> input;
  private final @Nullable T defaultValue;
  private @Nullable Snapshot<T> lastInput;

  DefaultFlow(Flow<T> input, @Nullable T defaultValue) {
    super("Default");
    Preconditions.checkNotNull(input);
    this.input = input;
    this.defaultValue = defaultValue;
  }

  @Override
  protected void firstListenerAdded() {
    input.addChangeListener(propagatingChangeListener);
  }

  @Override
  protected void lastListenerRemoved() {
    input.removeChangeListener(propagatingChangeListener);
    lastInput = null;
  }

  @Override
  protected void update() {
    final Snapshot<T> currentEvaluatedInput = input.getSnapshot();
    if (currentEvaluatedInput != lastInput) {
      lastInput = currentEvaluatedInput;
      final T inputValue = currentEvaluatedInput.getValue();
      setInternal(inputValue == null ? defaultValue : inputValue);
    }
  }

  @Override
  protected Set<Flow<?>> getDependencies() {
    return ImmutableSet.<Flow<?>>of(input);
  }
}
