// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.flow;

import com.google.common.base.Preconditions;

public final class ProcessorTemplates {
  private ProcessorTemplates() { }

  private static final class RawComposite {
    private final Object[] fields;

    RawComposite(Object[] fields) {
      Preconditions.checkNotNull(fields);
      this.fields = fields;
    }

    private static Flow<RawComposite> fromInputs(Flow<?> ... inputs) {
      return Flows.newComposite(inputs, new CompositeFactory<RawComposite>() {
        @Override
        protected RawComposite construct(Object[] args) {
          return new RawComposite(args);
        }
      });
    }
  }

  public static abstract class ProcessorTemplate<O> {
    /** May be overridden by subclasses. */
    public ValueProcessor.Synchronicity getSynchronicity() {
      return ValueProcessor.Synchronicity.SYNCHRONOUS;
    }
  }

  /* ============================================================================================ */

  public static interface Function2<O,I0,I1>       { O apply(I0 i0, I1 i1              ); }
  public static interface Function3<O,I0,I1,I2>    { O apply(I0 i0, I1 i1, I2 i2       ); }
  public static interface Function4<O,I0,I1,I2,I3> { O apply(I0 i0, I1 i1, I2 i2, I3 i3); }

  /* ============================================================================================ */
  public static abstract class ProcessorTemplate2<O,I0,I1>
      extends ProcessorTemplate<O>
  {
    // TODO[cc]: Consider renaming these to just "flow".
    public Flow<O> getFlow(
        Flow<I0> i0,
        Flow<I1> i1
    ) {
      return Flows.newProcessed(RawComposite.fromInputs(
          i0, i1
          ), new ValueProcessor<RawComposite,O>()
      {
        @Override
        @SuppressWarnings("unchecked")
        public O process(RawComposite input) throws InterruptedException {
          return ProcessorTemplate2.this.process(
              (I0) input.fields[0],
              (I1) input.fields[1]
          );
        }

        @Override
        public ValueProcessor.Synchronicity getSynchronicity() {
          return ProcessorTemplate2.this.getSynchronicity();
        }
      });
    }

    protected abstract O process(I0 arg0, I1 arg1) throws InterruptedException;
  }

  /* ============================================================================================ */
  public static abstract class ProcessorTemplate3<O,I0,I1,I2>
      extends ProcessorTemplate<O>
  {
    public Flow<O> getFlow(
        Flow<I0> i0,
        Flow<I1> i1,
        Flow<I2> i2
    ) {
      return Flows.newProcessed(RawComposite.fromInputs(
          i0, i1, i2
          ), new ValueProcessor<RawComposite,O>()
      {
        @Override
        @SuppressWarnings("unchecked")
        public O process(RawComposite input) throws InterruptedException {
          return ProcessorTemplate3.this.process(
              (I0) input.fields[0],
              (I1) input.fields[1],
              (I2) input.fields[2]
          );
        }

        @Override
        public ValueProcessor.Synchronicity getSynchronicity() {
          return ProcessorTemplate3.this.getSynchronicity();
        }
      });
    }

    protected abstract O process(I0 arg0, I1 arg1, I2 arg2) throws InterruptedException;
  }

  /* ============================================================================================ */
  public static abstract class ProcessorTemplate4<O,I0,I1,I2,I3>
      extends ProcessorTemplate<O>
  {
    public Flow<O> getFlow(
        Flow<I0> i0,
        Flow<I1> i1,
        Flow<I2> i2,
        Flow<I3> i3
    ) {
      return Flows.newProcessed(RawComposite.fromInputs(
          i0, i1, i2, i3
          ), new ValueProcessor<RawComposite,O>()
      {
        @Override
        @SuppressWarnings("unchecked")
        public O process(RawComposite input) throws InterruptedException {
          return ProcessorTemplate4.this.process(
              (I0) input.fields[0],
              (I1) input.fields[1],
              (I2) input.fields[2],
              (I3) input.fields[3]
          );
        }

        @Override
        public ValueProcessor.Synchronicity getSynchronicity() {
          return ProcessorTemplate4.this.getSynchronicity();
        }
      });
    }

    protected abstract O process(I0 arg0, I1 arg1, I2 arg2, I3 arg3) throws InterruptedException;
  }
}
