// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.flow;

public interface Refreshable {
  public void invalidate();
  public void refresh();

  public final class RefreshableAndFlow<T> {
    private final Flow<T> flow;
    private final SettableFlow<Object> nudgeable = Flows.newSettable();
    private final Refreshable refreshable = new Refreshable() {
      @Override
      public void invalidate() {
        nudgeable.set(null);
      }

      @Override
      public void refresh() {
        nudgeable.set(new Object());
      }
    };

    RefreshableAndFlow(Flow<T> input) {
      /* Model the refreshable flow as a composite of the external input flow and an otherwise
      meaningless placeholder object that can be changed to disturb the snapshot. */
      this.flow = Flows.newComposite2(input, nudgeable, (input2, nudgeable2) -> input2);
      getRefreshable().refresh();
    }

    public Flow<T> getFlow() {
      return flow;
    }

    public Refreshable getRefreshable() {
      return refreshable;
    }
  }
}
