// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.flow;

import com.google.common.base.Preconditions;
import javax.annotation.Nullable;

/**
 * Container for a concrete output value of {@link Flow}. This class is only of interest to
 * implementations of {@code Flow}. Such implementations can check whether input values have changed
 * since the last time they were polled by comparing the referential identity of the current and the
 * previous {@code Snapshot}s. This approach makes it possible to detect changes even if the
 * contained object has returned to the previous value after a transient change. Shallowly
 * immutable.
 */
public final class Snapshot<T> {
  private final @Nullable T value;
  final long version;

  public static <T> Snapshot<T> create() {
    return new Snapshot<>(null, 0);
  }

  public Snapshot<T> update(@Nullable T value) {
    return new Snapshot<>(value, version + 1);
  }

  private Snapshot(@Nullable T value, long version) {
    Preconditions.checkArgument(version >= 0);
    this.value = value;
    this.version = version;
  }

  public @Nullable T getValue() {
    return value;
  }

  @Override
  public String toString() {
    return (value == null ? "(null)" : value.toString()) + "#" + version;
  }
}
