// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.flow;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableSet;
import java.util.Set;
import javax.annotation.Nullable;

final class StickyFlow<T> extends Flow<T> {
  private final Flow<T> input;
  private final @Nullable T initialValue;
  private @Nullable Snapshot<T> lastNonNullInput;
  private boolean initialValueSet;

  StickyFlow(Flow<T> input, @Nullable T initialValue) {
    super("Sticky");
    Preconditions.checkNotNull(input);
    this.input = input;
    this.initialValue = initialValue;
  }

  @Override
  protected void firstListenerAdded() {
    input.addChangeListener(propagatingChangeListener);
  }

  @Override
  protected void lastListenerRemoved() {
    input.removeChangeListener(propagatingChangeListener);
    lastNonNullInput = null;
    initialValueSet = false;
  }

  @Override
  protected void update() {
    final Snapshot<T> currentEvaluatedInput = input.getSnapshot();
    if (lastNonNullInput == null && currentEvaluatedInput.getValue() == null && !initialValueSet) {
      initialValueSet = true;
      setInternal(initialValue);
    } else {
      if (currentEvaluatedInput != lastNonNullInput && currentEvaluatedInput.getValue() != null) {
        lastNonNullInput = currentEvaluatedInput;
        setInternal(currentEvaluatedInput.getValue());
      }
    }
  }

  @Override
  protected Set<Flow<?>> getDependencies() {
    return ImmutableSet.<Flow<?>>of(input);
  }
}
