// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.util;

import java.util.Iterator;
import java.util.function.Function;

public final class IterableUtil {
  private IterableUtil() {}

  public static <S,D> Iterable<D> transform(final Iterable<S> src, final Function<S,D> function) {
    return () -> {
      final Iterator<S> srcIter = src.iterator();
      return new Iterator<D>() {
        @Override
        public boolean hasNext() {
          return srcIter.hasNext();
        }

        @Override
        public D next() {
          return function.apply(srcIter.next());
        }

        @Override
        public void remove() {
          srcIter.remove();
        }
      };
    };
  }

  public static boolean hasMoreThanOneItem(Iterable<?> iterable) {
    final Iterator<?> iterator = iterable.iterator();
    if (!iterator.hasNext())
      return false;
    iterator.next();
    return iterator.hasNext();
  }
}
