// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.util;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Maps;
import com.google.common.collect.Multimap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.NoSuchElementException;
import java.util.Set;
import javax.annotation.Nullable;

/* TODO[cc]: Make this work with ConcurrentMap. Right now several of thse methods have race
             conditions. (Not too critical since these are for debugging only, however.) */
public final class MapUtil {
  private MapUtil() {}

  // TODO[cc]: Rename this class so that even set-related methods can feel at home in here.
  public static <K> void addChecked(Set<K> set, K value) {
    if (!set.add(value))
      throw new IndexOutOfBoundsException("Duplicate value " + value + " in set " + set);
  }

  public static <K> void removeChecked(Set<K> set, K value) {
    if (!set.remove(value))
      throw new IndexOutOfBoundsException("No value " + value + " in set " + set);
  }

  public static <V> @Nullable V getLoneElementOrNull(Set<V> set) {
    return set.size() == 1 ? set.iterator().next() : null;
  }

  public static <V> ImmutableSet<V> fromNullable(@Nullable V value) {
    return (value == null) ? ImmutableSet.of() : ImmutableSet.of(value);
  }

  public static <K,V> void putChecked(Map<K,V> map, K key, V value) {
    if (map.containsKey(key))
      throw new IndexOutOfBoundsException("Duplicate key " + key + " in map " + map);
    map.put(key, value);
  }

  public static <K,V> V getChecked(Map<K,V> map, K key) {
    V ret = map.get(key);
    if (ret == null && !map.containsKey(key))
      throw new NoSuchElementException("Could not find key " + key.toString() + " in map " + map);
    return ret;
  }

  public static <K,V> V removeChecked(Map<K,V> map, K key) {
    if (!map.containsKey(key))
      throw new NoSuchElementException("Could not find key " + key.toString() + " in map " + map);
    return map.remove(key);
  }

  public static <K,V> void putChecked(Multimap<K,V> map, K key, V value) {
    if (map.containsEntry(key, value)) {
      throw new IndexOutOfBoundsException("Duplicate entry (" +
          key.toString() + ", " + value.toString() + ") in multimap " + map);
    }
    map.put(key, value);
  }

  public static <K,V> void removeChecked(Multimap<K,V> multimap, K key, V value) {
    if (!multimap.containsEntry(key, value)) {
      throw new NoSuchElementException("Could not find entry (" +
          key.toString() + ", " + value.toString() + ") in multimap " + multimap);
    }
    multimap.remove(key, value);
  }

  /**
   * The iteration order of the returned map is the same as that of the original map; if a value
   * occured more than once in the input map, its first occurence will be used for ordering
   * purposes.
   */
  public static <K,V> ImmutableMap<V,K> invert(Map<K,V> map) {
    Map<V,K> ret = Maps.newLinkedHashMap();
    for (Entry<K,V> entry : map.entrySet())
      ret.put(entry.getValue(), entry.getKey());
    return ImmutableMap.copyOf(ret);
  }
}
