// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.util;

import com.google.common.base.Preconditions;

/** This class is typically used to work around Java's lack of closures. I could have used
java.util.concurrent.atomic.AtomicReference, but that class is not supported by GWT. */
public class ObjectReference<T> {
  private volatile T value;

  private ObjectReference() { }

  public static <T> ObjectReference<T> create() {
    return new ObjectReference<>();
  }

  public T getValue() {
    Preconditions.checkNotNull(value);
    return value;
  }

  /** @param value may not be null */
  public void setValue(T value) {
    Preconditions.checkNotNull(value);
    // The check is not atomic, but it's only for catching bugs anyway.
    Preconditions.checkState(this.value == null);
    this.value = value;
  }
}
