// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.util;

import com.google.common.base.Preconditions;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.logging.Level;
import java.util.logging.Logger;

/* I considered encouraging the use of lambdas in this interface, but they can't throw checked
exceptions, which makes them less useful. */
public final class Stopwatch {
  private static final Logger LOG = Logger.getLogger(Stopwatch.class.getName());
  private static final Level LOG_LEVEL = Level.INFO;
  private final String operationName;
  private final boolean loggable;
  private final long startTime;
  private final AtomicBoolean stopped = new AtomicBoolean(false);

  public Stopwatch(String operationName) {
    Preconditions.checkNotNull(operationName);
    this.operationName = operationName;
    this.loggable = LOG.isLoggable(LOG_LEVEL);
    this.startTime = loggable ? System.nanoTime() : 0L;
  }

  public void stop() {
    if (stopped.getAndSet(true))
      throw new IllegalStateException("Already stopped");
    if (!loggable)
      return;
    final long endTime = System.nanoTime();
    LOG.log(LOG_LEVEL, "{0} took {1}ms", new Object[] {
        operationName, Math.round((endTime - startTime) / 1000000.0) });
  }
}
