// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.branding;

import java.awt.Image;
import java.lang.reflect.Method;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.netbeans.api.annotations.common.StaticResource;
import org.openide.modules.ModuleInstall;
import org.openide.util.ImageUtilities;
import org.openide.util.NbPreferences;
import org.openide.util.Utilities;

public class Installer extends ModuleInstall {
  private static final boolean FORCE_DAILY_AUTOUPDATE = false;
  /** May be incremented to test autoupdate of the branding module. */
  private static final int BRANDING_VERSION = 1;
  // Looks somewhat ugly (at least on MacOS); don't use.
  private static final boolean STRETCHING_VIEW_TABS = false;
  @StaticResource
  private static final String RESOURCE_LARGEICON = "com/sieuferd/branding/largeicon.png";
  private static final Logger LOG = Logger.getLogger(Installer.class.getName());

  // TODO[pd]: Consider including a cleanup method as well.
  @Override
  public void restored() {
    LOG.log(Level.INFO, "Sieuferd branding module version {0}", BRANDING_VERSION);
    if (Utilities.isMac()) {
      /* Note: Setting the com.apple.mrj.application.apple.menu.about.name property here (or in the
      main method of a simple Swing app) did not have the desired effect of changing the MacOS
      application menu title on MacOS 10.8.2 and Java 1.7.0_10. It works in Java 1.6 only if the
      property is set before Swing is initialized. See this StackOverflow question:
      http://stackoverflow.com/questions/10702463/java-os-x-lion-setting-application-name-stopped-working-with-jdk1-7 */

      Image img = ImageUtilities.loadImage(RESOURCE_LARGEICON, false);
      if (img == null) {
        LOG.warning("Failed to load dock icon resource");
        return;
      }
      setDockIconImage(img);
    }

    if (STRETCHING_VIEW_TABS)
      System.setProperty("winsys.stretching_view_tabs", Boolean.toString(true));

    /* Point this to a non-existent layer path to hide the status bar. See
    http://wiki.netbeans.org/DevFaqRemoveStatusBar and
    o.n.core.windows.view.ui.MainWindow.initializeComponents. */
    System.setProperty("netbeans.winsys.status_line.path",
        "com/sieuferd/branding/com-sieuferd-branding-NoStatusBar.instance");

    /* TODO[pd]: Consider storing a preference flag to indicate whether default settings like this
                 have been applied yet, then avoid setting it again in the future. This will make it
                 possible for the user to modify the setting later if desired. (Though for now I
                 don't actually expose the action that modifies it.) */
    /* This seems to be the most correct way to set "factory-specified" preferences that are not
    editor-related; see Geertjan Wielenga's response on the platform-dev mailing list of
    2013-11-01. */
    NbPreferences.root().node("org/netbeans/modules/projectui")
        .putBoolean("synchronizeEditorWithViews", true);

    if (FORCE_DAILY_AUTOUPDATE) {
      /* This seems to already be the default, though I haven't been able to figure out from where
      in the NetBeans sources. */
      final int AutoupdateSettings_EVERY_DAY = 1;
      NbPreferences.root().node("/org/netbeans/modules/autoupdate")
          .putInt("period", AutoupdateSettings_EVERY_DAY);
    }
  }

  private static void setDockIconImage(Image img) {
    try {
      final Class<?> applicationClass = Class.forName("com.apple.eawt.Application");
      final Method getApplicationMethod = applicationClass.getMethod("getApplication");
      final Method setDockIconImageMethod =
          applicationClass.getMethod("setDockIconImage", Image.class);
      final Object applicationObject = getApplicationMethod.invoke(null);
      setDockIconImageMethod.invoke(applicationObject, img);
    } catch (Exception e) {
      LOG.log(Level.WARNING, "Failed to invoke com.apple.eawt.Application.setDockIconImage()", e);
    }
  }

  @Override
  public void validate() {
    // Do this early in the module loading cycle to make sure the setting isn't lost.
    // See http://wiki.netbeans.org/FaqPluginManagerCustomization
    System.setProperty("plugin.manager.check.interval", "EVERY_DAY");
  }
}
