// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.updater;

import org.netbeans.api.autoupdate.InstallSupport;
import org.netbeans.api.autoupdate.OperationContainer;
import org.netbeans.api.autoupdate.OperationContainer.OperationInfo;
import org.netbeans.api.autoupdate.UpdateElement;

final class OperationCollector {
  /** May be null. */
  private OperationContainer<InstallSupport> container;

  public void addUpdateElement(UpdateElement element) {
    if (container == null) {
      /* TODO[pd]: Consider if I need to do OperationContainer.createForInternalUpdate() if any of
                   the elements are internal updates (see the logic in OperationContainerImpl). Not
                   sure if this can happen. */
      /* According to the logic and comments in
      org.netbeans.modules.autoupdate.services.OperationContainerImpl, "at least first add must pass
      and respect type of operation"; as long as the OperationContainer is created with the correct
      type for the first UpdateElement, it is thereafter permissible to mix OperationType.INSTALL
      and OperationType.UPDATE operations. */
      container = element.getUpdateUnit().getInstalled() == null
          ? OperationContainer.createForInstall()
          : OperationContainer.createForUpdate();
    }
    OperationInfo<InstallSupport> oi = container.add(element);
    if (oi != null) {
      for (UpdateElement updateElement : oi.getRequiredElements())
        addUpdateElement(updateElement);
    }
  }

  /** @return is null when no further actions are necessary */
  public InstallSupport getInstallSupport() {
    return container == null ? null : container.getSupport();
  }
}
