// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project;

import java.awt.event.ActionListener;
import javax.annotation.Nullable;
import javax.swing.Icon;
import javax.swing.JComponent;
import org.openide.awt.Notification;
import org.openide.awt.NotificationDisplayer;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

/* Place the position before the expected delegate
(org.netbeans.modules.notifications.NotificationDisplayerImpl). */
@ServiceProvider(service = NotificationDisplayer.class, position = 50)
public class OpenDetailsNotificationDisplayer extends NotificationDisplayer {
  private @Nullable NotificationDisplayer getDelegate() {
    /* Assume lookupAll returns items in position order (NotificationDisplayer.getDefault() assumes
    the same for Lookup.getDefault().lookup(), whose Javadoc says it will return the "first"
    item). */
    for (NotificationDisplayer cand : Lookup.getDefault().lookupAll(NotificationDisplayer.class)) {
      if (!cand.getClass().equals(this.getClass()))
        return cand;
    }
    return null;
  }

  @Override
  public Notification notify(String title, Icon icon, String detailsText,
      ActionListener detailsAction, Priority priority)
  {
    final NotificationDisplayer delegate = getDelegate();
    if (delegate == null)
      return new TrivialNotification();
    openNotificationDetails();
    return delegate.notify(title, icon, detailsText, detailsAction, priority);
  }

  @Override
  public Notification notify(String title, Icon icon, JComponent balloonDetails,
      JComponent popupDetails, Priority priority)
  {
    final NotificationDisplayer delegate = getDelegate();
    if (delegate == null)
      return new TrivialNotification();
    openNotificationDetails();
    return delegate.notify(title, icon, balloonDetails, popupDetails, priority);
  }

  private void openNotificationDetails() {
    /* TODO[cc]: Move this class out of the Project module, then get rid of this check. For now I
                 wanted to avoid adding a window system dependency on the branding module. */
    if ("com/sieuferd/branding/com-sieuferd-branding-NoStatusBar.instance".equals(
        System.getProperty("netbeans.winsys.status_line.path")))
    {
      final TopComponent tc =
          WindowManager.getDefault().findTopComponent("NotificationCenterTopComponent");
      if (tc != null) {
        tc.open();
        tc.requestActive();
      }
    }
  }

  private static final class TrivialNotification extends Notification {
    @Override
    public void clear() {
      // No action.
    }
  }
}
