// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project;

import com.sieuferd.viewset.SieuferdProjectConfiguration;
import java.awt.Image;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.swing.Icon;
import org.netbeans.api.annotations.common.StaticResource;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectInformation;
import org.openide.util.ImageUtilities;

public class SieuferdProjectInformation implements ProjectInformation {
  @StaticResource
  private static final String PROJECT_ICON = "com/sieuferd/project/icons/sieuferdproject.png";
  private final PropertyChangeSupport pcs = new PropertyChangeSupport(this);
  private final SieuferdProject project;
  private final PropertyChangeListener configurationListener = (PropertyChangeEvent evt) -> {
    if (evt.getPropertyName().equals(SieuferdProjectConfiguration.PROP_PROJECT_NAME)) {
      pcs.firePropertyChange(ProjectInformation.PROP_DISPLAY_NAME,
          evt.getOldValue(), evt.getNewValue());
      pcs.firePropertyChange(ProjectInformation.PROP_NAME,
          createProgrammaticName((String) evt.getOldValue()),
          createProgrammaticName((String) evt.getNewValue()));
    }
  };

  public SieuferdProjectInformation(SieuferdProject project) {
    this.project = project;
  }

  private static String createProgrammaticName(String displayName) {
    final StringBuilder ret = new StringBuilder();
    for (int i = 0; i < displayName.length();) {
      final int charCount = Character.charCount(displayName.codePointAt(i));
      if (charCount == 1) {
        final char c = displayName.charAt(i);
        if (Character.isLetterOrDigit(c))
          ret.append(c);
      }
      i += charCount;
    }
    return ret.length() == 0 ? "_" : ret.toString();
  }

  @Override
  public String getName() {
    return createProgrammaticName(project.getConfiguration().getProjectName());
  }

  @Override
  public String getDisplayName() {
    return project.getConfiguration().getProjectName();
  }

  public static Image getIconImageStatic() {
    return ImageUtilities.loadImage(PROJECT_ICON, false);
  }

  public static Icon getIconStatic() {
    return ImageUtilities.loadImageIcon(PROJECT_ICON, false);
  }

  @Override
  public Icon getIcon() {
    return getIconStatic();
  }

  @Override
  public Project getProject() {
    return project;
  }

  @Override
  public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
    final boolean wasEmpty = pcs.getPropertyChangeListeners().length == 0;
    pcs.addPropertyChangeListener(listener);
    if (wasEmpty && pcs.getPropertyChangeListeners().length > 0)
      project.getConfiguration().addPropertyChangeListener(configurationListener);
  }

  @Override
  public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
    final boolean wasEmpty = pcs.getPropertyChangeListeners().length == 0;
    pcs.removePropertyChangeListener(listener);
    if (!wasEmpty && pcs.getPropertyChangeListeners().length == 0)
      project.getConfiguration().removePropertyChangeListener(configurationListener);
  }
}
