// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project;

import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import com.sieuferd.project.nodes.FieldNodePathFinder;
import com.sieuferd.project.nodes.NodeSettings;
import com.sieuferd.project.nodes.SieuferdProjectNode;
import org.netbeans.spi.project.ui.LogicalViewProvider;
import org.netbeans.spi.project.ui.PathFinder;
import org.openide.nodes.Node;

public class SieuferdProjectLogicalView implements LogicalViewProvider {
  private final SieuferdProject project;

  public SieuferdProjectLogicalView(SieuferdProject project) {
    this.project = project;
  }

  @Override
  public Node createLogicalView() {
    return new SieuferdProjectNode(project,
        new NodeSettings(AppliesTo.ROOT_ONLY, false, false, false, false, null, false));
  }

  @Override
  public Node findPath(Node root, Object target) {
    /* This method is called whenever the current ViewResultTopComponent is changed (due to
    Synchronize Editor with Views), though not every time a new cell selection is made. */

    if (FieldNodePathFinder.matchesDataOrFileObject(root, target))
      return root;

    // Based org.netbeans.modules.ant.freeform.ui.View.findPath()
    /* I use getNodes(true) here, but avoid infinite recursion into FieldNode hierarchies by only
    calling findPath recursively for nodes that do not define a PathFinder (FieldNode always
    does). */
    for (Node childNode : root.getChildren().getNodes(true)) {
      PathFinder childPathFinder = childNode.getLookup().lookup(PathFinder.class);
      if (childPathFinder != null) {
        Node ret = childPathFinder.findPath(childNode, target);
        if (ret != null)
          return ret;
      } else {
        Node ret = findPath(childNode, target);
        if (ret != null)
          return ret;
      }
    }
    return null;
  }
}
