// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project;

import com.google.common.base.Preconditions;
import com.sieuferd.datamodel.MutableView;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.ViewDefaults;
import com.sieuferd.datamodel.ViewListener;
import com.sieuferd.datamodel.ViewProperty;
import java.util.Set;
import javax.annotation.Nullable;

final class TransientView extends MutableView implements AutoCloseable {
  private volatile @Nullable MutableView delegate;

  TransientView(MutableView delegate) {
    Preconditions.checkNotNull(delegate);
    this.delegate = delegate;
  }

  @Override
  public void close() {
    delegate = null;
  }

  private MutableView getEffectiveDelegate() {
    MutableView ret = this.delegate;
    if (ret == null)
      throw new IllegalStateException("Transient view was invalidated");
    return ret;
  }

  @Override
  public void performWriteTransaction(Runnable runnable) {
    getEffectiveDelegate().performWriteTransaction(runnable);
  }

  @Override
  public <T> void set(ViewProperty<T> property, Path path, T value) throws IllegalArgumentException
  {
    getEffectiveDelegate().set(property, path, value);
  }

  @Override
  public void removePath(Path path) {
    getEffectiveDelegate().removePath(path);
  }

  @Override
  public void addPath(Path path) {
    getEffectiveDelegate().addPath(path);
  }

  @Override
  public void addViewListener(Path pathToListenTo, ViewListener listener) {
    throw new UnsupportedOperationException();
  }

  @Override
  public void removeViewListener(ViewListener listener) {
    throw new UnsupportedOperationException();
  }

  @Override
  public ViewDefaults getDefaults() {
    return getEffectiveDelegate().getDefaults();
  }

  @Override
  public boolean exists(Path path) {
    return getEffectiveDelegate().exists(path);
  }

  @Override
  public <T> T get(ViewProperty<T> property, Path path) {
    return getEffectiveDelegate().get(property, path);
  }

  @Override
  public Set<ViewProperty<?>> getDefinedProperties(Path path) {
    return getEffectiveDelegate().getDefinedProperties(path);
  }

  @Override
  public boolean defines(ViewProperty<?> property, Path path) {
    return getEffectiveDelegate().defines(property, path);
  }

  @Override
  public void performReadTransaction(Runnable runnable) {
    getEffectiveDelegate().performReadTransaction(runnable);
  }

  @Override
  public String toString() {
    return "TransientView(" + delegate.toString() + ")";
  }
}
