// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.actions;

import com.sieuferd.datamodel.MutableView;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import com.sieuferd.project.FieldContext;
import com.sieuferd.project.util.FieldSelection;
import com.sieuferd.query.Filter;
import com.sieuferd.query.QProp;
import com.sieuferd.query.Queries;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionRegistration;
import org.openide.util.NbBundle.Messages;

// TODO[cc]: Rename to ClearFilterAction (singular).
@ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.ClearFiltersAction")
@ActionRegistration(displayName = "#CTL_ClearFiltersAction")
@ActionReference(path = "Sieuferd/FieldContextMenu", position = 3300, separatorAfter = 3900)
@Messages("CTL_ClearFiltersAction=Clear Filter")
public final class ClearFiltersAction implements ActionListener {
  private final List<FieldContext> fieldContexts;

  public ClearFiltersAction(List<FieldContext> fieldContexts) {
    this.fieldContexts = fieldContexts;
  }

  @Override
  public void actionPerformed(ActionEvent e) {
    final FieldSelection fieldSelection =
        FieldSelection.fromFieldContexts(fieldContexts, AppliesTo.ANY);
    if (fieldSelection == null)
      return;
    fieldSelection.getViewContext().getView().mutate(Bundle.CTL_ClearFiltersAction(), true,
        (MutableView view) ->
    {
      for (Path path : fieldSelection.getSelectedPaths()) {
        view.set(QProp.FILTER, path, Filter.UNCONSTRAINED);
        if (path.isRelation()) {
          for (Path childPath : view.getChildPaths(path.getRelation())) {
            if (childPath.isRelation())
              view.set(QProp.FILTER, childPath, Filter.UNCONSTRAINED);
          }
        }
      }
      // Handle existential filters after first clearing all FILTER properties in question.
      for (Path path : fieldSelection.getSelectedPaths())
        Queries.propagateExistentialFilter(view, path, false);
    });
  }
}
