// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.actions;

import com.google.common.base.Preconditions;
import com.sieuferd.project.ViewContext;
import com.sieuferd.uiutil.ThreadUtil;
import com.sieuferd.viewset.SieuferdProjectXML;
import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.openide.awt.ActionID;
import org.openide.awt.ActionRegistration;
import org.openide.util.NbBundle.Messages;

@ActionID(
  category = "Sieuferd/RelationNodeContext",
  id = "com.sieuferd.project.nodes.CopyXMLAction"
)
@ActionRegistration(
  displayName = "#CTL_CopyXMLAction",
  asynchronous = true
)
@Messages("CTL_CopyXMLAction=Copy View XML")
public final class CopyXMLAction implements ActionListener {
  private static final Logger LOG = Logger.getLogger(CopyXMLAction.class.getName());
  private final ViewContext context;

  public CopyXMLAction(ViewContext context) {
    Preconditions.checkNotNull(context);
    this.context = context;
  }

  @Override
  public void actionPerformed(ActionEvent evt) {
    ThreadUtil.checkNotOnEDT();
    ByteArrayOutputStream os = new ByteArrayOutputStream();
    try {
      SieuferdProjectXML.writeSieuferdViewXML(
          context.getView(), context.getView().getDefaults(), os);
    } catch (IOException e) {
      LOG.log(Level.WARNING, "Failed to generate view XML", e);
      return;
    }
    StringSelection stringSelection;
    try {
      stringSelection = new StringSelection(os.toString("UTF-8"));
    } catch (UnsupportedEncodingException e) {
      throw new AssertionError(e);
    }
    Toolkit.getDefaultToolkit().getSystemClipboard().setContents(stringSelection, null);
  }
}
