// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.actions;

import com.google.common.collect.ImmutableList;
import com.sieuferd.datamodel.MutableView;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import com.sieuferd.project.FieldContext;
import com.sieuferd.project.ViewContext;
import com.sieuferd.project.util.FieldSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionRegistration;
import org.openide.util.NbBundle;
import org.openide.util.RequestProcessor;

@ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.DeleteFieldAction")
@ActionRegistration(displayName = "#CTL_DeleteFieldAction")
@ActionReference(path = "Sieuferd/FieldContextMenu", position = 5600)
@NbBundle.Messages("CTL_DeleteFieldAction=Delete")
public final class DeleteFieldAction implements ActionListener {
  private static final RequestProcessor RP = new RequestProcessor("Remove View", 1);
  private final ImmutableList<FieldContext> fieldContexts;

  public DeleteFieldAction(List<FieldContext> fieldContexts) {
    this.fieldContexts = ImmutableList.copyOf(fieldContexts);
  }

  public static String getDisplayName() {
    return Bundle.CTL_DeleteFieldAction();
  }

  @Override
  public void actionPerformed(ActionEvent e) {
    final FieldSelection fieldSelection =
        FieldSelection.fromFieldContexts(fieldContexts, AppliesTo.ANY);
    if (fieldSelection == null)
      return;
    final ViewContext viewContext = fieldSelection.getViewContext();
    if (fieldSelection.getSelectedPaths().contains(Path.ROOT)) {
      RP.post(() -> {
        viewContext.getProject().getViewSet().removeView(viewContext.getView());
      });
    } else {
      viewContext.getView().mutate(Bundle.CTL_DeleteFieldAction(), true, (MutableView view) -> {
        for (Path selectedPath : fieldSelection.getSelectedPaths()) {
          if (view.exists(selectedPath))
            view.removePath(selectedPath);
        }
      });
    }
  }
}
