// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.actions;

import com.google.common.base.Preconditions;
import com.sieuferd.datamodel.MutableView;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.Views;
import com.sieuferd.project.SieuferdProject;
import com.sieuferd.project.ViewContext;
import com.sieuferd.project.viewfile.VSProp;
import com.sieuferd.query.QProp;
import com.sieuferd.query.Queries;
import com.sieuferd.uiutil.ThreadUtil;
import com.sieuferd.viewset.GatedView;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Action;
import org.openide.awt.ActionID;
import org.openide.awt.ActionRegistration;
import org.openide.awt.Actions;
import org.openide.filesystems.FileObject;
import org.openide.util.ContextAwareAction;
import org.openide.util.Mutex;
import org.openide.util.NbBundle.Messages;
import org.openide.util.lookup.Lookups;

@ActionID(
    category = "Sieuferd/RelationNodeContext",
    id = "com.sieuferd.project.nodes.NewViewFromCopy"
)
@ActionRegistration(
    displayName = "#CTL_NewViewFromCopyAction",
    asynchronous = true
)
@Messages("CTL_NewViewFromCopyAction=Create Perspective")
public final class NewViewFromCopyAction implements ActionListener {
  private static final Logger LOG = Logger.getLogger(NewViewFromCopyAction.class.getName());
  private final ViewContext context;

  public NewViewFromCopyAction(ViewContext context) {
    Preconditions.checkNotNull(context);
    this.context = context;
  }

  @Override
  public void actionPerformed(ActionEvent evt) {
    ThreadUtil.checkNotOnEDT();
    final MutableView template = Views.newMutable(context.getView());
    if (template.get(QProp.PRIMARY, Path.ROOT))
      Queries.showHeadingPathsOnly(template, Path.ROOT);
    template.set(QProp.PRIMARY, Path.ROOT, false);
    final GatedView newView = context.getProject().getViewSet().createAndAddView(template);
    selectViewInProjects(context.getProject(), newView);
  }

  /** May be called from any thread. */
  private static void selectViewInProjects(final SieuferdProject project, final GatedView view) {
    Mutex.EVENT.writeAccess(new Runnable() {
      @Override
      @SuppressWarnings({"BroadCatchBlock", "TooBroadCatch"})
      public void run() {
        try {
          final FileObject fo = project.getViewSet().getFileObjectForViewIdentifier(
              view.get(VSProp.VIEW_IDENTIFIER, Path.ROOT));
          /* There doesn't seem to be a publicly available API for this; see the thread "How to get
          a ProjectNode for a given Project and select it in project view" on the platform-dev
          mailing list (started 2013-01-02). */
          Action selectNodeAction = ((ContextAwareAction) Actions.forID(
              "Window/SelectDocumentNode", "org.netbeans.modules.project.ui.SelectInProjects"))
              .createContextAwareInstance(Lookups.fixed(fo));
          // See http://wiki.netbeans.org/DevFaqInvokeActionProgrammatically .
          selectNodeAction.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_FIRST, ""));
        } catch (Exception e) {
          LOG.log(Level.WARNING, "Failed to select newly created view in Projects tab", e);
        }
      }
    });
  }
}
