// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.actions;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableSet;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import com.sieuferd.datamodel.Views;
import com.sieuferd.project.FieldContext;
import com.sieuferd.project.util.FieldSelection;
import com.sieuferd.query.Queries;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionRegistration;
import org.openide.util.NbBundle.Messages;

public abstract class SortAction implements ActionListener {
  private final ImmutableList<FieldContext> fieldContexts;
  private final boolean descending;
  private final boolean afterPrevious;

  public SortAction(List<FieldContext> fieldContexts, boolean descending, boolean afterPrevious) {
    this.fieldContexts = ImmutableList.copyOf(fieldContexts);
    this.descending = descending;
    this.afterPrevious = afterPrevious;
  }

  @Override
  public void actionPerformed(ActionEvent e) {
    final FieldSelection fieldSelection =
        FieldSelection.fromFieldContexts(fieldContexts, AppliesTo.ANY);
    if (fieldSelection == null)
      return;
    if (fieldSelection.getSelectedPaths().equals(ImmutableSet.of(Path.ROOT)))
      return;
    final String actionDisplayName = descending
        ? (afterPrevious ?
            Bundle.CTL_SortDescendingAfterPreviousAction() : Bundle.CTL_SortDescendingAction())
        : (afterPrevious ?
            Bundle.CTL_SortAscendingAfterPreviousAction() : Bundle.CTL_SortAscendingAction());
    fieldSelection.getViewContext().getView().mutate(actionDisplayName, true, view -> {
      boolean useAfterPrevious = afterPrevious;
      for (Path path : Views.getPathsInViewOrder(view, fieldSelection.getSelectedPaths())) {
        Queries.sort(view, path, descending, useAfterPrevious);
        useAfterPrevious = true;
      }
    });
  }

  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.SortAscendingAction")
  @ActionRegistration(displayName = "#CTL_SortAscendingAction",
      iconBase = "com/sieuferd/project/icons/badge_sort_asc.png")
  @ActionReference(path = "Sieuferd/FieldContextMenu", position = 2100, separatorBefore = 2000)
  /*
  @ActionReferences({
    @ActionReference(path = "Sieuferd/FieldContextMenu", position = 2100, separatorBefore = 2000),
    @ActionReference(path = "Menu/Edit", position = 100)
  })
  */
  // Mnemonic test:
  // @Messages("CTL_SortAscendingAction=Sort &Ascending")
  @Messages("CTL_SortAscendingAction=Sort Ascending")
  public static final class SortAscendingAction extends SortAction {
    public SortAscendingAction(List<FieldContext> fieldContexts) {
      super(fieldContexts, false, false);
    }
  }

  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.SortDescendingAction")
  @ActionRegistration(displayName = "#CTL_SortDescendingAction",
      iconBase = "com/sieuferd/project/icons/badge_sort_desc.png")
  @ActionReference(path = "Sieuferd/FieldContextMenu", position = 2200)
  @Messages("CTL_SortDescendingAction=Sort Descending")
  public static final class SortDescendingAction extends SortAction {
    public SortDescendingAction(List<FieldContext> fieldContexts) {
      super(fieldContexts, true, false);
    }
  }

  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.SortAscendingAfterPreviousAction")
  @ActionRegistration(displayName = "#CTL_SortAscendingAfterPreviousAction",
      iconBase = "com/sieuferd/project/icons/badge_sort_asc_sub2.png")
  @ActionReference(path = "Sieuferd/FieldContextMenu", position = 2300)
  @Messages("CTL_SortAscendingAfterPreviousAction=Sort Ascending after Previous")
  public static final class SortAscendingAfterPreviousAction extends SortAction {
    public SortAscendingAfterPreviousAction(List<FieldContext> fieldContexts) {
      super(fieldContexts, false, true);
    }
  }

  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.SortDescendingAfterPreviousAction")
  @ActionRegistration(displayName = "#CTL_SortDescendingAfterPreviousAction",
      iconBase = "com/sieuferd/project/icons/badge_sort_desc_sub2.png")
  @ActionReference(path = "Sieuferd/FieldContextMenu", position = 2400)
  @Messages("CTL_SortDescendingAfterPreviousAction=Sort Descending after Previous")
  public static final class SortDescendingAfterPreviousAction extends SortAction {
    public SortDescendingAfterPreviousAction(List<FieldContext> fieldContexts) {
      super(fieldContexts, true, true);
    }
  }
}
