// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.actions;

import com.google.common.base.Preconditions;
import java.awt.Component;
import java.awt.event.ActionEvent;
import javax.annotation.Nullable;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JToggleButton;
import javax.swing.JToggleButton.ToggleButtonModel;
import org.netbeans.api.annotations.common.StaticResource;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.util.ContextAwareAction;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.util.actions.Presenter;

@ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.ToggleSchemaOnly")
@ActionRegistration(
    lazy = false,
    displayName = "#CTL_ToggleSchemaOnlyAction")
@ActionReferences({
  @ActionReference(path = "Sieuferd/ViewResultToolbar", name = "sieuferd-toggle-schema-only",
      position = 900, separatorBefore = 850),
})
@NbBundle.Messages("CTL_ToggleSchemaOnlyAction=Toggle Schema-Only Layout")
public final class ToggleSchemaOnlyAction extends AbstractAction
    implements Presenter.Toolbar, ContextAwareAction
{
  @StaticResource
  private static final String ICON = "com/sieuferd/project/icons/toolbar_remeasure.png";
  private final @Nullable ToggleButtonModel model;

  // Create a subclass to make lookups more specific.
  public static class SchemaOnlyButtonModel extends ToggleButtonModel {
  }

  private ToggleSchemaOnlyAction(Lookup lookup) {
    super(Bundle.CTL_ToggleSchemaOnlyAction(), ImageUtilities.loadImageIcon(ICON, false));
    Preconditions.checkNotNull(lookup);
    /* TODO[pd]: The right thing to do would probably be to listen for a change in the lookup as
                 well. */
    this.model = lookup.lookup(SchemaOnlyButtonModel.class);
    setEnabled(model != null);
  }

  public ToggleSchemaOnlyAction() {
    this(Utilities.actionsGlobalContext());
  }

  @Override
  public void actionPerformed(ActionEvent e) {
    if (model != null)
      model.setSelected(!model.isSelected());
  }

  @Override
  public Component getToolbarPresenter() {
    JToggleButton ret = new JToggleButton();
    if (model != null)
      ret.setModel(model);
    ret.setEnabled(isEnabled());
    ret.setIcon((Icon) getValue(SMALL_ICON));
    return ret;
  }

  @Override
  public Action createContextAwareInstance(Lookup actionContext) {
    return new ToggleSchemaOnlyAction(actionContext);
  }
}
