// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.actions;

import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.util.NbBundle;

public final class ViewResultActions {
  private ViewResultActions() { }

  // TODO[cc]: To be thorough, localize the "Sieuferd" action category name.
  /* TODO[cc]: File a bug about the "This Layer" tree in the IDE not correctly showing the display
               name of actions created with the convention in the Add Action wizard, when the
               displayName is a bundle reference. */

  // The following actions are in the order they appear in the toolbar.

  /* ******************************************************************************************** */
  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.RefreshView")
  @ActionRegistration(
      iconBase = "com/sieuferd/project/icons/toolbar_requery.png",
      displayName = "#CTL_RefreshViewAction")
  @ActionReferences({
    @ActionReference(path = "Keymaps/NetBeans", name = "D-R"),
    @ActionReference(path = "Sieuferd/ViewResultToolbar", position = 100),
  })
  @NbBundle.Messages("CTL_RefreshViewAction=Refresh View")
  public static final String REFRESH_VIEW_ACTION_KEY = "sieuferd-refresh-view";

  /* ******************************************************************************************** */
  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.StopQuery")
  @ActionRegistration(
      iconBase = "com/sieuferd/project/icons/toolbar_stop.png",
      displayName = "#CTL_StopQueryAction")
  @ActionReference(path = "Sieuferd/ViewResultToolbar", position = 200)
  @NbBundle.Messages("CTL_StopQueryAction=Stop Query")
  public static final String STOP_QUERY_ACTION_KEY = "sieuferd-stop-query";

  /* ******************************************************************************************** */
  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.TableLayout")
  @ActionRegistration(
      iconBase = "com/sieuferd/project/icons/toolbar_layout_table.png",
      displayName = "#CTL_TableLayoutAction")
  @ActionReference(path = "Sieuferd/ViewResultToolbar", position = 300, separatorBefore = 250)
  @NbBundle.Messages("CTL_TableLayoutAction=Table Layout")
  public static final String TABLE_LAYOUT_ACTION_KEY = "sieuferd-table-layout";

  /* ******************************************************************************************** */
  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.OutlineLayout")
  @ActionRegistration(
      iconBase = "com/sieuferd/project/icons/toolbar_layout_report.png",
      displayName = "#CTL_OutlineLayoutAction")
  @ActionReference(path = "Sieuferd/ViewResultToolbar", position = 400)
  @NbBundle.Messages("CTL_OutlineLayoutAction=Outline Layout")
  public static final String OUTLINE_LAYOUT_ACTION_KEY = "sieuferd-outline-layout";

  /* ******************************************************************************************** */
  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.Remeasure")
  @ActionRegistration(
      iconBase = "com/sieuferd/project/icons/toolbar_remeasure.png",
      displayName = "#CTL_RemeasureAction")
  @ActionReference(path = "Sieuferd/ViewResultToolbar", position = 500)
  @NbBundle.Messages("CTL_RemeasureAction=Remeasure Field Statistics")
  public static final String REMEASURE_ACTION_KEY = "sieuferd-remeasure";

  /* ******************************************************************************************** */
  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.OriginalSize")
  @ActionRegistration(
      iconBase = "com/sieuferd/project/icons/original_size2.png",
      displayName = "#CTL_OriginalSizeAction")
  @ActionReferences({
    @ActionReference(path = "Keymaps/NetBeans", name = "D-0"),
    @ActionReference(path = "Sieuferd/ViewResultToolbar", position = 600, separatorBefore = 550)
  })
  @NbBundle.Messages("CTL_OriginalSizeAction=Original Size")
  public static final String ORIGINAL_SIZE_ACTION_KEY = "sieuferd-original-size";

  /* ******************************************************************************************** */
  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.ZoomIn")
  @ActionRegistration(
      iconBase = "com/sieuferd/project/icons/zoom_in.png",
      displayName = "#CTL_ZoomInAction")
  @ActionReferences({
    @ActionReference(path = "Keymaps/NetBeans", name = "D-EQUALS"),
    @ActionReference(path = "Keymaps/NetBeans", name = "D-PLUS"),
    @ActionReference(path = "Sieuferd/ViewResultToolbar", position = 700)
  })
  @NbBundle.Messages("CTL_ZoomInAction=Zoom In")
  public static final String ZOOM_IN_ACTION_KEY = "sieuferd-zoom-in";

  /* ******************************************************************************************** */
  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.ZoomOut")
  @ActionRegistration(
      iconBase = "com/sieuferd/project/icons/zoom_out.png",
      displayName = "#CTL_ZoomOutAction")
  @ActionReferences({
    @ActionReference(path = "Keymaps/NetBeans", name = "D-MINUS"),
    @ActionReference(path = "Sieuferd/ViewResultToolbar", position = 800)
  })
  @NbBundle.Messages("CTL_ZoomOutAction=Zoom Out")
  public static final String ZOOM_OUT_ACTION_KEY = "sieuferd-zoom-out";

  /* ******************************************************************************************** */

  // ToggleSchemaOnlyAction goes here; see separate file.

  /* ******************************************************************************************** */

  // Undo/Redo shadows go here; see layer.xml.

  /* ******************************************************************************************** */

  // Actions not currently represented in the toolbar.

  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.EditCell")
  @ActionRegistration(
      displayName = "#CTL_EditCellAction")
  @ActionReferences({
    // The standard shortcut on Excel for Windows (accept regardless of OS).
    @ActionReference(path = "Keymaps/NetBeans", name = "F2"),
    // The standard shortcut on Excel for Mac (accept regardless of OS).
    @ActionReference(path = "Keymaps/NetBeans", name = "C-U")
  })
  @NbBundle.Messages("CTL_EditCellAction=Edit Cell")
  public static final String EDIT_CELL = "sieuferd-edit-cell";
}
