// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.actions.join;

import com.google.common.collect.ImmutableMap;
import com.sieuferd.project.actions.join.ArrowWidgetModel.Arrow;
import com.sieuferd.uiutil.ThreadUtil;
import com.sieuferd.uiutil.UIUtil;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class ArrowWidget extends JComponent {
  private final Color selectedColor;
  private final Color unselectedColor;

  public ArrowWidget() {
    final Color toSelectedColor = UIManager.getColor("List.selectionBackground");
    final Color toUnselectedColor = UIManager.getColor("controlShadow");
    selectedColor = (toSelectedColor == null) ? Color.BLUE : toSelectedColor;
    unselectedColor = (toUnselectedColor == null) ? Color.BLACK : toUnselectedColor;
  }

  private ArrowWidgetModel model;
  private final ChangeListener changeListener = (ChangeEvent e) -> {
    ThreadUtil.checkOnEDT();
    repaint();
  };

  public void setModel(ArrowWidgetModel model) {
    if (this.model != null)
      this.model.removeChangeListener(changeListener);
    this.model = model;
    if (this.model != null)
      this.model.addChangeListener(changeListener);
  }

  @Override
  protected void paintComponent(Graphics g0) {
    if (model == null)
      return;
    Graphics2D g = UIUtil.createGraphicsWithRenderingHintsConfigured(g0, ImmutableMap.of());
    try {
      paintHelper(g);
    } finally {
      g.dispose();
    }
  }

  private static final int STROKE = 3;
  private static final int STRAIGHT_WIDTH = 10;
  private static final boolean CAP = false;

  private void paintHelper(Graphics2D g) {
    for (Arrow arrow : model.getArrows()) {
      final int effLeftY;
      final int effRightY;
      if (CAP) {
        effLeftY = Math.max(1, Math.min(getHeight() + 1 - STROKE, arrow.leftY));
        effRightY = Math.max(1, Math.min(getHeight() + 1 - STROKE, arrow.rightY));
      } else {
        effLeftY = arrow.leftY;
        effRightY = arrow.rightY;
      }
      g.setColor(arrow.selected ? selectedColor : unselectedColor);
      g.setStroke(new BasicStroke(STROKE));
      g.drawPolyline(
          new int[] {0       , STRAIGHT_WIDTH, getWidth() - STRAIGHT_WIDTH, getWidth()  },
          new int[] {effLeftY, effLeftY      , effRightY                  , effRightY   },
          4
        );
    }
  }
}
