// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.datasource;

import java.awt.Dimension;
import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import com.sieuferd.project.datasource.DatabaseTableModel.DatabaseTableColumn;
import com.sieuferd.uiutil.UIUtil;

public final class DatabaseTableUtil {
  private static final int EXTRA_WIDTH_PX = 20;

  private DatabaseTableUtil() { }

  public static DatabaseTableModel configureDatabaseTable(final JTable table) {
    // See org.netbeans.modules.maven.api.customizer.support.SelectedItemsTable .

    final int tableCountColumnWidth = Math.max(
        table.getFontMetrics(table.getTableHeader().getFont())
            .stringWidth(Bundle.MSG_DatabaseTableColumn_TableCount()),
        table.getFontMetrics(table.getFont()).stringWidth("00000"));
    final DatabaseTableModel model = new DatabaseTableModel(tableCountColumnWidth);
    // Set model before making width adjustments, otherwise the adjustments are lost.
    table.setModel(model);
    table.getSelectionModel().setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    table.setShowGrid(false);
    table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
    table.getColumnModel().getColumn(DatabaseTableColumn.MANAGED.ordinal())
        .setMaxWidth(new JCheckBox().getPreferredSize().width);
    table.getColumnModel().getColumn(DatabaseTableColumn.TABLE_COUNT.ordinal())
        .setMaxWidth(tableCountColumnWidth + EXTRA_WIDTH_PX);
    table.getColumnModel().getColumn(DatabaseTableColumn.TABLE_COUNT.ordinal())
        .setMinWidth(tableCountColumnWidth + EXTRA_WIDTH_PX);
    table.setRowHeight(
        table.getFontMetrics(table.getFont()).getHeight() + (2 * table.getRowMargin()));
    table.getTableHeader().setPreferredSize(new Dimension(
        table.getTableHeader().getPreferredSize().width,
        table.getFontMetrics(table.getTableHeader().getFont()).getHeight() +
            (2 * table.getRowMargin())));
    UIUtil.configureBooleanTableValueShortcuts(table, DatabaseTableColumn.MANAGED.ordinal());
    return model;
  }
}
