// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.datasource;

import com.google.common.base.Preconditions;
import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import com.sieuferd.project.SieuferdProject;
import com.sieuferd.viewset.SieuferdProjectConfiguration;
import org.netbeans.api.project.ProjectManager;
import org.netbeans.api.project.ProjectUtils;
import org.netbeans.spi.project.ui.CustomizerProvider;
import org.netbeans.spi.project.ui.support.ProjectCustomizer;
import org.openide.util.lookup.Lookups;

public class SieuferdProjectCustomizerProvider implements CustomizerProvider {
  public static final String CUSTOMIZER_FOLDER_PATH = "Projects/com-sieuferd-project/Customizer";

  private final SieuferdProject project;

  public SieuferdProjectCustomizerProvider(SieuferdProject project) {
    Preconditions.checkNotNull(project);
    this.project = project;
  }

  @Override
  public void showCustomizer() {
    final SieuferdProjectConfiguration configurationCopy = new SieuferdProjectConfiguration();
    ProjectManager.mutex().readAccess(() -> {
      configurationCopy.copyFrom(project.getConfiguration());
    });
    Dialog dialog = ProjectCustomizer.createCustomizerDialog(
        CUSTOMIZER_FOLDER_PATH,
        /* The project system requires the project itself to be in the lookup here, even if we
        aren't using it ourselves. */
        Lookups.fixed(project, configurationCopy),
        null, // Preselected category
        new OKOptionListener(configurationCopy),
        null // Can be null
      );
    dialog.setTitle(ProjectUtils.getInformation(project).getDisplayName());
    dialog.setVisible(true);
  }

  private class OKOptionListener implements ActionListener {
    private final SieuferdProjectConfiguration configurationCopy;

    OKOptionListener(SieuferdProjectConfiguration configurationCopy) {
      this.configurationCopy = configurationCopy;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
      ProjectManager.mutex().writeAccess(() -> {
        project.getConfiguration().copyFrom(configurationCopy);
      });
      // TODO[pd]: Store the updated configuration.
    }
  }
}
