// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.formulabar;

import com.google.common.base.Preconditions;
import com.sieuferd.datamodel.MutableView;
import com.sieuferd.datamodel.Strepco;
import com.sieuferd.project.ViewContext;
import com.sieuferd.util.ObjectReference;
import com.sieuferd.widget.InteractionProvider.Acceptance;
import javax.annotation.concurrent.Immutable;

/**
 * The minimal relevant state required to load and store values during cursor editing for a
 * particular {@link CursorEditTarget}. Should be comparable using {@code equals}/{@code hashCode}
 * so that the {@link CursorEditManager} can void disturbing a cursor edit operation when the
 * {@code CursorEditTarget} has changed but not the subset of the state relevant to the
 * {@code CursorEditModel}.
 */
@Immutable
public abstract class CursorEditKey {
  private final CursorEditType type;
  private final boolean readOnly;

  public CursorEditKey(CursorEditType type, boolean readOnly) {
    Preconditions.checkNotNull(type);
    this.type = type;
    this.readOnly = readOnly;
  }

  public final CursorEditType getType() {
    return type;
  }

  public final boolean isReadOnly() {
    return readOnly;
  }

  /**
   * Attempt to set the value from the string in the formula bar. If the value is a view property,
   * the update should be made on the {@code GatedView} found via
   * {@code CursorEditTarget.getViewContext().getView()}. Before performing the actual update, the
   * view transaction should check that relevant paths still exist using
   * {@link #checkPathsExist(CursorEditTarget, MutableView)}.
   *
   * <p>This method will not be called if {@link #isReadOnly()} returns true.
   *
   * @param strepco the value of {@link CursorEditTarget#getStrepco()} for the target that was
   *        passed to {@link CursorEditType#createModel(CursorEditTarget,ObjectReference)} when this
   *        {@code CursorEditModel} was created.
   */
  public Acceptance setValue(ViewContext viewContext, Strepco strepco, String newValueString) {
    if (isReadOnly())
      throw new AssertionError();
    throw new UnsupportedOperationException();
  }
}
