// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.formulabar;

import com.google.common.base.Preconditions;
import com.sieuferd.cell.support.Location;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.View;
import com.sieuferd.datamodel.ViewResult;
import com.sieuferd.project.ViewContext;
import com.sieuferd.datamodel.Strepco;

/**
 * A cursor location along with any associated state that may be needed to manage editing of the
 * content under the cursor.
 *
 * Shallowly immutable.
 */
public final class CursorEditTarget {
  private final ViewContext viewContext;
  private final Location cursorLocation;
  private final ViewResult result;
  private final Strepco strepco;

  public CursorEditTarget(ViewContext viewContext, Location cursorLocation, ViewResult result) {
    Preconditions.checkNotNull(viewContext);
    Preconditions.checkNotNull(result);
    Preconditions.checkNotNull(cursorLocation);
    Preconditions.checkArgument(pathsExist(result.getView(), cursorLocation));
    this.viewContext = viewContext;
    this.cursorLocation = cursorLocation;
    this.result = result;
    this.strepco = Strepco.create(getResult().getView(), getCursorLocation().getPath());
  }

  @Override
  public String toString() {
    return "CursorEditTarget(" + cursorLocation +
        ", {" + result.getData().getCompleteTupleCount(Path.ROOT) + " complete tuples in root})";
  }

  public static boolean pathsExist(View view, Location cursorLocation) {
    return view.exists(cursorLocation.getPath()) &&
        (cursorLocation.getIdentifyingPath() == null ||
        view.exists(cursorLocation.getIdentifyingPath()));
  }

  public ViewContext getViewContext() {
    return viewContext;
  }

  public Strepco getStrepco() {
    return strepco;
  }

  /**
   * Get the {@code CursorLocation}. Both {@code CursorLocation.getPath()} and
   * {@code CursorLocation.getIdentifyingPath()}, if not null, are guaranteed to exist
   * in the view found via {@code getResult().getView()}.
   */
  public Location getCursorLocation() {
    return cursorLocation;
  }

  public ViewResult getResult() {
    return result;
  }

  public boolean isValue() {
    return cursorLocation.isValue();
  }
}
