// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.formulabar;

import com.google.common.base.Preconditions;
import com.sieuferd.util.ObjectReference;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;
import javax.swing.Icon;
import org.netbeans.api.annotations.common.StaticResource;

@Immutable
public abstract class CursorEditType {
  private static final String ICON_FOLDER = "com/sieuferd/project/icons/";
  @StaticResource
  protected static final String ICON_VALUE = ICON_FOLDER + "cursoredit_value.png";
  @StaticResource
  protected static final String ICON_FORMULA = ICON_FOLDER + "cursoredit_formula.png";
  @StaticResource
  protected static final String ICON_LABEL = ICON_FOLDER + "cursoredit_label.png";
  @StaticResource
  protected static final String ICON_NONE = ICON_FOLDER + "cursoredit_none.png";

  private final Icon icon;
  private final String mimeType;
  private final boolean usePlainEditor;

  public CursorEditType(Icon icon, String mimeType, boolean usePlainEditor) {
    Preconditions.checkNotNull(icon);
    Preconditions.checkNotNull(mimeType);
    this.icon = icon;
    this.mimeType = mimeType;
    this.usePlainEditor = usePlainEditor;
  }

  public final Icon getIcon() {
    return icon;
  }

  public final String getMimeType() {
    return mimeType;
  }

  // TODO[cc]: Consider having the CursorEditType generate the appropriate EditorKit object instead.
  public boolean getUsePlainEditor() {
    return usePlainEditor;
  }

  /**
   * Create a {@code CursorEditModel} object for this content type if this content type is
   * applicable to the specified cursor location, and compute the original string representation of
   * the value.
   *
   * @param originalValueString populated iff the return value is not null
   * @return null if the target's cursor location is not applicable to this edit type
   */
  public abstract @Nullable CursorEditKey createEditKey(
      CursorEditTarget target, ObjectReference<String> originalValueString);
}
