// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.formulabar;

import javax.annotation.Nullable;
import org.netbeans.api.lexer.Token;
import org.netbeans.spi.lexer.Lexer;
import org.netbeans.spi.lexer.LexerInput;
import org.netbeans.spi.lexer.LexerRestartInfo;
import org.netbeans.spi.lexer.TokenFactory;

// TODO[te]: Run the tests mentioned in the Javadoc for Lexer.
public class DataValueLexer implements Lexer<DataValueTokenID> {
  private final LexerInput input;
  private final TokenFactory<DataValueTokenID> factory;

  public DataValueLexer(LexerRestartInfo<DataValueTokenID> info) {
    this.input = info.input();
    this.factory = info.tokenFactory();
  }

  @Override
  public @Nullable Token<DataValueTokenID> nextToken() {
    /* In o.n.editor.ext.plain.PlainSyntax, each line is lexed as a single token. An even simpler
    approach would be to give each character its own token, but at least once I have seen
    significant performance degradation from doing this, so tokenize line by line instead. See
    ebakke's comments on https://netbeans.org/bugzilla/show_bug.cgi?id=237724 . Note that
    PlainSyntax appears to implement an older SPI (o.n.editor.Syntax) that requires more careful
    state-keeping than we need here. */
    int c;
    while ((c = input.read()) != LexerInput.EOF) {
      if (c == '\n')
        break;
    }
    return (input.readLength() == 0) ? null : factory.createToken(DataValueTokenID.DEFAULT);
  }

  @Override
  public Object state() {
    return null;
  }

  @Override
  public void release() {
    // No action.
  }
}
