// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.formulabar;

import com.google.common.base.Preconditions;
import org.netbeans.api.lexer.TokenId;

/**
 * Expression token IDs for tokens produced by {@link ExpressionLexer} for the NetBeans-based editor
 * features. Most of the IDs correspond directly to a single
 * {@link com.sieuferd.datamodel.expressions.ExpressionTokenType}, with the exception of the path
 * reference tokens, which are further subdivided on the NetBeans Lexer end than by the ANTLR lexer.
 */
public enum ExpressionTokenId implements TokenId {
  /* Keep the categories in sync with com/sieuferd/project/formulabar/TokenBundle.properties and
  expression-token-colorings.xml; ExpressionHighlightingTest.testTokenCategories should detect any
  mismatch between these. */
  IDENTIFIER("identifier"),
  PATH_REFERENCE_CORE("path-reference"),
  PATH_REFERENCE_OPEN_BRACKET("separator"),
  PATH_REFERENCE_CLOSE_BRACKET("separator"),
  ARGUMENT_SEPARATOR("separator"),
  PARENTHESIS_LEFT("separator"),
  PARENTHESIS_RIGHT("separator"),
  LITERAL_NUMBER("literal-number"),
  LITERAL_STRING("literal-string"),
  LITERAL_TEMPORAL("literal-temporal"),
  LITERAL_BOOLEAN("literal-boolean"),
  LITERAL_ERROR("literal-error"),
  LITERAL_NULL("literal-null"),
  OPERATOR("operator"),
  WHITESPACE("whitespace"),
  OTHER("default");

  private final String primaryCategory;

  ExpressionTokenId(String primaryCategory) {
    Preconditions.checkNotNull(primaryCategory);
    this.primaryCategory = primaryCategory;
  }

  @Override
  public String primaryCategory() {
    return primaryCategory;
  }
}
