// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.formulabar;

import com.google.common.collect.Maps;
import com.sieuferd.uiutil.api.KeyStrokeForwardable;
import com.sieuferd.upstream.localeditor.LocalEditor;
import com.sieuferd.util.MapUtil;
import com.sieuferd.widget.CellWidget;
import com.sieuferd.widget.CursorEditInputMap;
import java.awt.CardLayout;
import java.util.Map;
import javax.annotation.Nullable;
import javax.swing.JEditorPane;
import javax.swing.JPanel;

public final class FormulaBar extends JPanel implements CursorEditorProvider {
  private final Map<CursorEditType,EditorCard> editorCards = Maps.newLinkedHashMap();
  private final CardLayout cardLayout;
  private @Nullable EditorCard currentEditorCard;
  private int visibleRows = 1;
  private @Nullable KeyStrokeForwardable keyStrokeForwardable;

  private static class EditorCard {
    final String cardName;
    final LocalEditor localEditor;

    public EditorCard(int id) {
      this.cardName = "editor" + id;
      this.localEditor = new LocalEditor();
    }
  }

  public FormulaBar() {
    initComponents();
    cardLayout = (CardLayout) editorCardsPanel.getLayout();
    updateEditorProperties();
  }

  @Override
  public JEditorPane update(CursorEditType type) {
    currentEditorCard = editorCards.get(type);
    if (currentEditorCard == null) {
      currentEditorCard = new EditorCard(editorCards.size());
      MapUtil.putChecked(editorCards, type, currentEditorCard);
      editorCardsPanel.add(currentEditorCard.localEditor, currentEditorCard.cardName);
    }
    cardLayout.show(editorCardsPanel, currentEditorCard.cardName);
    updateEditorProperties();
    label.setIcon(type.getIcon());
    label.setLabelFor(currentEditorCard.localEditor);
    return currentEditorCard.localEditor.getEditorPane();
  }

  public void setExtendedInputMode(CellWidget cellWidget, boolean value) {
    for (EditorCard editorCard : editorCards.values()) {
      CursorEditInputMap.configureCursorEditor(cellWidget,
          editorCard.localEditor.getEditorPane(), true);
    }
  }

  public void setKeyStrokeForwardable(@Nullable KeyStrokeForwardable keyStrokeForwardable) {
    if (this.keyStrokeForwardable != keyStrokeForwardable) {
      this.keyStrokeForwardable = keyStrokeForwardable;
      updateEditorProperties();
    }
  }

  public @Nullable KeyStrokeForwardable getKeyStrokeForwardable() {
    return keyStrokeForwardable;
  }

  public void setVisibleRows(int visibleRows) {
    if (this.visibleRows != visibleRows) {
      this.visibleRows = visibleRows;
      updateEditorProperties();
    }
  }

  public int getVisibleRows() {
    return this.visibleRows;
  }

  private void updateEditorProperties() {
    for (EditorCard editorCard : editorCards.values()) {
      editorCard.localEditor.setVisibleRows(visibleRows);
      editorCard.localEditor.setWrapping(visibleRows != 1);
      editorCard.localEditor.setKeyStrokeForwardable(
          editorCard != currentEditorCard ? null : keyStrokeForwardable);
    }
  }

  @SuppressWarnings("unchecked")
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {
    java.awt.GridBagConstraints gridBagConstraints;

    label = new javax.swing.JLabel();
    editorCardsPanel = new javax.swing.JPanel();

    setOpaque(false);
    setLayout(new java.awt.GridBagLayout());

    label.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/sieuferd/project/icons/cursoredit_none.png"))); // NOI18N
    label.setVerticalAlignment(javax.swing.SwingConstants.TOP);
    label.addMouseListener(new java.awt.event.MouseAdapter() {
      public void mouseClicked(java.awt.event.MouseEvent evt) {
        labelMouseClicked(evt);
      }
    });
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 8);
    add(label, gridBagConstraints);

    editorCardsPanel.setOpaque(false);
    editorCardsPanel.setLayout(new java.awt.CardLayout());
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 1;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints.weightx = 1.0;
    gridBagConstraints.weighty = 1.0;
    add(editorCardsPanel, gridBagConstraints);
  }// </editor-fold>//GEN-END:initComponents

  private void labelMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_labelMouseClicked
    setVisibleRows(getVisibleRows() == 1 ? 5 : 1);
  }//GEN-LAST:event_labelMouseClicked

  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JPanel editorCardsPanel;
  private javax.swing.JLabel label;
  // End of variables declaration//GEN-END:variables
}
