// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.formulabar;

import com.sieuferd.uiutil.ThreadUtil;
import com.sieuferd.uiutil.UIUtil;
import com.sieuferd.upstream.localeditor.LocalEditor;
import com.sieuferd.widget.InplaceEditor;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.util.logging.Logger;
import javax.annotation.Nullable;
import javax.swing.JEditorPane;
import javax.swing.KeyStroke;
import javax.swing.border.EmptyBorder;
import javax.swing.text.JTextComponent;

// TODO[cc]: Consider moving to com.sieuferd.project.result and making package-private.
public final class InplaceEditorImpl implements InplaceEditor {
  private static final Logger LOG = Logger.getLogger(InplaceEditorImpl.class.getName());
  private final LocalEditor localEditor = new LocalEditor(new EmptyBorder(0, 0, 0, 0));

  public InplaceEditorImpl() {
    localEditor.setWrapping(false);
    localEditor.setVisibleRows(null);
  }

  @Override
  public LocalEditor getComponent() {
    return localEditor;
  }

  @Override
  public JTextComponent getFocusComponent() {
    return localEditor.getEditorPane();
  }

  /** Get the editor pane for this inplace editor. Will always return the same object. */
  public JEditorPane getEditorPane() {
    return localEditor.getEditorPane();
  }

  @Override
  public boolean forwardUnhandledKeyStroke(KeyStroke ks, KeyEvent evt, boolean pressed) {
    return localEditor.forwardUnhandledKeyStroke(ks, evt, pressed);
  }

  @Override
  public boolean shouldStartEditing(@Nullable KeyEvent evt) {
    final boolean editable = localEditor.getEditorPane().isEditable();
    if (evt == null) {
      return editable;
    } else if (UIUtil.isCharacterInsertionEvent(evt)) {
      if (!editable) {
        Toolkit.getDefaultToolkit().beep();
      } else {
        return true;
      }
    }
    return false;
  }

  @Override
  public void prepare(boolean clear) {
    ThreadUtil.checkOnEDT();
    if (clear) {
      UIUtil.setDocumentText(localEditor.getEditorPane().getDocument(), "");
    } else {
      /* Put the cursor at the end with no selection rather than selecting all the existing text;
      this is what Excel does when edit mode is entered via a keyboard shortcut (F2 on Windows). */
      localEditor.getEditorPane().setCaretPosition(
          localEditor.getEditorPane().getDocument().getLength());
    }
  }

  @Override
  public void setActive(boolean active) {
    // No action.
  }
}
