// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.formulabar.types;

import com.google.common.base.Preconditions;
import com.sieuferd.datamodel.CProp;
import com.sieuferd.datamodel.MutableView;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.Strepco;
import com.sieuferd.project.ViewContext;
import com.sieuferd.project.derivatives.ViewUpdater;
import com.sieuferd.project.formulabar.DataValueLanguageConfig;
import com.sieuferd.project.formulabar.CursorEditKey;
import com.sieuferd.project.formulabar.CursorEditType;
import com.sieuferd.project.formulabar.CursorEditTarget;
import com.sieuferd.util.ObjectReference;
import com.sieuferd.widget.InteractionProvider.Acceptance;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle.Messages;
import org.openide.util.lookup.ServiceProvider;

@ServiceProvider(service = CursorEditType.class)
public final class LabelType extends CursorEditType {
  public LabelType() {
    super(ImageUtilities.loadImageIcon(ICON_LABEL, false), DataValueLanguageConfig.MIME_TYPE, true);
  }

  public static String getActionDisplayName() {
    return Bundle.CTL_EditLabel();
  }

  @Messages("CTL_EditLabel=Edit Label")
  @Override
  public CursorEditKey createEditKey(
      CursorEditTarget target, ObjectReference<String> originalValueString)
  {
    if (target.isValue())
      return null;
    final Path path = target.getCursorLocation().getPath();
    originalValueString.setValue(target.getResult().getView().get(CProp.LABEL, path));
    return new LabelEditKey(path);
  }

  private final class LabelEditKey extends CursorEditKey {
    private final Path path;

    public LabelEditKey(Path path) {
      super(LabelType.this, false);
      Preconditions.checkNotNull(path);
      this.path = path;
    }

    @Override
    public Acceptance setValue(ViewContext viewContext, Strepco strepco, String newValueString) {
      viewContext.getView().mutate(Bundle.CTL_EditLabel(), true, (MutableView view) ->
      {
        if (view.exists(path))
          ViewUpdater.userSetProperty(view, CProp.LABEL, path, newValueString);
      });
      /* Always accept, even if the label could not be set because the path no longer exists; this
      seems to be a reasonable thing to do if the path was actually deleted. */
      return Acceptance.ACCEPTED_IMMEDIATELY;
    }

    @Override
    public int hashCode() {
      return path.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
      if (!(obj instanceof LabelEditKey))
        return false;
      final LabelEditKey other = (LabelEditKey) obj;
      return this.path.equals(other.path);
    }
  }
}
