// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.formulabar.types;

import com.google.common.base.Preconditions;
import com.sieuferd.cell.support.Location;
import com.sieuferd.datamodel.Data;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.Path.PrimitivePath;
import com.sieuferd.datamodel.PrimitiveValue;
import com.sieuferd.datamodel.Tuple;
import com.sieuferd.layout.PrimitiveValueFormatter;
import com.sieuferd.project.formulabar.DataValueLanguageConfig;
import com.sieuferd.project.formulabar.CursorEditKey;
import com.sieuferd.project.formulabar.CursorEditType;
import com.sieuferd.project.formulabar.CursorEditTarget;
import com.sieuferd.query.QProp;
import com.sieuferd.util.ObjectReference;
import java.util.Objects;
import javax.annotation.Nullable;
import org.openide.util.ImageUtilities;
import org.openide.util.lookup.ServiceProvider;

@ServiceProvider(service = CursorEditType.class)
public final class ValueType extends CursorEditType {
  public ValueType() {
    super(ImageUtilities.loadImageIcon(ICON_VALUE, false), DataValueLanguageConfig.MIME_TYPE, true);
  }

  @Override
  public CursorEditKey createEditKey(
      CursorEditTarget target, ObjectReference<String> originalValueString)
  {
    final Location cursorLocation = target.getCursorLocation();
    final Path path = cursorLocation.getPath();
    final @Nullable Tuple identifyingTuple = cursorLocation.getIdentifyingTuple();
    final Data targetData = target.getResult().getData();
    if (!target.isValue() ||
        path.isRelation() ||
        target.getResult().getView().get(QProp.SQL_COLUMN_NAME, path).isEmpty() ||
        !target.getResult().getData().isProjected(path) ||
        cursorLocation.getIdentifyingPath() == null ||
        !targetData.equals(cursorLocation.getData()))
    {
      return null;
    }
    final PrimitiveValue content = target.getResult().getData().getTupleField(
        identifyingTuple, path.getPrimitive());
    originalValueString.setValue(getStringRepresentation(content));
    return new ValueEditKey(identifyingTuple, path.getPrimitive());
  }

  private static String getStringRepresentation(PrimitiveValue value) {
    final @Nullable Object content = value.getContent();
    if (content == null)
      return "(null)";
    {
      String ret = PrimitiveValueFormatter.formatIfArray(content);
      if (ret != null)
        return ret;
    }
    return content.toString();
  }

  private final class ValueEditKey extends CursorEditKey {
    private final Tuple identifyingTuple;
    private final PrimitivePath path;

    public ValueEditKey(Tuple identifyingTuple, PrimitivePath path) {
      super(ValueType.this, true);
      Preconditions.checkNotNull(identifyingTuple);
      Preconditions.checkNotNull(path);
      this.identifyingTuple = identifyingTuple;
      this.path = path;
    }

    @Override
    public int hashCode() {
      return Objects.hash(identifyingTuple, path);
    }

    @Override
    public boolean equals(Object obj) {
      if (!(obj instanceof ValueEditKey))
        return false;
      final ValueEditKey other = (ValueEditKey) obj;
      return this.identifyingTuple.equals(other.identifyingTuple)
          && this.path.equals(other.path);
    }
  }
}
