// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.navigator;

import com.google.common.base.Preconditions;
import com.sieuferd.datamodel.ImmutableView;
import com.sieuferd.datamodel.MutableView;
import com.sieuferd.project.FieldContext;
import com.sieuferd.project.actions.MoveAction;
import com.sieuferd.uiutil.EnterFilterInputMap;
import com.sieuferd.uiutil.ThreadUtil;
import com.sieuferd.util.ObjectReference;
import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.util.Set;
import javax.annotation.Nullable;
import javax.swing.JPanel;
import org.openide.DialogDescriptor;
import org.openide.DialogDisplayer;
import org.openide.NotifyDescriptor;
import org.openide.util.Mutex;
import org.openide.util.NbBundle.Messages;

// TODO[cc]: Consider getting rid of this.
public class FieldsDialogPanel extends JPanel {
  @SuppressWarnings("LeakingThisInConstructor")
  public FieldsDialogPanel() {
    initComponents();
    MoveAction.connectMoveAction(this, fieldsPanel.getLookup(), moveUpButton, false);
    MoveAction.connectMoveAction(this, fieldsPanel.getLookup(), moveDownButton, true);
    EnterFilterInputMap.configure(fieldsPanel.getOutlineView().getOutline());
  }

  public void setFieldContexts(final Set<FieldContext> fieldContexts) {
    ThreadUtil.checkOnEDT();
    Preconditions.checkNotNull(fieldContexts);
    fieldsPanel.selectFieldContexts(fieldContexts, true);
  }

  public @Nullable FieldContext getRootFieldContext() {
    return fieldsPanel.getRootFieldContext();
  }

  @Messages("TIT_FieldsDialog=Fields")
  public static void showDialog(final Set<FieldContext> fieldContexts) {
    Mutex.EVENT.postWriteRequest(() -> {
      final FieldsDialogPanel fieldsDialogPanel = new FieldsDialogPanel();
      fieldsDialogPanel.setFieldContexts(fieldContexts);
      final FieldContext rootFieldContext = fieldsDialogPanel.getRootFieldContext();
      if (rootFieldContext == null)
        return;
      final ImmutableView viewBefore = ImmutableView.copyOf(rootFieldContext.getView());
      DialogDescriptor dialogDescriptor =
          new DialogDescriptor(fieldsDialogPanel, Bundle.TIT_FieldsDialog());
      dialogDescriptor.setClosingOptions(new Object[] { NotifyDescriptor.OK_OPTION });
      final ObjectReference<Dialog> dialogRef = ObjectReference.create();
      dialogDescriptor.setButtonListener((ActionEvent evt) -> {
        if (NotifyDescriptor.CANCEL_OPTION.equals(evt.getSource()) ||
            NotifyDescriptor.CLOSED_OPTION.equals(evt.getSource())) {
          // Revert any changes made.
          /* TODO[pd]: Rewind the Undo list instead. Consolidate modifications made in the dialog
          into a single undoable edit. */
          // TODO[pd]: Localize the display name if I end up keeping this action.
          rootFieldContext.getView().mutate("Cancel Field Selection", true,
              (MutableView view) ->
          {
            view.copyFrom(viewBefore);
          });
          dialogRef.getValue().dispose();
        }
      });
      final Dialog dialog = DialogDisplayer.getDefault().createDialog(dialogDescriptor);
      dialogRef.setValue(dialog);
      dialog.setVisible(true);
      dialog.dispose();
    });
  }

  @SuppressWarnings("unchecked")
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {
    java.awt.GridBagConstraints gridBagConstraints;

    mainLabel = new javax.swing.JLabel();
    fieldsPanel = new com.sieuferd.project.navigator.FieldsPanel();
    moveUpButton = new javax.swing.JButton();
    moveDownButton = new javax.swing.JButton();

    setLayout(new java.awt.GridBagLayout());

    org.openide.awt.Mnemonics.setLocalizedText(mainLabel, org.openide.util.NbBundle.getMessage(FieldsDialogPanel.class, "FieldsDialogPanel.mainLabel.text")); // NOI18N
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints.weightx = 1.0;
    gridBagConstraints.insets = new java.awt.Insets(10, 10, 5, 10);
    add(mainLabel, gridBagConstraints);

    fieldsPanel.setBorder(javax.swing.BorderFactory.createEtchedBorder());
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 1;
    gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints.weightx = 1.0;
    gridBagConstraints.weighty = 1.0;
    gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 5);
    add(fieldsPanel, gridBagConstraints);

    org.openide.awt.Mnemonics.setLocalizedText(moveUpButton, org.openide.util.NbBundle.getMessage(FieldsDialogPanel.class, "FieldsDialogPanel.moveUpButton.text")); // NOI18N
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 1;
    gridBagConstraints.gridy = 1;
    gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 10);
    add(moveUpButton, gridBagConstraints);

    org.openide.awt.Mnemonics.setLocalizedText(moveDownButton, org.openide.util.NbBundle.getMessage(FieldsDialogPanel.class, "FieldsDialogPanel.moveDownButton.text")); // NOI18N
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 1;
    gridBagConstraints.gridy = 2;
    gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
    gridBagConstraints.insets = new java.awt.Insets(0, 0, 10, 10);
    add(moveDownButton, gridBagConstraints);
  }// </editor-fold>//GEN-END:initComponents


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private com.sieuferd.project.navigator.FieldsPanel fieldsPanel;
  private javax.swing.JLabel mainLabel;
  private javax.swing.JButton moveDownButton;
  private javax.swing.JButton moveUpButton;
  // End of variables declaration//GEN-END:variables
}
