// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.navigator;

import com.google.common.collect.Multimap;
import com.sieuferd.datamodel.ViewProperty;
import com.sieuferd.datamodel.ViewPropertySet;
import com.sieuferd.project.NBViewSet;
import java.util.Map;
import javax.swing.JPanel;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.nodes.PropertySupport;
import org.openide.nodes.Sheet;

final class ViewPropertyColumnSelectorPanel extends JPanel {
  private final Map<String,Boolean> selectedMap;

  private final Node columnSelectorNode = new AbstractNode(Children.LEAF) {
    @Override
    protected Sheet createSheet() {
      final Sheet ret = new Sheet();
      final Multimap<ViewPropertySet, ViewProperty<?>> propertiesBySet =
          NBViewSet.VIEW_DEFAULTS.getSortedPropertiesBySet();
      for (ViewPropertySet viewPropertySet : propertiesBySet.keySet()) {
        final Sheet.Set ps = new Sheet.Set();
        ps.setName(viewPropertySet.getName());
        ps.setDisplayName(viewPropertySet.getDisplayName());
        ps.setShortDescription(viewPropertySet.getShortDescription());
        ret.put(ps);
        for (final ViewProperty<?> prop : propertiesBySet.get(viewPropertySet)) {
          final String key = ViewPropertyTableColumnSelector.getColumnIdentifierForProperty(prop);
          ps.put(new PropertySupport<Boolean>(prop.getBeanPropertyName(), Boolean.class,
              prop.getDisplayName(), prop.getShortDescription(), true,
              !ViewPropertyTableColumnSelector.TREE_COLUMN_PROPERTIES.contains(prop) &&
              selectedMap.containsKey(key))
          {
            @Override
            public Boolean getValue() {
              if (ViewPropertyTableColumnSelector.TREE_COLUMN_PROPERTIES.contains(prop))
                return true;
              Boolean ret = selectedMap.get(key);
              return ret == null ? false : ret;
            }

            @Override
            public void setValue(Boolean value) {
              selectedMap.put(key, value);
            }
          });
        }
      }
      return ret;
    }
  };

  /* TODO[cc]: Don't use the string representation here, just use ViewProperty<?>. Then make
               TREE_COLUMN_PROPERTIES and getColumnIdentifierForProperty private. */
  /** @param selectedMap will be modified. */
  public ViewPropertyColumnSelectorPanel(Map<String,Boolean> selectedMap) {
    initComponents();
    this.selectedMap = selectedMap;
    propertySheet.setNodes(new Node[] { columnSelectorNode });
  }

  /**
   * This method is called from within the constructor to initialize the form. WARNING: Do NOT
   * modify this code. The content of this method is always regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {
    java.awt.GridBagConstraints gridBagConstraints;

    propertySheetLabel = new javax.swing.JLabel();
    propertySheet = new org.openide.explorer.propertysheet.PropertySheet();

    setLayout(new java.awt.GridBagLayout());

    propertySheetLabel.setLabelFor(propertySheet);
    org.openide.awt.Mnemonics.setLocalizedText(propertySheetLabel, org.openide.util.NbBundle.getMessage(ViewPropertyColumnSelectorPanel.class, "ViewPropertyColumnSelectorPanel.propertySheetLabel.text")); // NOI18N
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints.weightx = 1.0;
    gridBagConstraints.insets = new java.awt.Insets(0, 0, 10, 0);
    add(propertySheetLabel, gridBagConstraints);

    propertySheet.setBorder(javax.swing.BorderFactory.createEtchedBorder());
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 1;
    gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints.weightx = 1.0;
    gridBagConstraints.weighty = 1.0;
    add(propertySheet, gridBagConstraints);
  }// </editor-fold>//GEN-END:initComponents

  // Variables declaration - do not modify//GEN-BEGIN:variables
  private org.openide.explorer.propertysheet.PropertySheet propertySheet;
  private javax.swing.JLabel propertySheetLabel;
  // End of variables declaration//GEN-END:variables
}
