// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.nodes;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableSet;
import com.sieuferd.datamodel.CProp;
import com.sieuferd.datamodel.MutableView;
import com.sieuferd.datamodel.Path;
import com.sieuferd.project.FieldContext;
import com.sieuferd.project.FieldContext.RelationFieldContext;
import java.util.List;
import javax.swing.SwingUtilities;
import org.openide.nodes.Children;
import org.openide.nodes.Node;

public class FieldChildren extends Children.Keys<Path> {
  private final RelationFieldContext fieldContext;
  private final NodeSettings settings;
  private final Object lock = new Object();
  private ImmutableList<Path> visibleChildPaths = ImmutableList.of();
  private boolean added;

  public FieldChildren(RelationFieldContext fieldContext, NodeSettings settings) {
    /* Not sure what difference the lazy argument does here. I want lazily expanded nodes, though,
    so turn it on. */
    super(true);
    Preconditions.checkNotNull(fieldContext);
    Preconditions.checkNotNull(settings);
    this.fieldContext = fieldContext;
    this.settings = settings;
  }

  /* Just leave it to FieldNode to keep the list of children up to date, since FieldNode is already
  listening to the view, since FieldNode may need to substitute Children.LEAF when there are no
  children, and since Children.removeNotify() is not well-suited for removing strong listeners
  anyway. */

  void setVisibleChildPaths(List<Path> visibleChildPaths) {
    synchronized (lock) {
      this.visibleChildPaths = ImmutableList.copyOf(visibleChildPaths);
      if (added)
        setKeys(visibleChildPaths);
    }
  }

  @Override
  protected void addNotify() {
    synchronized (lock) {
      added = true;
      setKeys(visibleChildPaths);
    }
    if (!fieldContext.get(CProp.EXPANDED)) {
      // Do the marking operation later, to avoid deadlocks.
      SwingUtilities.invokeLater(() -> {
        fieldContext.getView().mutate("Mark Field Expanded", false, (MutableView view) -> {
          view.set(CProp.EXPANDED, fieldContext.getPath(), true);
        });
      });
    }
  }

  @Override
  protected void removeNotify() {
    synchronized (lock) {
      added = false;
      setKeys(ImmutableSet.of());
    }
  }

  @Override
  protected Node[] createNodes(Path key) {
    return new Node[] {
      new FieldNode(FieldContext.create(fieldContext.getViewContext(), key), settings)
    };
  }
}
