// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.nodes;

import com.sieuferd.project.FieldContext;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.annotation.concurrent.ThreadSafe;
import org.openide.nodes.Sheet;

/** Implementations should be thread-safe. */
@ThreadSafe
public abstract class FieldNodeProperties {
  private final PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

  public abstract Sheet createSheet();

  public final synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
    propertyChangeSupport.addPropertyChangeListener(listener);
  }

  public final synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
    propertyChangeSupport.removePropertyChangeListener(listener);
  }

  // TODO[cc]: Hide this.
  public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
    propertyChangeSupport.firePropertyChange(propertyName, oldValue, newValue);
  }

  public static interface FieldNodePropertiesProvider {
    public FieldNodeProperties createFieldNodeProperties(FieldContext fieldContext);
  }
}
