// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.nodes;

import com.google.common.base.Preconditions;
import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import com.sieuferd.project.nodes.FieldNodeProperties.FieldNodePropertiesProvider;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class NodeSettings {
  private final AppliesTo includedPathTypes;
  private final boolean usingFullContextMenu;
  private final boolean actionsDisabled;
  private final boolean checkable;
  private final boolean labelBadgeVisible;
  private final boolean includeAffordanceNodes;

  // TODO[cc]: Consider getting rid of this.
  private final @Nullable FieldNodePropertiesProvider customPropertiesProvider;

  // TODO[cc]: Consider providing a builder class, or some common static initializers.
  public NodeSettings(AppliesTo includedPathTypes, boolean usingFullContextMenu,
      boolean actionsDisabled, boolean checkable, boolean labelBadgeVisible,
      FieldNodePropertiesProvider customPropertiesProvider, boolean includeAffordanceNodes)
  {
    Preconditions.checkNotNull(includedPathTypes);
    this.includedPathTypes = includedPathTypes;
    this.usingFullContextMenu = usingFullContextMenu;
    this.actionsDisabled = actionsDisabled;
    this.checkable = checkable;
    this.labelBadgeVisible = labelBadgeVisible;
    this.customPropertiesProvider = customPropertiesProvider;
    this.includeAffordanceNodes = includeAffordanceNodes;
  }

  public AppliesTo getIncludedPathTypes() {
    return includedPathTypes;
  }

  public boolean isUsingFullContextMenu() {
    return usingFullContextMenu;
  }

  public boolean getActionsDisabled() {
    return actionsDisabled;
  }

  public boolean isCheckable() {
    return checkable;
  }

  public boolean isLabelBadgeVisible() {
    return labelBadgeVisible;
  }

  public FieldNodePropertiesProvider getCustomPropertiesProvider() {
    return customPropertiesProvider;
  }

  public boolean getIncludeAffordanceNodes() {
    return includeAffordanceNodes;
  }
}
