// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.nodes;

import com.sieuferd.project.SieuferdProject;
import com.sieuferd.project.derivatives.ViewUpdater;
import com.sieuferd.project.SieuferdIcons;
import java.awt.Image;
import javax.swing.Action;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.ui.support.ProjectActionPerformer;
import org.netbeans.spi.project.ui.support.ProjectSensitiveActions;
import org.openide.awt.ActionID;
import org.openide.awt.ActionRegistration;
import org.openide.awt.Actions;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.util.NbBundle.Messages;
import org.openide.util.lookup.Lookups;

public class ViewFolderNode extends AbstractNode {
  private final boolean primary;

  @Messages({
    "LBL_PrimaryViewFolder=Data Tables",
    "LBL_SecondaryViewFolder=Perspectives"
  })
  public ViewFolderNode(SieuferdProject project, NodeSettings nodeSettings, boolean primary) {
    super(Children.create(new ViewFolderChildFactory(project, nodeSettings, primary), true),
        Lookups.fixed(project));
    this.primary = primary;
    setName(primary ? Bundle.LBL_PrimaryViewFolder() : Bundle.LBL_SecondaryViewFolder());
  }

  @Override
  public Action[] getActions(boolean context) {
    if (context || !primary)
      return new Action[] {};
    return new Action[] {
      Actions.forID("Sieuferd", "com.sieuferd.actionids.RefreshPrimaryViewsAction"),
      Actions.forID("Sieuferd", "com.sieuferd.actionids.ImportAccessAction")
    };
  }

  @Override
  public Image getIcon(int type) {
    return SieuferdIcons.getTreeFolderIcon(false);
  }

  @Override
  public Image getOpenedIcon(int type) {
    return SieuferdIcons.getTreeFolderIcon(true);
  }

  /* TODO[cc]: Move this to com.sieuferd.project.actions (in a holder class for actions initialized
               via static factory methods). */

  @ActionID(category = "Sieuferd", id = "com.sieuferd.actionids.RefreshPrimaryViewsAction")
  @ActionRegistration(displayName = "#CTL_RefreshPrimaryViewsAction")
  @Messages("CTL_RefreshPrimaryViewsAction=Refresh Table List")
  public static Action createRefreshPrimaryViewsAction() {
    /* ProjectSensitiveActions are "always considered synchronous"; see
    https://netbeans.org/projects/platform/lists/dev/archive/2005-04/message/124 . */
    return ProjectSensitiveActions.projectSensitiveAction(new ProjectActionPerformer() {
      @Override
      public boolean enable(Project project) {
        return (project instanceof SieuferdProject);
      }

      @Override
      public void perform(Project project) {
        if (project instanceof SieuferdProject)
          ViewUpdater.refreshPrimaryViews((SieuferdProject) project);
      }
    }, Bundle.CTL_RefreshPrimaryViewsAction(), null);
  }
}
