// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.propertyeditors;

import com.sieuferd.query.OptionalPrimitivePath;
import com.sieuferd.datamodel.expressions.Expression;
import com.sieuferd.datamodel.expressions.Expression.PathExpression;
import com.sieuferd.datamodel.expressions.ExpressionErrorReport.ExpressionParseException;
import com.sieuferd.datamodel.Strepco;
import com.sieuferd.datamodel.expressions.Expressions;
import org.openide.nodes.PropertyEditorRegistration;

@PropertyEditorRegistration(targetType = OptionalPrimitivePath.class)
public class OptionalPrimitivePathEditor extends ViewPropertyEditor<OptionalPrimitivePath> {
  @Override
  public String valueToText(OptionalPrimitivePath value) {
    if (!value.isPresent())
      return "";
    final Strepco strepco = getStrepco();
    if (strepco == null)
      return null;
    return Expressions.generateString(strepco, new PathExpression(value.getPath()));
  }

  @Override
  public OptionalPrimitivePath textToValue(String text) throws IllegalArgumentException {
    if (text.trim().isEmpty())
      return OptionalPrimitivePath.absent();
    final Strepco strepco = getStrepco();
    if (strepco == null)
      throw new IllegalArgumentException("Cannot set path reference out of context");
    final Expression exp;
    try {
      exp = Expressions.parse(strepco, text);
    } catch (ExpressionParseException e) {
      throw new IllegalArgumentException("Failed to parse path reference", e);
    }
    if (!(exp instanceof PathExpression))
      throw new IllegalArgumentException("Value must be a path reference");
    PathExpression ret = (PathExpression) exp;
    if (ret.getPath().isRelation())
      throw new IllegalArgumentException("Value must reference primitive paths only");
    return OptionalPrimitivePath.of(ret.getPath().getPrimitive());
  }
}
