// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.result;

import com.sieuferd.uiutil.ThreadUtil;
import com.sieuferd.project.viewfile.ViewDataObject;
import org.openide.cookies.CloseCookie;
import org.openide.cookies.OpenCookie;
import org.openide.loaders.OpenSupport;
import org.openide.windows.CloneableOpenSupport;
import org.openide.windows.CloneableTopComponent;

/* See org.netbeans.modules.properties.PropertiesOpen and
org.netbeans.modules.image.ImageOpenSupport and their associated TopComponent classes. */
/* According to http://wiki.netbeans.org/DevFaqEditorTopComponent , "As of 6.9, the Openable
interface is preferred to OpenCookie", but that's probably when making use of the interfaces rather
than when implementing them. Since OpenCookie/CloseCookie extend Openable/Closable, the
backwards-compatible thing to do is to implement the former here. */
public class ViewOpenSupport extends OpenSupport implements OpenCookie, CloseCookie {
  /** May be called from any thread. */
  public ViewOpenSupport(ViewDataObject viewDataObject) {
    super(viewDataObject.getPrimaryEntry(), new Environment(viewDataObject));
  }

  @Override
  protected CloneableTopComponent createCloneableTopComponent() {
    ThreadUtil.checkOnEDT();
    ViewResultTopComponent ret = new ViewResultTopComponent();
    ret.setViewDataObject(entry.getDataObject().getLookup().lookup(ViewDataObject.class));
    return ret;
  }

  @Override
  protected String messageOpened() {
    /* This message would be shown in the status bar for about 4 seconds after the window has
    opened. Don't display it. */
    return null;
  }

  @Override
  protected String messageOpening() {
    /* Return null rather than the default, as the default uses the non-user-friendly file name
    associated with the view. Not quite sure where this is displayed, though. */
    return null;
  }

  // TODO[cc]: Consider removing this. ImageOpenSupport made more use of it.
  /* TODO[pd]: Maybe I ought to mark the environment as invalid if a view is deleted. See
               ImageOpenSupport.Environment.fileRemoved(). Although the editor already seems to
               close automatically when the underlying DataObject is removed. */
  private static final class Environment extends OpenSupport.Env {
    private static final long serialVersionUID = 1L;

    Environment(ViewDataObject dataObject) {
      super(dataObject);
    }

    // TODO[cc]: Do I need to override this? Consult existing classes that do.
    @Override
    public CloneableOpenSupport findCloneableOpenSupport() {
      return getDataObject().getLookup().lookup(ViewOpenSupport.class);
    }
  }
}
