// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.result;

import com.google.common.base.Optional;
import com.google.common.base.Preconditions;
import com.sieuferd.project.actions.ViewResultActions;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.LinkedHashMultimap;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Multimap;
import com.google.common.collect.Sets;
import com.sieuferd.cell.styles.PathHighlights;
import com.sieuferd.cell.support.Location;
import com.sieuferd.cell.support.CursorStyle;
import com.sieuferd.cell.support.Selection;
import com.sieuferd.cell.support.SelectionModel;
import com.sieuferd.upstream.toolbar.LocalContextToolbar;
import com.sieuferd.datamodel.CProp;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.Strepco;
import com.sieuferd.datamodel.View;
import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import com.sieuferd.datamodel.ViewResult;
import com.sieuferd.datamodel.ViewFlow;
import com.sieuferd.datamodel.Views;
import com.sieuferd.datamodel.proptypes.Warning;
import com.sieuferd.flow.Flow;
import com.sieuferd.flow.SettableFlow;
import com.sieuferd.flow.Flows;
import com.sieuferd.uiutil.flow.UIFlows;
import com.sieuferd.project.FieldContext;
import com.sieuferd.project.ViewContext;
import com.sieuferd.project.derivatives.ViewUpdater;
import com.sieuferd.uiutil.ThreadUtil;
import com.sieuferd.project.SieuferdIcons;
import com.sieuferd.project.nodes.FieldNode;
import com.sieuferd.project.nodes.FieldNodePathFinder;
import com.sieuferd.project.nodes.NodeSettings;
import com.sieuferd.project.actions.ToggleSchemaOnlyAction.SchemaOnlyButtonModel;
import com.sieuferd.project.actions.ToolboxInvoker;
import com.sieuferd.project.formulabar.CursorEditTarget;
import com.sieuferd.project.formulabar.CursorEditManager;
import com.sieuferd.project.formulabar.FormulaBar;
import com.sieuferd.project.toolbox.Toolbox;
import com.sieuferd.project.toolbox.ToolboxTab;
import com.sieuferd.project.util.QueryLabelFormatter;
import com.sieuferd.project.viewfile.ViewDataObject;
import com.sieuferd.query.QProp;
import com.sieuferd.datamodel.expressions.Expression.PathExpression;
import com.sieuferd.datamodel.expressions.Expressions;
import com.sieuferd.upstream.toolbar.WrapperAction;
import com.sieuferd.util.Functional;
import com.sieuferd.uiutil.UIUtil;
import com.sieuferd.util.UnitObject;
import com.sieuferd.widget.CellWidgetActions;
import com.sieuferd.widget.CellWidgetUtil;
import com.sieuferd.widget.GeneratedCell;
import com.sieuferd.uiutil.api.PopupLike;
import com.sieuferd.uiutil.SwingString;
import com.sieuferd.uiutil.flow.UIModelFlows;
import com.sieuferd.upstream.actions.ActionUtil;
import com.sieuferd.widget.InplaceEditor;
import com.sieuferd.widget.InteractionProvider;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.beans.PropertyVetoException;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.lang.ref.WeakReference;
import java.util.AbstractMap.SimpleEntry;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.PreferenceChangeEvent;
import java.util.prefs.PreferenceChangeListener;
import java.util.prefs.Preferences;
import javax.annotation.Nullable;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.BoundedRangeModel;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.DefaultEditorKit;
import org.netbeans.api.editor.mimelookup.MimeLookup;
import org.netbeans.api.editor.mimelookup.MimePath;
import org.openide.awt.UndoRedo;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.nodes.NodeOp;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle.Messages;
import org.openide.util.RequestProcessor;
import org.openide.util.io.SafeException;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import org.openide.windows.CloneableTopComponent;
import org.openide.windows.TopComponent;

// TODO[pd]: Go over CloneableEditor and see if there are other things I need to do similarly here.
/* According to https://forums.netbeans.org/ntopic29888.html and http://eppleton.de/blog/?p=619 ,
"writeExternal and readExternal shouldn't be used anymore. Since version 6.7 writeProperties and
readProperties metho[d]s are mean[t] to be used." But I can't get the properties approach to work
with CloneableTopComponent. Specifically, cloning wouldn't work, and existing tabs would not be used
for reopening when opening a view if the existing tab was created from persistence. */
@TopComponent.Description(
    preferredID = "ViewResultTopComponent",
    persistenceType = TopComponent.PERSISTENCE_ONLY_OPENED)
@Messages({
  "# {0} - label of the path for which the warning applies",
  "# {1} - warning display message",
  "LBL_WarningTooltipHTMLFmt=<b>{0}:</b> {1}",
  "LBL_WarningTooltipDeletedField=deleted field"
})
public final class ViewResultTopComponent extends CloneableTopComponent {
  private static final RequestProcessor RP = new RequestProcessor("Refresh Primary View", 1);
  private static final Logger LOG = Logger.getLogger(ViewResultTopComponent.class.getName());
  private static final double QUERY_LIMIT_MARGIN_FACTOR = 2.0;
  private static final int QUERY_LIMIT_MINIMUM_MAX = 10000;
  private static final boolean DEBUG_TEST_TWIN_FORMULA_BAR = false;
  /* See similar implementation in LocalContextToolbar. */
  private static final String TOOLBAR_VISIBLE_PROP = "toolbarVisible";
  private static final double ZOOM_FACTOR = 0.2;
  /* I've decided to use the regular progress bar for now, as the spinner is too subtle when put all
  the way to the right-hand corner of the screen. */
  private final boolean USE_PROGRESS_SPINNER_ON_AQUA = false;
  private final @Nullable FormulaBar debugTwinFormulaBar;
  private final JLabel warningTooltipLabel = new JLabel();
  // Initialize the toolbox here to avoid a delay the first time it is opened.
  private final Toolbox toolbox = Toolbox.getGlobalInstance();

  private final SettableFlow<ViewDataObject> viewDataObjectSettable =
      Flows.newSettable();
  private final SettableFlow<UnitObject> openedSettable = Flows.newSettable();

  /* Wrap in Flows.newValve to ensure that we are only subscribed to the ViewResultProvider when
  the component is open. This in turn avoids memory leaks due to listeners that are attached to but
  not removed from other Flow objects in this class, in particular through
  {@code Flows.bindEDT}. */
  private final Flow<ViewResultProvider> resultProviderFlow = Flows.newValve(openedSettable,
      Flows.newIndirect(viewDataObjectSettable,
      vdo -> UIFlows.newSingularLookup(vdo.getLookup(), ViewResultProvider.class)));
  private final UndoRedo undoRedo = ViewResultProvider.createUndoRedo(resultProviderFlow);
  private final Flow<ViewContext> viewContextFlow = Flows.newIndirect(resultProviderFlow,
      viewResultProvider -> viewResultProvider.getViewContextFlow());
  private final SchemaOnlyButtonModel schemaOnlyButtonModel = new SchemaOnlyButtonModel();
  private final Flow<Boolean> schemaOnlyButtonFlow =
      UIModelFlows.toggleButton(schemaOnlyButtonModel);
  private final Flow<GeneratedCell> generatedCellFlow;
  private final Flow<Selection> selectionFlow;
  private final CursorEditManager cursorEditManager;
  private final InplaceCursorEditorProvider inplaceCursorEditorProvider =
      new InplaceCursorEditorProvider();

  private final InstanceContent dataObjectInstanceContent = new InstanceContent();
  /**
   * The ExplorerManager is used to provide standard Node actions and to provide the correct
   * lookup for selected nodes, but is not otherwise exposed.
   */
  private final ExplorerManager explorerManager = new ExplorerManager();

  private final LocalContextToolbar buttonToolbar;
  private final @Nullable Preferences toolbarVisiblePreferences;

  private final PreferenceChangeListener toolbarVisibleListener = (PreferenceChangeEvent evt) -> {
    if (evt == null || evt.getKey() == null || evt.getKey().equals(TOOLBAR_VISIBLE_PROP))
      SwingUtilities.invokeLater(this::updateToolbarVisibility);
  };
  private final ChangeListener verticalScrollbarChangeListener = new ChangeListener() {
    @Override
      public void stateChanged(ChangeEvent event) {
      final BoundedRangeModel model = scrollPane.getVerticalScrollBar().getModel();
      cellWidget.setFreezeScrollables(model.getValueIsAdjusting());
      updateQueryLimit();
    }
  };
  private final ToolboxInvoker toolboxInvoker = new ToolboxInvoker() {
    @Override
    public void invoke(@Nullable Class<? extends ToolboxTab> tabClass) {
      PopupLike popup = getPreparedPopup(true, tabClass);
      if (popup != null)
        cellWidget.getUI().openCursorPopup(popup);
    }
  };

  private void updateQueryLimit() {
    ThreadUtil.checkOnEDT();
    final GeneratedCell generatedCell = cellWidget.getGeneratedCell();
    if (generatedCell == null)
      return;
    final BoundedRangeModel model = scrollPane.getVerticalScrollBar().getModel();
    final double viewportEnd = model.getValue() + model.getExtent();
    final double resultHeight = generatedCell.getCell().getDimensions().getY().getDouble();
    final ViewResultProvider vrp = resultProviderFlow.get();
    if (vrp == null)
      return;
    /* Estimate the limit needed to fill the screen to the end of the viewport based on the
    previous result's limit and resulting layout dimensions, plus some extra wiggle room
    (QUERY_LIMIT_MARGIN_FACTOR). The assumption is that the layout height is roughly proportional to
    the limit. */
    final int displayedLimit = generatedCell.getResult().getLimit().getLimit();
    final int newMinimumQueryLimit =
        (int) Math.min(QUERY_LIMIT_MINIMUM_MAX, Math.round(
        (viewportEnd / resultHeight) * displayedLimit * QUERY_LIMIT_MARGIN_FACTOR));
    vrp.setMinimumQueryLimit(this, newMinimumQueryLimit);
    if (viewportEnd > resultHeight && generatedCell.getResult().getData().isIncomplete())
      vrp.ensureQueryLimitIncreased(generatedCell.getResult().getLimit());
  }

  private @Nullable PopupLike getPreparedPopup(boolean useToolbox,
      @Nullable Class<? extends ToolboxTab> toolboxTabClass)
  {
    Preconditions.checkArgument(useToolbox || toolboxTabClass == null);
    /* TODO[pd]: Make Delete/Move Up/Move Down work properly in the context menu. This may involve
                 adding entries to the action map supplied by getLookup() (e.g. "delete"), allowing
                 new options for FieldNode (e.g. to enable Move Up/Move Down actions), and possibly
                 keeping nodes in a proper child/parent hierarchy. */
    Node activatedNodes[] = getActivatedNodes();
    if (activatedNodes == null)
      return null;
    if (useToolbox) {
      Set<FieldContext> fieldContexts = Sets.newLinkedHashSet();
      for (Node node : activatedNodes)
        fieldContexts.addAll(node.getLookup().lookupAll(FieldContext.class));
      toolbox.setFieldContexts(fieldContexts);
      if (toolboxTabClass != null)
        toolbox.setActiveTab(toolboxTabClass);
      return toolbox.getPopup();
    } else {
      // Based on similar code in org.openide.explorer.view.TreeView.
      Action[] actions = NodeOp.findActions(activatedNodes);
      if (actions.length > 0) {
        // Use the same lookup as for the toolbar.
        JPopupMenu ret = ActionUtil.actionsToPopup(actions, getLookup(), true);
        if (ret.getSubElements().length > 0)
          return PopupLike.fromJPopupMenu(ret);
      }
      return null;
    }
  }

  private final InteractionProvider interactionProvider = new InteractionProvider() {
    @Override
    public @Nullable PopupLike getMousePopup(int modifiersEx) {
      return getPreparedPopup((modifiersEx & MouseEvent.SHIFT_DOWN_MASK) != 0, null);
    }

    @Override
    public JComponent getCursorTooltip() {
      Location cursorLocation = cellWidget.getSelectionModel().getCursorLocation();
      if (cursorLocation == null)
        return null;
      GeneratedCell generatedCell = cellWidget.getGeneratedCell();
      if (generatedCell == null)
        return null;
      SwingString warningMessage = generateWarningMessage(
          generatedCell.getResult().getView(), generatedCell.getResult().getWarnings(),
          ImmutableSet.of(cursorLocation.getPath()), false);
      warningTooltipLabel.setText(warningMessage.getComplete());
      return warningMessage.isEmpty() ? null : warningTooltipLabel;
    }

    @Override
    public @Nullable InplaceEditor getInplaceEditor() {
      return inplaceCursorEditorProvider.getInplaceEditor();
    }

    @Override
    public Acceptance acceptEdit() {
      return cursorEditManager.acceptEdit();
    }

    @Override
    public void cancelEdit() {
      cursorEditManager.cancelEdit();
    }
  };

  // TODO[cc]: Move these elsewhere, and rename the bundle keys appropriately.
  public static SwingString getPathDisplayLabel(Strepco strepco, Path path) {
    if (!strepco.getView().exists(path))
      return SwingString.styled(
          "<i>(" + SwingString.toSwingHTML(Bundle.LBL_WarningTooltipDeletedField()) + ")</i>");
    String ret = Expressions.generateString(strepco, new PathExpression(path));
    if (!(ret.startsWith("[") && ret.endsWith("]")))
      throw new AssertionError();
    return SwingString.plain(ret.substring(1, ret.length() - 1));
  }

  public static SwingString generateWarningMessage(View view,
      ImmutableMultimap<Path,Warning> allWarnings, Set<Path> selectedPaths, boolean colored)
  {
    final Multimap<Path,Warning> warnings = LinkedHashMultimap.create();
    for (Path path : Views.getPathsWithAncestors(selectedPaths))
      warnings.putAll(path, allWarnings.get(path));
    final List<Entry<SwingString,Warning>> labelsAndWarnings = Lists.newArrayList();
    final boolean includeLabel =
        warnings.keySet().size() > 1 || !warnings.keySet().equals(selectedPaths);
    final Strepco strepco = Strepco.create(view, Views.findLowestCommonAncestor(selectedPaths));
    for (Entry<Path,Warning> entry : warnings.entries()) {
      final Path path = entry.getKey();
      final Warning warning = entry.getValue();
      final SwingString label =
          includeLabel ? getPathDisplayLabel(strepco, path) : SwingString.EMPTY;
      labelsAndWarnings.add(new SimpleEntry<>(label, warning));
    }
    return generateWarningMessage(labelsAndWarnings, colored);
  }

  public static SwingString generateWarningMessage(
      List<Entry<SwingString,Warning>> labelsAndWarnings, boolean colored)
  {
    if (labelsAndWarnings.isEmpty()) {
      return SwingString.EMPTY;
    } else {
      final StringBuilder ret = new StringBuilder();
      if (colored) {
        ret.append("<font color=\"")
            .append(SwingString.htmlColor("nb.errorForeground")).append("\">");
      }
      /* TODO[pd]: Get rid of this line deduplication logic once I can get the query evaluator to
                   avoid generating the duplicated messages in the first place. */
      final Set<String> lines = Sets.newLinkedHashSet();
      for (Entry<SwingString,Warning> entry : labelsAndWarnings) {
        final SwingString label = entry.getKey();
        final Warning warning = entry.getValue();
        String messageHTML = SwingString.toSwingHTML(warning.getDisplayMessage());
        lines.add((label.isEmpty() ? messageHTML :
            Bundle.LBL_WarningTooltipHTMLFmt(label.getForConcatenation(), messageHTML)) +
            "<br>\n");
      }
      for (String line : lines)
        ret.append(line);
      if (colored)
        ret.append("</font>");
      return SwingString.styled(ret.toString());
    }
  }

  private final Action stopQueryAction = new AbstractAction() {
    @Override
    public void actionPerformed(ActionEvent e) {
      final ViewResultProvider resultProvider = resultProviderFlow.get();
      if (resultProvider == null)
        return;
      resultProvider.stop();
    }
  };
  private final Action refreshViewAction = new AbstractAction() {
    @Override
    public void actionPerformed(ActionEvent e) {
      final ViewResultProvider resultProvider = resultProviderFlow.get();
      if (resultProvider == null)
        return; // Shouldn't happen; action should only be enabled while resultProvider is not null.
      final ViewContext viewContext = resultProvider.getViewContextFlow().get();
      if (viewContext != null && viewContext.getView().get(QProp.PRIMARY, Path.ROOT)) {
        RP.post(() -> {
          resultProvider.getRefreshable().invalidate();
          try {
            ViewUpdater.refreshPrimaryView(viewContext);
          } finally {
            resultProvider.getRefreshable().refresh();
          }
        });
      } else {
        resultProvider.getRefreshable().refresh();
      }
    }
  };
  private final Action originalSizeAction = new AbstractAction() {
    @Override
    public void actionPerformed(ActionEvent e) {
      cellWidget.setScaling(1.0);
    }
  };
  private final Action zoomInAction = new AbstractAction() {
    @Override
    public void actionPerformed(ActionEvent e) {
      cellWidget.setScaling(cellWidget.getScaling() * (1 + ZOOM_FACTOR));
    }
  };
  private final Action zoomOutAction = new AbstractAction() {
    @Override
    public void actionPerformed(ActionEvent e) {
      /* Using this formula ensures that an equal number of Zoom Out and Zoom In operations will
      always result in the original scaling (except for rounding error). */
      cellWidget.setScaling(cellWidget.getScaling() / (1 + ZOOM_FACTOR));
    }
  };

  private final ImmutableMap<Object,Object> EXPOSED_WIDGET_ACTIONS = ImmutableMap.builder()
      .put(CellWidgetActions.SELECT_ALL, DefaultEditorKit.selectAllAction)
      .put(CellWidgetActions.CUT, DefaultEditorKit.cutAction)
      .put(CellWidgetActions.COPY, DefaultEditorKit.copyAction)
      .put(CellWidgetActions.PASTE, DefaultEditorKit.pasteAction)
      .put(CellWidgetActions.SCROLL_UP_CHANGE_SELECTION, DefaultEditorKit.pageUpAction)
      .put(CellWidgetActions.SCROLL_DOWN_CHANGE_SELECTION, DefaultEditorKit.pageDownAction)
      .put(CellWidgetActions.OPEN_POPUP_MENU, "org.openide.actions.PopupAction")
      .put(CellWidgetActions.START_EDITING, ViewResultActions.EDIT_CELL)
      .build();

  private void applyLookAndFeelTweaks() {
    // Based on similar cut-and-pasted code from TabsComponent, CloneableEditor etc.

    /* Apply the standard toolbar border to the entire toolbarPanel rather than to just the
    buttonToolbar. */
    Border b = (Border) UIManager.get("Nb.Editor.Toolbar.border");
    toolbarPanel.setBorder(b);
    buttonToolbar.setBorder(null);
    buttonToolbar.setBorderPainted(false);

    if (UIUtil.isAqua()) {
      toolbarPanel.setBackground(UIManager.getColor("NbExplorerView.background"));
      if (USE_PROGRESS_SPINNER_ON_AQUA)
        progressBar.putClientProperty("JProgressBar.style", "circular");
      // There's an analogous tweak in TabsComponent.
      buttonToolbar.setBorder(BorderFactory.createEmptyBorder(3, 0, 0, 0));
      buttonToolbar.setBorderPainted(true);
    }
    // Make sure the toolbar has the same background color as the toolbarPanel.
    buttonToolbar.setOpaque(false);
  }

  public ViewResultTopComponent() {
    ThreadUtil.checkOnEDT();

    ActionMap map = this.getActionMap();
    map.put(DefaultEditorKit.copyAction, ExplorerUtils.actionCopy(explorerManager));
    map.put(DefaultEditorKit.cutAction, ExplorerUtils.actionCut(explorerManager));
    map.put(DefaultEditorKit.pasteAction, ExplorerUtils.actionPaste(explorerManager));
    map.put("delete", ExplorerUtils.actionDelete(explorerManager, true));
    map.put(ViewResultActions.REFRESH_VIEW_ACTION_KEY, refreshViewAction);
    map.put(ViewResultActions.STOP_QUERY_ACTION_KEY, stopQueryAction);
    map.put(ViewResultActions.ORIGINAL_SIZE_ACTION_KEY, originalSizeAction);
    map.put(ViewResultActions.ZOOM_IN_ACTION_KEY, zoomInAction);
    map.put(ViewResultActions.ZOOM_OUT_ACTION_KEY, zoomOutAction);

    /* Objects included in lookup:
       - The ExplorerManager's generated lookup, minus any DataObject/FileObject objects, since the
         "Synchronize Editor with Views" feature does not work when multiple such objects exist in
         the lookup.
       - A single entry for the current ViewDataObject (see above).
       - Fixed objects required to support certain toolbar actions (e.g. SchemaOnlyButtonModel,
         UndoRedo.Provider).

       Objects added to the lookup by the ExplorerManager:
       - ActionMap, as recommended by the Javadoc for TopComponent. Required for the local toolbar
         and context actions to work.
       - Activated nodes and their lookups. Putting nodes in the lookup passed to associateLookup()
         automatically makes these nodes the activated nodes of the TopComponent; see
         TopComponent.SynchronizeNodes. This seems to be the right way to set activated nodes; see
         the log warning emitted by TopComponent.multiviewActivatedNodes() and the discussion in
         https://netbeans.org/bugzilla/show_bug.cgi?id=67257 . Also note that when
         associateLookup() is used and no nodes are selected, getActivatedNodes() will return an
         empty array rather than null, which ensures the correct behavior of the TopComponent's
         lookup with respect to activated nodes in our case (see Javadoc for getLookup()).

       Objects added to the lookup via selected nodes' own lookups:
       - DataObject, via the selected FieldNode's lookup. Makes each ViewResultTopComponent
         automatically open and close with their project. Also needed for the Navigator.
    */
    associateLookup(new ProxyLookup(Lookups.fixed(
          schemaOnlyButtonModel,
          toolboxInvoker,
          new UndoRedo.Provider() {
            @Override
            public UndoRedo getUndoRedo() {
              return ViewResultTopComponent.this.getUndoRedo();
            }
          }
        ),
        Lookups.exclude(
            ExplorerUtils.createLookup(explorerManager, map), DataObject.class, FileObject.class),
        new AbstractLookup(dataObjectInstanceContent)));
    initComponents();
    cursorEditManager = new CursorEditManager(cellWidget);
    // Set this after initializing cursorEditManager.
    cellWidget.setInteractionProvider(interactionProvider);
    if (DEBUG_TEST_TWIN_FORMULA_BAR) {
      debugTwinFormulaBar = new FormulaBar();
      debugTwinFormulaBar.setBorder(javax.swing.BorderFactory.createEmptyBorder(2, 7, 4, 4));
      toolbarPanel.add(debugTwinFormulaBar, java.awt.BorderLayout.SOUTH);
    } else {
      debugTwinFormulaBar = null;
    }

    {
      warningTooltipLabel.setBorder(new EmptyBorder(2, 3, 2, 3));
      warningTooltipLabel.setOpaque(true);
      warningTooltipLabel.setBackground(Color.WHITE);
      warningTooltipLabel.setVerticalAlignment(SwingConstants.TOP);
      warningTooltipLabel.setVerticalTextPosition(SwingConstants.TOP);
      warningTooltipLabel.setIcon(
          ImageUtilities.loadImageIcon(SieuferdIcons.ICON_WARNING, false));
    }

    // TODO[cc]: Move this to CellWidget.
    /* This shortcut is in addition to any shortcuts configured through the keymap system,
    specifically for when the CellWidget is focused. Do not use WHEN_ANCESTOR_OF_FOCUSED_COMPONENT,
    since pressing Enter while in an inplace edit should invoke the CellWidget's default action for
    the key, which is a logical "down" cursor movement. */
    /* Use KeyStroke.getKeyStroke('\n') instead of KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0) to
    make sure that it is the KEY_TYPED event rather than the KEY_PRESSED event that gets
    intercepted, otherwise the KEY_TYPED event will be forwarded to the freshly opened inplace
    editor. */
    UIUtil.getSafeInputMap(cellWidget, JComponent.WHEN_FOCUSED).put(
        KeyStroke.getKeyStroke('\n'), CellWidgetActions.START_EDITING);

    /* No need for manual focus traversal within the TopComponent; the CellWidget should always be
    focused except when interacting with text fields in the toolbar. Disabling the focus traversal
    keys avoids interference with editor tab navigation with Ctrl+Tab. */
    cellWidget.setFocusTraversalKeysEnabled(false);

    {
      /* Not sure I understand why a weak reference is needed, but it can't hurt--that's the way it
      was done in NbEditorToolBar (from which WrapperAction was originally lifted). */
      final WeakReference<JComponent> cellWidgetRef = new WeakReference<>(cellWidget);
      for (Entry<Object,Object> entry : EXPOSED_WIDGET_ACTIONS.entrySet()) {
        Action a = cellWidget.getActionMap().get(entry.getKey());
        if (a == null) {
          LOG.log(Level.WARNING, "Failed to locate CellWidget action '{0}'", a);
        } else {
          getActionMap().put(entry.getValue(), new WrapperAction(cellWidgetRef, a));
        }
      }
    }

    buttonToolbar = new LocalContextToolbar(this, getLookup(), "Sieuferd/ViewResultToolbar");
    /* For a floatable toolbar, I would have to use BorderLayout.NORTH here, but I want the toolbar
    to consume any extra vertical space caused by internal padding in the GridBagLayout, so I use
    CENTER. (I don't want the toolbar to be floatable anyway.) */
    toolbarWrapPanel.add(buttonToolbar, BorderLayout.CENTER);
    buttonToolbar.setFloatable(false);
    applyLookAndFeelTweaks();
    toolbarVisiblePreferences = MimeLookup.getLookup(MimePath.EMPTY).lookup(Preferences.class);
    updateToolbarVisibility();

    final Flow<ViewResult> viewResultFlow = Flows.newIndirect(resultProviderFlow,
        viewResultProvider -> viewResultProvider.getViewResultFlow());
    final Flow<UnitObject> evaluationProgressFlow = Flows.newIndirect(resultProviderFlow,
        viewResultProvider -> viewResultProvider.getProgressFlow());

    generatedCellFlow = CellWidgetUtil.getCellGenerator(cellWidget, viewResultFlow,
        schemaOnlyButtonFlow, QueryLabelFormatter.INSTANCE, false);
    // Flows.printGraphVizDependencyChart(System.out, generatedCellFlow);
    final SelectionModel selectionModel = cellWidget.getSelectionModel();
    selectionFlow = UIFlows.fromPropertyBean(SelectionModel.PROP_SELECTION,
        listener -> { selectionModel.addPropertyChangeListener(listener); },
        listener -> { selectionModel.removePropertyChangeListener(listener); },
        () -> selectionModel.getSelection());
    final Flow<CursorEditTarget> cursorEditTargetFlow = Flows.newComposite3(selectionFlow,
        viewResultFlow, viewContextFlow,
        (Selection selection, ViewResult viewResult, ViewContext viewContext) ->
    {
      Location cursorLocation = selection.getCursorLocation();
      if (cursorLocation == null ||
          !CursorEditTarget.pathsExist(viewResult.getView(), cursorLocation))
      {
        return null;
      } else {
        return new CursorEditTarget(viewContext, cursorLocation, viewResult);
      }
    });
    UIFlows.bindEDT(cursorEditTargetFlow, (final CursorEditTarget target) -> {
      /* I previously put this in a SwingUtilities.invokeLater to try to work around a highlight
      flicker problem. Don't do this; it prevents the CursorEditManager from immediately seeing
      the effects of CursorEditModel.setValue(String). */
      cursorEditManager.setTarget(target);
      /* Put the formula bar's editor in extended input mode by default. It can be temporarily
      turned to non-extended input mode by moving the cursor in the reference selection model. */
      formulaBar.setExtendedInputMode(cellWidget, true);
    });
    UIFlows.bindEDT(selectionFlow, (Selection selection) -> {
      final Location cursorLocation = (selection == null) ? null : selection.getCursorLocation();
      final Set<Path> paths = Sets.newLinkedHashSet();
      if (selection != null) {
        paths.addAll(selection.getSelectedPaths());
        if (cursorLocation != null && paths.isEmpty())
          paths.add(cursorLocation.getPath());
      }
      setSelectedNodesFromPaths(paths);
    });
    UIFlows.bindEDT(viewDataObjectSettable, (ViewDataObject value) -> {
      dataObjectInstanceContent.set(Optional.fromNullable(value).asSet(), null);
    });
    UIFlows.bindEDT(Flows.newIndirect(viewContextFlow,
        viewContext -> new ViewFlow(viewContext.getView(),
        Functional.cons(SieuferdIcons.ICON_PROPERTIES, CProp.LABEL),
        ImmutableSet.<Path>of(Path.ROOT))
    ), view -> {
      final String name = (view == null) ? "" : view.get(CProp.LABEL, Path.ROOT);
      setName(name);
      setDisplayName(name);
      setToolTipText(name);
      setIcon(view == null ? null : SieuferdIcons.getViewIcon(view));
    });
    UIFlows.bindEDT(generatedCellFlow, (GeneratedCell generatedCell) -> {
      if (generatedCell != null) {
        cellWidget.setGeneratedCell(generatedCell);
        updateQueryLimit();
      }
    });
    UIFlows.bindEDT(Flows.newComposite2(evaluationProgressFlow, generatedCellFlow,
        (progress, generatedCell) -> UnitObject.INSTANCE), (UnitObject value) ->
    {
      setInProgress(value == null);
    });
    UIFlows.bindEDT(evaluationProgressFlow, (UnitObject value) -> {
      stopQueryAction.setEnabled(value == null);
    });
    UIFlows.bindEDT(viewContextFlow, (ViewContext value) -> {
      explorerManager.setRootContext(
          value == null ? Node.EMPTY : new FieldNode(FieldContext.create(value, Path.ROOT),
              new NodeSettings(AppliesTo.ANY, true, false, false, false, null, false)));
    });
    UIFlows.bindEDT(resultProviderFlow, (ViewResultProvider resultProvider) -> {
      refreshViewAction.setEnabled(resultProvider != null);
    });
    final Flow<PathHighlights> highlightsFlow = Flows.newComposite2(
        Flows.newDefault(cursorEditManager.getReferenceHighlightsFlow(), PathHighlights.NONE),
        viewResultFlow, (referenceHighlights, viewResult) ->
    {
      // Don't display warning highlights during reference editing.
      if (referenceHighlights.getEditLocation() != null) {
        return referenceHighlights;
      } else {
        Map<Path,Color> highlights = Maps.newLinkedHashMap();
        for (Path warningPath : viewResult.getWarnings().keySet())
          highlights.put(warningPath, CursorStyle.WARNING_COLOR);
        return new PathHighlights(highlights, null);
      }
    });
    UIFlows.bindEDT(highlightsFlow, (PathHighlights highlights) -> {
      cellWidget.setPathHighlights(highlights == null ? PathHighlights.NONE : highlights);
    });
    UIFlows.bindEDT(cursorEditManager.getReferenceSelectionModelFlow(), referenceSelectionModel -> {
      cellWidget.setReferenceSelectionModel(referenceSelectionModel);
      /* Have the reference selection cursor start in the same place as the regular one. Don't
      disturb the reference selection model if there already is a selection. */
      if (referenceSelectionModel != null && referenceSelectionModel.getCursorLocation() == null) {
        referenceSelectionModel.setCursorLocation(
            cellWidget.getSelectionModel().getCursorLocation(), false);
      }
    });
  }

  private void setInProgress(boolean inProgress) {
    ThreadUtil.checkOnEDT();
    if (inProgress == progressBar.isIndeterminate())
      return;
    progressBar.setIndeterminate(inProgress);
    ((CardLayout) progressPanel.getLayout()).show(progressPanel,
        inProgress ? "progressVisible" : "progressInvisible");
  }

  @Override
  public boolean requestFocusInWindow() {
    return cellWidget.requestFocusInWindow();
  }

  @Override
  public void requestFocus() {
    cellWidget.requestFocus();
  }

  @Override
  protected void componentActivated() {
    super.componentActivated();
    requestFocusInWindow();
  }

  private void updateToolbarVisibility() {
    ThreadUtil.checkOnEDT();
    toolbarPanel.setVisible(toolbarVisiblePreferences == null ||
        toolbarVisiblePreferences.getBoolean(TOOLBAR_VISIBLE_PROP, true));
  }

  private void setSelectedNodesFromPaths(Set<Path> paths) {
    ThreadUtil.checkOnEDT();
    final List<FieldContext> activatedFields = Lists.newArrayList();
    final ViewContext viewContext = viewContextFlow.get();
    if (viewContext != null) {
      for (Path path : paths)
        activatedFields.add(FieldContext.create(viewContext, path));
    }
    // For now, always expose some node. This makes the navigator work better.
    if (activatedFields.isEmpty() && viewContext != null)
      activatedFields.add(FieldContext.create(viewContext, Path.ROOT));
    setSelectedNodes(
        FieldNodePathFinder.findNodes(explorerManager.getRootContext(), activatedFields));
  }

  private void setSelectedNodes(Node nodes[]) {
    try {
      explorerManager.setSelectedNodes(nodes);
    } catch (PropertyVetoException e) {
      LOG.log(Level.WARNING, "Unexpected exception while setting selected nodes", e);
    }
  }

  public void setViewDataObject(@Nullable ViewDataObject viewDataObject) {
    viewDataObjectSettable.set(viewDataObject);
  }

  public @Nullable ViewDataObject getViewDataObject() {
    return viewDataObjectSettable.getWithoutListener();
  }

  @Override
  public UndoRedo getUndoRedo() {
    return undoRedo;
  }

  @Override
  public void componentOpened() {
    ThreadUtil.checkOnEDT();
    super.componentOpened();
    openedSettable.set(UnitObject.INSTANCE);
    scrollPane.getVerticalScrollBar().getModel().addChangeListener(verticalScrollbarChangeListener);
    if (toolbarVisiblePreferences != null)
      toolbarVisiblePreferences.addPreferenceChangeListener(toolbarVisibleListener);
    updateToolbarVisibility();
    cursorEditManager.registerEditorProvider(inplaceCursorEditorProvider);
    cursorEditManager.registerEditorProvider(formulaBar);
    if (debugTwinFormulaBar != null)
      cursorEditManager.registerEditorProvider(debugTwinFormulaBar);
  }

  @Override
  public void componentClosed() {
    ThreadUtil.checkOnEDT();
    {
      final ViewResultProvider vrp = resultProviderFlow.get();
      if (vrp != null)
        vrp.setMinimumQueryLimit(this, 0);
    }
    super.componentClosed();
    openedSettable.set(null);
    scrollPane.getVerticalScrollBar().getModel()
        .removeChangeListener(verticalScrollbarChangeListener);
    if (toolbarVisiblePreferences != null)
      toolbarVisiblePreferences.removePreferenceChangeListener(toolbarVisibleListener);
    setSelectedNodes(new Node[] {});
    cursorEditManager.unregisterEditorProvider(inplaceCursorEditorProvider);
    cursorEditManager.unregisterEditorProvider(formulaBar);
    if (debugTwinFormulaBar != null)
      cursorEditManager.unregisterEditorProvider(debugTwinFormulaBar);
  }

  @Override
  public void writeExternal(ObjectOutput out) throws IOException {
    ThreadUtil.checkOnEDT();
    super.writeExternal(out);
    // Version
    out.writeObject((short) 1);
    out.writeObject(getViewDataObject());
    out.writeInt(formulaBar.getVisibleRows());
  }

  @Override
  @SuppressWarnings({"ThrowableInstanceNotThrown", "ThrowableInstanceNeverThrown"})
  public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
    ThreadUtil.checkOnEDT();
    super.readExternal(in);
    /* Ignore errors here, including incorrect types. See similar implementations in the NetBeans
    codebase and
    http://bits.netbeans.org/dev/javadoc/org-openide-windows/org/openide/windows/doc-files/api.html
    "do not throw the original exception from it to the writeExternal or readExternal callers - this
    will make the whole window system stream be treated as corrupt and discarded". */
    Object obj1 = in.readObject();
    if (!(obj1 instanceof Short) || ((Short) obj1) != 1)
      throw new SafeException(new IOException("Serialization version not supported"));
    Object obj2 = in.readObject();
    if (obj2 == null || obj2 instanceof ViewDataObject)
      setViewDataObject((ViewDataObject) obj2);
    formulaBar.setVisibleRows(in.readInt());
  }

  /**
   * This method is called from within the constructor to initialize the form. WARNING: Do NOT
   * modify this code. The content of this method is always regenerated by the Form Editor.
   */
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {
    java.awt.GridBagConstraints gridBagConstraints;

    toolbarPanel = new javax.swing.JPanel();
    upperPanel = new javax.swing.JPanel();
    toolbarWrapPanel = new javax.swing.JPanel();
    progressPanel = new javax.swing.JPanel();
    hiddenProgressPanel = new javax.swing.JPanel();
    progressBar = new javax.swing.JProgressBar();
    formulaBar = new com.sieuferd.project.formulabar.FormulaBar();
    scrollPane = new javax.swing.JScrollPane();
    cellWidget = new com.sieuferd.widget.CellWidget();

    setLayout(new java.awt.BorderLayout());

    toolbarPanel.setLayout(new java.awt.BorderLayout());

    upperPanel.setOpaque(false);
    upperPanel.setLayout(new java.awt.GridBagLayout());

    toolbarWrapPanel.setOpaque(false);
    toolbarWrapPanel.setLayout(new java.awt.BorderLayout());
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints.ipady = 4;
    gridBagConstraints.weightx = 1.0;
    gridBagConstraints.insets = new java.awt.Insets(1, 0, 0, 0);
    upperPanel.add(toolbarWrapPanel, gridBagConstraints);

    progressPanel.setOpaque(false);
    progressPanel.setLayout(new java.awt.CardLayout());

    hiddenProgressPanel.setOpaque(false);
    hiddenProgressPanel.setLayout(new java.awt.BorderLayout());
    progressPanel.add(hiddenProgressPanel, "progressInvisible");
    progressPanel.add(progressBar, "progressVisible");

    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 1;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints.insets = new java.awt.Insets(3, 0, 0, 4);
    upperPanel.add(progressPanel, gridBagConstraints);

    toolbarPanel.add(upperPanel, java.awt.BorderLayout.NORTH);

    formulaBar.setBorder(javax.swing.BorderFactory.createEmptyBorder(2, 7, 4, 4));
    formulaBar.setKeyStrokeForwardable(cellWidget);
    toolbarPanel.add(formulaBar, java.awt.BorderLayout.CENTER);

    add(toolbarPanel, java.awt.BorderLayout.NORTH);

    scrollPane.setBorder(null);
    scrollPane.setViewport(new com.sieuferd.uiutil.scroll.OverlayViewport());
    scrollPane.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    scrollPane.setViewportView(cellWidget);

    add(scrollPane, java.awt.BorderLayout.CENTER);
  }// </editor-fold>//GEN-END:initComponents

  // Variables declaration - do not modify//GEN-BEGIN:variables
  private com.sieuferd.widget.CellWidget cellWidget;
  private com.sieuferd.project.formulabar.FormulaBar formulaBar;
  private javax.swing.JPanel hiddenProgressPanel;
  private javax.swing.JProgressBar progressBar;
  private javax.swing.JPanel progressPanel;
  private javax.swing.JScrollPane scrollPane;
  private javax.swing.JPanel toolbarPanel;
  private javax.swing.JPanel toolbarWrapPanel;
  private javax.swing.JPanel upperPanel;
  // End of variables declaration//GEN-END:variables
}
