// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.toolbox;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableSet;
import com.sieuferd.project.FieldContext;
import com.sieuferd.project.actions.MoveAction;
import com.sieuferd.uiutil.EnterFilterInputMap;
import com.sieuferd.upstream.toolbar.ToolbarUtil;
import com.sieuferd.uiutil.ThreadUtil;
import com.sieuferd.uiutil.UIUtil;
import java.awt.Component;
import java.awt.Dimension;
import java.util.Set;
import javax.annotation.Nullable;
import javax.swing.ScrollPaneConstants;
import javax.swing.UIManager;
import javax.swing.border.MatteBorder;
import org.openide.explorer.view.OutlineView;
import org.openide.util.NbBundle.Messages;

@Messages("TIT_FieldsTabPanel=Fields")
public final class FieldsTabPanel extends ToolboxTab {
  private Set<FieldContext> fieldContexts = ImmutableSet.of();
  private boolean opened;

  @SuppressWarnings("LeakingThisInConstructor")
  FieldsTabPanel() {
    initComponents();
    if (UIUtil.isAqua()) {
      // I'd use a bottom border as well, but it becomes ugly when the horizontal scrollbar shows.
      fieldsPanel.getOutlineView().setViewportBorder(
          new MatteBorder(4, 0, 0, 0, UIManager.getColor("NbExplorerView.background")));
    }
    MoveAction.connectMoveAction(this, fieldsPanel.getLookup(), moveUpButton, false);
    MoveAction.connectMoveAction(this, fieldsPanel.getLookup(), moveDownButton, true);
    ToolbarUtil.setBorderOnHover(moveUpButton);
    ToolbarUtil.setBorderOnHover(moveDownButton);
    final OutlineView outlineView = fieldsPanel.getOutlineView();
    outlineView.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
    // Setting the table header to null would cause a NPE; hide it this way instead.
    outlineView.getOutline().getTableHeader().setVisible(false);
    outlineView.getOutline().getTableHeader().setPreferredSize(new Dimension(0, 0));
    EnterFilterInputMap.configure(outlineView.getOutline());
  }

  public @Nullable FieldContext getRootFieldContext() {
    return fieldsPanel.getRootFieldContext();
  }

  @Override
  public void setFieldContexts(final Set<FieldContext> fieldContexts) {
    ThreadUtil.checkOnEDT();
    Preconditions.checkNotNull(fieldContexts);
    this.fieldContexts = ImmutableSet.copyOf(fieldContexts);
    updateSelection();
  }

  private void updateSelection() {
    // The FieldsPanel selection needs to be set after the component has been made visible.
    if (opened)
      fieldsPanel.selectFieldContexts(fieldContexts, false);
  }

  @Override
  public void setOpened(boolean opened) {
    if (this.opened != opened) {
      this.opened = opened;
      fieldsPanel.setActionsActivated(opened);
      updateSelection();
    }
  }

  @Override
  protected Component getDefaultFocusComponent() {
    return fieldsPanel;
  }

  @Override
  public String getTitle() {
    return Bundle.TIT_FieldsTabPanel();
  }

  @SuppressWarnings("unchecked")
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {
    java.awt.GridBagConstraints gridBagConstraints;

    fieldsPanel = new com.sieuferd.project.navigator.FieldsPanel();
    moveUpButton = new javax.swing.JButton();
    moveDownButton = new javax.swing.JButton();

    setOpaque(false);
    setPreferredSize(new java.awt.Dimension(350, 500));
    setLayout(new java.awt.GridBagLayout());

    fieldsPanel.setBorder(javax.swing.BorderFactory.createMatteBorder(0, 0, 0, 1, javax.swing.UIManager.getDefaults().getColor("Separator.shadow")));
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 0;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
    gridBagConstraints.weightx = 1.0;
    gridBagConstraints.weighty = 1.0;
    add(fieldsPanel, gridBagConstraints);
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 2;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
    add(moveUpButton, gridBagConstraints);
    gridBagConstraints = new java.awt.GridBagConstraints();
    gridBagConstraints.gridx = 2;
    gridBagConstraints.gridy = 1;
    gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
    gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
    gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
    gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
    add(moveDownButton, gridBagConstraints);
  }// </editor-fold>//GEN-END:initComponents


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private com.sieuferd.project.navigator.FieldsPanel fieldsPanel;
  private javax.swing.JButton moveDownButton;
  private javax.swing.JButton moveUpButton;
  // End of variables declaration//GEN-END:variables
}
