// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.toolbox;

import com.sieuferd.project.FieldContext;
import java.awt.Component;
import java.util.Set;
import javax.annotation.Nullable;
import javax.swing.JPanel;

// TODO[cc]: Consider storing the JPanel as a field instead of extending from it.
public abstract class ToolboxTab extends JPanel {
  public abstract void setFieldContexts(final Set<FieldContext> fieldContexts);
  /**
   * Called to indicate whether the tab is opened and visible to the user. Context-sensitive tabs
   * can use this to determine when to attach and detach listeners from externals objects. The
   * {@code FieldContext} selection set in {@link #setFieldContexts(Set)} may remain unchanged and
   * non-empty even when the tab is not open. The latter behavior may aid tabs in retaining user
   * state (e.g. selection) between subsequent invocations.
   */
  public abstract void setOpened(boolean opened);
  public abstract String getTitle();
  protected abstract @Nullable Component getDefaultFocusComponent();

  @Override
  public final boolean requestFocusInWindow() {
    if (!isFocusable())
      return false;
    final Component defaultFocusComponent = getDefaultFocusComponent();
    if (defaultFocusComponent == null) {
      return super.requestFocusInWindow();
    } else {
      return defaultFocusComponent.requestFocusInWindow();
    }
  }

  @Override
  public final void requestFocus() {
    if (!isFocusable())
      return;
    final Component defaultFocusComponent = getDefaultFocusComponent();
    if (defaultFocusComponent == null) {
      super.requestFocus();
    } else {
      defaultFocusComponent.requestFocus();
    }
  }
}
