// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.util;

import com.sieuferd.uiutil.ThreadUtil;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.FlavorTable;
import java.awt.datatransfer.SystemFlavorMap;
import java.lang.reflect.InvocationTargetException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.openide.util.Utilities;

public final class ExcelClipboardUtil {
  private static final Logger LOG = Logger.getLogger(ExcelClipboardUtil.class.getName());
  private static DataFlavor excelDataFlavor;

  private ExcelClipboardUtil() { }

  public static DataFlavor getExcelDataFlavor() {
    ThreadUtil.checkOnEDT();
    if (excelDataFlavor == null) {
      excelDataFlavor = new DataFlavor("application/x-ms-excel-clipboard", "Excel");
      final String nat;
      if (Utilities.isMac()) {
        // Tested with Excel for Mac 2011.
        nat = "CorePasteboardFlavorType 0x454D4253"; // EMBS
      } else if (Utilities.isWindows()) {
        // Tested with Excel for Windows 2013.
        nat = "Biff8";
      } else {
        nat = null;
      }
      if (nat != null) {
        final SystemFlavorMap map = (SystemFlavorMap) SystemFlavorMap.getDefaultFlavorMap();
        map.addUnencodedNativeForFlavor(excelDataFlavor, nat);
        map.addFlavorForUnencodedNative(nat, excelDataFlavor);
        // TODO[pd]: Remove this once JDK-8136781 is fixed. Or keep it for older Java versions?
        if (Utilities.isMac()) {
          /* Workaround for JDK-8136781
          ( http://bugs.java.com/bugdatabase/view_bug.do?bug_id=JDK-8136781 ). Force calling of
          DataTransferer.getFormatForNativeAsLong, which is necessary for getting indexForFormat to
          return a valid index when CClipboard.getClipboardFormats in
          macosx/native/sun/awt/CClipboard.m calls it. */
          /* Do this by reflection in case the sun.awt.datatransfer becomes inaccessible in future
          JDK versions. */
          try {
            // sun.awt.datatransfer.DataTransferer.getInstance().getFormatsForFlavor(result, map);
            final Class<?> dtClass = Class.forName("sun.awt.datatransfer.DataTransferer");
            final Object dataTransferer = dtClass.getDeclaredMethod("getInstance").invoke(null);
            dtClass.getDeclaredMethod("getFormatsForFlavor", DataFlavor.class, FlavorTable.class)
                .invoke(dataTransferer, excelDataFlavor, map);
          } catch (NoSuchMethodException | IllegalAccessException | IllegalArgumentException |
              InvocationTargetException | ClassNotFoundException e)
          {
            LOG.log(Level.INFO, "Failed to apply workaround for bug JDK-8136781", e);
          }
        }
      }
    }
    return excelDataFlavor;
  }
}
