// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.util;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import com.sieuferd.datamodel.ImmutableView;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.ViewProperty;
import com.sieuferd.datamodel.ViewProperty.AppliesTo;
import com.sieuferd.datamodel.ViewFlow;
import com.sieuferd.flow.Flow;
import com.sieuferd.flow.Flows;
import com.sieuferd.project.ViewContext;
import java.util.Collections;
import java.util.Set;
import org.openide.util.Lookup;

// TODO[cc]: Find a better name for this and FieldSelection.

public final class SelectionSnapshot {
  private final FieldSelection fieldSelection;
  private final ImmutableView viewSnapshot;

  private SelectionSnapshot(FieldSelection fieldSelection, ImmutableView viewSnapshot) {
    Preconditions.checkNotNull(fieldSelection);
    Preconditions.checkNotNull(viewSnapshot);
    this.fieldSelection = fieldSelection;
    this.viewSnapshot = viewSnapshot;
  }

  public ViewContext getViewContext() {
    return fieldSelection.getViewContext();
  }

  public Set<Path> getSelectedPaths() {
    return getSelectedPaths(true);
  }

  public Set<Path> getSelectedPaths(boolean existentPathsOnly) {
    if (existentPathsOnly) {
      final Set<Path> ret =
          Sets.newLinkedHashSetWithExpectedSize(fieldSelection.getSelectedPaths().size());
      for (Path path : fieldSelection.getSelectedPaths()) {
        if (viewSnapshot.exists(path))
          ret.add(path);
      }
      return Collections.unmodifiableSet(ret);
    } else {
      return fieldSelection.getSelectedPaths();
    }
  }

  public ImmutableView getViewSnapshot() {
    return viewSnapshot;
  }

  public static Flow<SelectionSnapshot> fromLookup(
      Lookup lookup, final AppliesTo acceptablePathTypes, Set<ViewProperty<?>> viewProperties,
      final boolean trackSelectedPathOnly)
  {
    return fromFieldSelection(FieldSelection.fromLookup(lookup, acceptablePathTypes),
        viewProperties, trackSelectedPathOnly);
  }

  private static Flow<SelectionSnapshot> fromFieldSelection(
      Flow<FieldSelection> fieldSelectionFlow, Set<ViewProperty<?>> viewProperties,
      final boolean trackSelectedPathOnly)
  {
    final ImmutableSet<ViewProperty<?>> viewPropertiesImmutable =
        ImmutableSet.copyOf(viewProperties);
    final Flow<ImmutableView> viewFlow = Flows.newIndirect(fieldSelectionFlow, fieldSelection ->
        new ViewFlow(fieldSelection.getViewContext().getView(), viewPropertiesImmutable,
        trackSelectedPathOnly ? fieldSelection.getSelectedPaths() : null));
    return Flows.newComposite2(fieldSelectionFlow, viewFlow,
        (fieldSelection, view) -> new SelectionSnapshot(fieldSelection, view));
  }

  @Override
  public String toString() {
    return "SelectionSnapshot(" + fieldSelection + ", " + viewSnapshot + ")";
  }
}
