// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.viewfile;

import com.google.common.base.Preconditions;
import com.sieuferd.project.result.ViewOpenSupport;
import com.sieuferd.project.result.ViewResultProvider;
import java.io.IOException;
import java.util.logging.Logger;
import javax.annotation.concurrent.ThreadSafe;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.CookieSet;
import org.openide.nodes.Node.Cookie;

@DataObject.Registration(mimeType = ViewDataObject.MIME_TYPE)
// TODO[cc]: Move this class to a different package, and delete this package.
@ThreadSafe
public class ViewDataObject extends MultiDataObject {
  private static final Logger LOG = Logger.getLogger(ViewDataObject.class.getName());
  public static final String MIME_TYPE = "application/sieuferd-viewdataobject";

  public ViewDataObject(FileObject fileObject, MultiFileLoader loader)
      throws DataObjectExistsException, IOException
  {
    super(fileObject, loader);
    // TODO[cc]: Is there an easier way to make this type check?
    @SuppressWarnings("unchecked")
    final Class<? extends Cookie>[] cookieTypes = (Class<? extends Cookie>[])
        new Class<?>[] { ViewOpenSupport.class, ViewResultProvider.class };
    // See org.netbeans.modules.image.ImageDataObject.
    /* While CookieSet is outdated, I believe it needs to be used here for compatibility reasons;
    adding a cookie to a CookieSet will make sure the cookie is also available in the Lookup
    returned by getLookup(). */
    getCookieSet().add(cookieTypes, new CookieSet.Factory() {
      private ViewOpenSupport openSupport;
      private ViewResultProvider resultProvider;

      // This method may be called from threads other than the EDT.
      @Override
      @SuppressWarnings("unchecked")
      public synchronized <T extends Cookie> T createCookie(Class<T> clazz) {
        if (clazz.isAssignableFrom(ViewOpenSupport.class)) {
          if (openSupport == null)
            openSupport = new ViewOpenSupport(ViewDataObject.this);
          return (T) openSupport;
        } else if (clazz.isAssignableFrom(ViewResultProvider.class)) {
          if (resultProvider == null)
            resultProvider = new ViewResultProvider(ViewDataObject.this);
          return (T) resultProvider;
        } else {
          return null;
        }
      }
    });
  }

  public static ViewDataObject forFileObject(FileObject fileObject) throws IOException {
    Preconditions.checkNotNull(fileObject);
    /* "The usage pattern is to ask a DataObject for instances of interfaces that are the things
    your code will actually interact with, by calling
    DataObject.getLookup().lookup(SomeType.class)". http://wiki.netbeans.org/DevFaqDataObject */
    ViewDataObject ret = DataObject.find(fileObject).getLookup().lookup(ViewDataObject.class);
    if (ret == null)
      throw new IOException("FileObject did not resolve to a ViewDataObject");
    return ret;
  }

  @Override
  protected int associateLookup() {
    // No need to override getLookup(), just return 1 here.
    return 1;
  }

  // The user won't be looking at the DataNode directly, so I'm not overriding createNodeDelegate().
}
