// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.viewfile;

import com.google.common.base.Preconditions;
import java.math.BigInteger;
import java.net.URI;
import java.security.SecureRandom;
import java.util.Objects;

public final class ViewIdentifier {
  private static final SecureRandom RANDOM = new SecureRandom();
  private final String stringRep;

  private ViewIdentifier(String stringRep) {
    Preconditions.checkNotNull(stringRep);
    this.stringRep = stringRep;
  }

  public static ViewIdentifier newTransient() {
    synchronized (RANDOM) {
      // See http://stackoverflow.com/questions/41107/how-to-generate-a-random-alpha-numeric-string
      return new ViewIdentifier("sieuferd://transient/" + new BigInteger(160, RANDOM).toString(32));
    }
  }

  public static ViewIdentifier fromURI(URI uri) {
    return new ViewIdentifier(uri.toString());
  }

  public static ViewIdentifier fromStringRepresentation(String str) {
    return new ViewIdentifier(str);
  }

  @Override
  public String toString() {
    return stringRep;
  }

  @Override
  public int hashCode() {
    return stringRep.hashCode();
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof ViewIdentifier))
      return false;
    ViewIdentifier other = (ViewIdentifier) obj;
    return Objects.equals(this.stringRep, other.stringRep);
  }
}
