// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.project.formulabar;

import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import com.sieuferd.datamodel.expressions.ExpressionTokenType;
import com.sieuferd.util.StringUtil;
import java.util.Collections;
import java.util.ResourceBundle;
import java.util.Set;
import javax.swing.text.AttributeSet;
import org.junit.Assert;
import org.junit.Test;
import org.netbeans.api.editor.mimelookup.MimeLookup;
import org.netbeans.api.editor.mimelookup.MimePath;
import org.netbeans.api.editor.settings.FontColorSettings;
import org.openide.util.NbBundle;

public class ExpressionHighlightingTest {
  // TODO[te]: Run analogous tests on DataValue as well.

  /** Test the mapping between ExpressionTokenId and ExpressionTokenType. */
  @Test
  public void testTokenMapping() {
    final Set<ExpressionTokenId> foundExpressionTokenIds = Sets.newLinkedHashSet();
    foundExpressionTokenIds.add(ExpressionTokenId.PATH_REFERENCE_OPEN_BRACKET);
    foundExpressionTokenIds.add(ExpressionTokenId.PATH_REFERENCE_CORE);
    foundExpressionTokenIds.add(ExpressionTokenId.PATH_REFERENCE_CLOSE_BRACKET);
    for (ExpressionTokenType tokenType : ExpressionTokenType.values()) {
      if (!(tokenType == ExpressionTokenType.PATH_REFERENCE ||
          tokenType == ExpressionTokenType.UNTERMINATED_PATH_REFERENCE))
      {
        final ExpressionTokenId id = ExpressionLexer.tokenIdFromType(tokenType);
        if (tokenType.equals(ExpressionTokenType.UNTERMINATED_LITERAL_STRING)) {
          Assert.assertEquals(ExpressionTokenType.LITERAL_STRING.name(), id.name());
        } else if (tokenType.equals(ExpressionTokenType.UNTERMINATED_LITERAL_TEMPORAL)) {
          Assert.assertEquals(ExpressionTokenType.LITERAL_TEMPORAL.name(), id.name());
        } else {
          Assert.assertEquals(tokenType.name(), id.name());
        }
        foundExpressionTokenIds.add(id);
      }
    }
    Assert.assertEquals(ImmutableSet.copyOf(ExpressionTokenId.values()), foundExpressionTokenIds);
  }

  @Test
  public void testTokenCategories() {
    Set<String> primaryCategories = Sets.newLinkedHashSet();
    for (ExpressionTokenId tokenId : ExpressionTokenId.values())
      primaryCategories.add(tokenId.primaryCategory());
    ResourceBundle tokenBundle =
        NbBundle.getBundle("com/sieuferd/project/formulabar/TokenBundle");
    final Set<String> categoriesInTokenBundle =
        ImmutableSet.copyOf(Collections.list(tokenBundle.getKeys()));
    for (String key : categoriesInTokenBundle) {
      if (StringUtil.isEmptyOrSpaceOnly(tokenBundle.getString(key)))
        throw new AssertionError("Empty display value in token bundle for key " + key);
    }
    Assert.assertEquals(primaryCategories, categoriesInTokenBundle);
    MimePath mimePath = MimePath.parse(ExpressionLanguageConfig.MIME_TYPE);
    FontColorSettings fcs = MimeLookup.getLookup(mimePath).lookup(FontColorSettings.class);
    Set<String> missingCategorySettings = Sets.newLinkedHashSet();
    for (String primaryCategory : primaryCategories) {
      AttributeSet as = fcs.getTokenFontColors(primaryCategory);
      if (as == null)
        missingCategorySettings.add(primaryCategory);
    }
    if (!missingCategorySettings.isEmpty()) {
      throw new AssertionError("Missing token coloring setting for primaryCategory keys: " +
          missingCategorySettings);
    }
  }
}
