// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableList;
import java.awt.event.KeyEvent;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.KeyStroke;
import org.netbeans.swing.outline.Outline;

public final class EnterFilterInputMap extends FilterInputMap {
  private final KeyStroke ENTER = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0);
  private final JComponent focusComponent;

  private EnterFilterInputMap(JComponent focusComponent) {
    Preconditions.checkNotNull(focusComponent);
    this.focusComponent = focusComponent;
  }

  /**
   * Adjust the input map of an {@link Outline} or {@code JList} component such that the Enter key
   * can be used to close the dialog under correct circumstances. If an embedded "quick search" pane
   * is in focus, it can still be closed with Enter without also closing the dialog.
   */
  public static final void configure(JComponent focusComponent) {
    for (int condition :
        ImmutableList.of(JComponent.WHEN_FOCUSED, JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT))
    {
      InputMap existingInputMap = focusComponent.getInputMap(condition);
      if (!(existingInputMap instanceof EnterFilterInputMap)) {
        InputMap filterInputMap = new EnterFilterInputMap(focusComponent);
        filterInputMap.setParent(existingInputMap);
        focusComponent.setInputMap(condition, filterInputMap);
      }
    }
  }

  @Override
  protected boolean excludesKeyStroke(KeyStroke keyStroke) {
    return keyStroke.equals(ENTER) && focusComponent.isFocusOwner();
  }
}
