// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil;

import javax.swing.ActionMap;
import javax.swing.JPanel;
import javax.swing.text.DefaultEditorKit;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.util.Lookup;

public class ExplorerManagerProviderPanel extends JPanel
    implements ExplorerManager.Provider, Lookup.Provider
{
  private final ExplorerManager explorerManager = new ExplorerManager();
  private final Lookup lookup;
  private boolean actionsActivated = true;

  public ExplorerManagerProviderPanel() {
    ActionMap map = this.getActionMap();
    map.put(DefaultEditorKit.copyAction, ExplorerUtils.actionCopy(explorerManager));
    map.put(DefaultEditorKit.cutAction, ExplorerUtils.actionCut(explorerManager));
    map.put(DefaultEditorKit.pasteAction, ExplorerUtils.actionPaste(explorerManager));
    map.put("delete", ExplorerUtils.actionDelete(explorerManager, true));
    lookup = ExplorerUtils.createLookup(explorerManager, map);
  }

  @Override
  public final ExplorerManager getExplorerManager() {
    return explorerManager;
  }

  public void setActionsActivated(boolean actionsActivated) {
    ThreadUtil.checkOnEDT();
    if (actionsActivated != this.actionsActivated) {
      ExplorerUtils.activateActions(getExplorerManager(), actionsActivated);
      this.actionsActivated = actionsActivated;
    }
  }

  /* Providing an ExplorerManager without a Lookup can lead to odd behavior, such as actions invoked
  from a node's context menu being applied to the node currently selected in the global lookup
  instead. For this reason, this class implements Lookup.Provider as well. */

  @Override
  public final Lookup getLookup() {
    return lookup;
  }
}
