// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import java.util.List;
import javax.annotation.Nullable;
import javax.swing.InputMap;
import javax.swing.KeyStroke;

public abstract class FilterInputMap extends InputMap {
  public FilterInputMap() {
  }

  protected abstract boolean excludesKeyStroke(KeyStroke keyStroke);

  @Override
  public KeyStroke[] allKeys() {
    return subtractKeysArray(super.allKeys());
  }

  @Override
  public KeyStroke[] keys() {
    return subtractKeysArray(super.keys());
  }

  @Override
  public int size() {
    return subtractKeysList(super.keys()).size();
  }

  private List<KeyStroke> subtractKeysList(@Nullable KeyStroke[] originalKeys) {
    if (originalKeys == null)
      return ImmutableList.of();
    final List<KeyStroke> ret = Lists.newArrayListWithCapacity(originalKeys.length);
    for (KeyStroke key : originalKeys) {
      if (!excludesKeyStroke(key))
        ret.add(key);
    }
    return ret;
  }

  private @Nullable KeyStroke[] subtractKeysArray(@Nullable KeyStroke[] originalKeys) {
    final List<KeyStroke> ret = subtractKeysList(originalKeys);
    /* Using nulls here instead of an empty array is a bad convention, but it seems to be how this
    API works. */
    if (ret.isEmpty())
      return null;
    return ret.toArray(new KeyStroke[ret.size()]);
  }

  @Override
  public Object get(KeyStroke keyStroke) {
    Object ret = super.get(keyStroke);
    return (ret == null || excludesKeyStroke(keyStroke)) ? null : ret;
  }
}
