// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.text.MessageFormat;
import java.util.function.Function;
import javax.swing.JList;
import javax.swing.UIManager;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

/* See org.netbeans.modules.apisupport.project.ui.customizer.JavaPlatformComponentFactory.Renderer .
An alternative implementation used org.openide.awt.HtmlRenderer instead (like
org.netbeans.modules.java.api.common.ui.PlatformUiSupport ), but in this case baseline alignment did
not work properly. Also, extend from BasicComboBoxRenderer instead of rolling my own, as I had
rendering problems on Windows otherwise. */
public final class TypedComboBoxCellRenderer<T> extends BasicComboBoxRenderer {
  private final Color errorForeground = UIManager.getColor("nb.errorForeground");
  private final TypedComboBoxModel<T> model;
  private final String missingFmt;
  private final Function<T,String> stringMapper;
  private Color foregroundColor = getForeground();
  private boolean errorColor = false;

  public TypedComboBoxCellRenderer(
      TypedComboBoxModel<T> model, String missingFmt, Function<T,String> stringMapper)
  {
    this.model = model;
    this.missingFmt = missingFmt;
    this.stringMapper = stringMapper;
  }

  @Override
  /* Hard to avoid this warning (about the JList parameter) as long as I need to extend from
  BasicComboBoxRenderer, which is not generic. */
  @SuppressWarnings("rawtypes")
  public Component getListCellRendererComponent(
      JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
  {
    String formatted;
    final T typedValue = model.castValue(value);
    final boolean toErrorColor;
    if (value == null) {
      toErrorColor = false;
      formatted = "";
    } else {
      String mapped = stringMapper.apply(typedValue);
      // Just because Function.apply() is annoyingly marked as @Nullable.
      if (mapped == null)
        mapped = "";
      if (model.itemMissing(typedValue)) {
        toErrorColor = true;
        formatted = MessageFormat.format(missingFmt, mapped);
      } else {
        toErrorColor = false;
        formatted = mapped;
      }
    }
    errorColor = toErrorColor;
    // Deal with some formatting issues related to empty strings.
    if (formatted.isEmpty())
      formatted = " ";
    setForeground(foregroundColor);
    // TODO[pd]: Find a better way to do this.
    /* Avoid having extremely long strings impact the preferred width of the ComboBox. This was a
    problem for the DataSourceCustomizer, as the containing panel would introduce scrollbars when
    a long JDBC URL was initially selected. */
    Component ret =
        super.getListCellRendererComponent(list, formatted, index, isSelected, cellHasFocus);
    Dimension oldPreferredSize = ret.getPreferredSize();
    ret.setPreferredSize(
        new Dimension(Math.min(oldPreferredSize.width, 100), oldPreferredSize.height));
    return ret;
  }

  @Override
  public void setForeground(Color fg) {
    this.foregroundColor = fg;
    super.setForeground(errorColor ? errorForeground : foregroundColor);
  }
}
