// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import java.util.List;
import java.util.Set;
import javax.annotation.Nullable;
import javax.swing.AbstractListModel;
import javax.swing.ComboBoxModel;

// TODO[cc]: Replace with DefaultComboBoxModel, since the latter has been made generic as of Java 7.
public final class TypedComboBoxModel<T> extends AbstractListModel<T> implements ComboBoxModel<T> {
  private final Class<T> type;
  private ImmutableList<T> contents = ImmutableList.of();
  /** Order is significant. */
  private @Nullable ImmutableSet<T> externalItems = ImmutableSet.of();
  private @Nullable T initialSelection;
  private @Nullable T selectedItem;

  public TypedComboBoxModel(Class<T> type) {
    this.type = type;
    refresh();
  }

  /** @param externalItems can be null to indicate that the list has not yet been loaded, in
             which case itemMissing() will always return false. */
  public void setExternalItems(@Nullable List<T> externalItems) {
    this.externalItems = ImmutableSet.copyOf(externalItems);
    refresh();
  }

  public ImmutableSet<T> getExternalItems() {
    return externalItems;
  }

  public boolean itemMissing(T item) {
    return externalItems != null && !externalItems.contains(item);
  }

  /** Define a single item that will always be available in the list. */
  public void setInitialSelection(@Nullable T initialSelection) {
    this.initialSelection = initialSelection;
    refresh();
  }

  public @Nullable T getInitialSelection() {
    return initialSelection;
  }

  /** Will always fire contents changed events. */
  private void refresh() {
    final int previousSize = contents.size();
    Set<T> toContents = Sets.newLinkedHashSet();
    if (externalItems != null)
      toContents.addAll(externalItems);
    // Ensure that past selections are available even if the connections have been deleted.
    if (selectedItem != null)
      toContents.add(selectedItem);
    if (initialSelection != null)
      toContents.add(initialSelection);
    contents = ImmutableList.copyOf(toContents);
    fireContentsChanged(this, 0, previousSize);
  }

  @Override
  public int getSize() {
    return contents.size();
  }

  @Override
  public T getElementAt(int index) {
    return contents.get(index);
  }

  @SuppressWarnings("unchecked")
  public T castValue(Object value) {
    if (value != null && !type.isAssignableFrom(value.getClass()))
      throw new IllegalArgumentException("Expected combo box item of type " + type.getName());
    return (T) value;
  }

  public void setSelectedItemTyped(@Nullable T newSelection) {
    selectedItem = newSelection;
    refresh();
  }

  /**
   * @param newSelection must be a String. The new selection does not necessarily have to already be
   *        in the list.
   */
  @Override
  public void setSelectedItem(@Nullable Object newSelection) {
    setSelectedItemTyped(castValue(newSelection));
  }

  @Override
  public @Nullable T getSelectedItem() {
    return selectedItem;
  }
}
