// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil.flow;

import com.google.common.base.Preconditions;
import java.awt.Component;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

final class FocusedFlow extends EDTAdapterFlow<Boolean> {
  private final Component component;
  private final FocusListener focusListener = new FocusListener() {
    @Override
    public void focusGained(FocusEvent e) {
      markModelUpdated();
    }

    @Override
    public void focusLost(FocusEvent e) {
      markModelUpdated();
    }
  };

  FocusedFlow(Component component) {
    super("AWTFocused");
    Preconditions.checkNotNull(component);
    this.component = component;
  }

  @Override
  protected void addModelListener() {
    component.addFocusListener(focusListener);
  }

  @Override
  protected void removeModelListener() {
    component.removeFocusListener(focusListener);
  }

  @Override
  protected Boolean getModelValue() {
    return component.isFocusOwner();
  }
}
