// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil.flow;

import com.google.common.base.Preconditions;
import com.sieuferd.flow.ModelAdapterFlow;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;

final class LookupFlow<L> extends ModelAdapterFlow<LookupResultSnapshot<L>> {
  private final Class<L> valueClass;
  /* Keep strong references to the Lookup, the Lookup.Result and the LookupListener to avoid
  problems relating to weak references. See the Javadoc for Lookup.addLookupListener. */
  private final Lookup lookup;
  private final Lookup.Result<L> lookupResult;

  private final LookupListener lookupListener = (LookupEvent evt) -> {
    markModelUpdated();
  };

  LookupFlow(Lookup lookup, Lookup.Template<L> template) {
    super("Lookup");
    Preconditions.checkNotNull(lookup);
    this.lookup = lookup;
    this.valueClass = template.getType();
    this.lookupResult = lookup.lookup(template);
  }

  @Override
  protected void addModelListener() {
    lookupResult.addLookupListener(lookupListener);
  }

  @Override
  protected void removeModelListener() {
    lookupResult.removeLookupListener(lookupListener);
  }

  @Override
  protected LookupResultSnapshot<L> getModelValue() {
    return LookupResultSnapshot.create(valueClass, lookupResult);
  }
}
