// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil.flow;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import org.openide.util.Lookup;
import org.openide.util.Lookup.Item;

/** Shallowly immutable. */
public final class LookupResultSnapshot<L> {
  private final ImmutableList<? extends Lookup.Item<L>> allItems;

  public LookupResultSnapshot(
      Class<L> valueClass, Collection<? extends Lookup.Item<L>> allItems)
  {
    Preconditions.checkNotNull(valueClass);
    for (Lookup.Item<L> item : allItems) {
      Preconditions.checkArgument(valueClass.isAssignableFrom(item.getType()));
      /* Don't check the actual instance, in case it is lazily created. Probably rare, but the
      Javadoc for Lookup.Item.getInstance() seems to suggest it might be the case. */
    }
    this.allItems = ImmutableList.copyOf(allItems);
  }

  public Iterable<L> allInstances() {
    // TODO[pd]: Consider returning a lazy Iterable instead.
    List<L> ret = Lists.newArrayList();
    for (Item<L> item : allItems()) {
      L instance = item.getInstance();
      if (instance != null)
        ret.add(instance);
    }
    return Collections.unmodifiableList(ret);
  }

  public Collection<? extends Lookup.Item<L>> allItems() {
    return allItems;
  }

  @Override
  public String toString() {
    return "LookupResultSnapshot";
  }

  public static <L> LookupResultSnapshot<L> create(
      Class<L> valueClass, Lookup.Result<L> lookupResult)
  {
    return new LookupResultSnapshot<>(valueClass, lookupResult.allItems());
  }
}
