// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil.flow;

import com.google.common.base.Preconditions;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

final class PropertyBeanFlow<T> extends EDTAdapterFlow<T> {
  private final PropertyBeanAccessor<T> accessor;

  private final PropertyChangeListener propertyChangeListener = new PropertyChangeListener() {
    @Override
    public void propertyChange(PropertyChangeEvent evt) {
      if (evt.getPropertyName() == null || accessor.getPropertyName().equals(evt.getPropertyName()))
        markModelUpdated();
    }
  };

  PropertyBeanFlow(PropertyBeanAccessor<T> accessor) {
    super("PropertyBean");
    Preconditions.checkNotNull(accessor);
    this.accessor = accessor;
  }

  @Override
  protected void addModelListener() {
    accessor.addPropertyChangeListener(propertyChangeListener);
  }

  @Override
  protected void removeModelListener() {
    accessor.removePropertyChangeListener(propertyChangeListener);
  }

  @Override
  protected T getModelValue() {
    return accessor.getPropertyValue();
  }
}
