// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil.flow;

import com.google.common.base.Preconditions;
import com.sieuferd.uiutil.UIUtil;
import java.util.logging.Logger;
import javax.annotation.Nullable;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;

/**
 * Flow adapter for Swing's {@link javax.swing.text.Document} class.
 */
final class SwingDocumentFlow extends EDTAdapterFlow<String> {
  private static final Logger LOG = Logger.getLogger(SwingDocumentFlow.class.getName());
  private final Document document;

  private final DocumentListener documentListener = new DocumentListener() {
    @Override
    public void insertUpdate(DocumentEvent e) {
      markModelUpdated();
    }

    @Override
    public void removeUpdate(DocumentEvent e) {
      markModelUpdated();
    }

    @Override
    public void changedUpdate(DocumentEvent e) {
      markModelUpdated();
    }
  };

  SwingDocumentFlow(Document document) {
    super("SwingDocument");
    Preconditions.checkNotNull(document);
    this.document = document;
  }

  @Override
  protected void addModelListener() {
    document.addDocumentListener(documentListener);
  }

  @Override
  protected void removeModelListener() {
    document.removeDocumentListener(documentListener);
  }

  @Override
  protected @Nullable String getModelValue() {
    return UIUtil.getDocumentText(document);
  }
}
