// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil.table;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import com.sieuferd.uiutil.api.TableColors;
import java.awt.Component;
import java.util.EventObject;
import java.util.Set;
import javax.swing.JTable;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.table.TableCellEditor;

// TODO[cc]: Consider getting rid of this.
public class ColorOverrideEditor implements TableCellEditor {
  private final TableCellEditor delegate;
  private final TableColors colors;
  // Synchronized on itself.
  private final Set<CellEditorListener> listeners = Sets.newLinkedHashSet();
  // Ensure that ChangeEvent objects have the delegate as the source.
  private final CellEditorListener listener = new CellEditorListener() {
    @Override
    public void editingStopped(ChangeEvent e) {
      ChangeEvent delegatedEvent = new ChangeEvent(this);
      for (CellEditorListener listener : getListenersSnapshot())
        listener.editingStopped(delegatedEvent);
    }

    @Override
    public void editingCanceled(ChangeEvent e) {
      ChangeEvent delegatedEvent = new ChangeEvent(this);
      for (CellEditorListener listener : getListenersSnapshot())
        listener.editingCanceled(delegatedEvent);
    }
  };

  private ImmutableSet<CellEditorListener> getListenersSnapshot() {
    synchronized (listeners) {
      return ImmutableSet.copyOf(listeners);
    }
  }

  public ColorOverrideEditor(TableCellEditor delegate, TableColors colors) {
    Preconditions.checkNotNull(delegate);
    this.delegate = delegate;
    this.colors = colors;
  }

  @Override
  public Component getTableCellEditorComponent(
      JTable table, Object value, boolean isSelected, int row, int column)
  {
    Component ret = delegate.getTableCellEditorComponent(table, value, isSelected, row, column);
    ret.setForeground(colors.getForeground(table, isSelected));
    ret.setBackground(colors.getBackground(table, isSelected));
    return ret;
  }

  @Override
  public Object getCellEditorValue() {
    return delegate.getCellEditorValue();
  }

  @Override
  public boolean isCellEditable(EventObject anEvent) {
    return delegate.isCellEditable(anEvent);
  }

  @Override
  public boolean shouldSelectCell(EventObject anEvent) {
    return delegate.shouldSelectCell(anEvent);
  }

  @Override
  public boolean stopCellEditing() {
    return delegate.stopCellEditing();
  }

  @Override
  public void cancelCellEditing() {
    delegate.cancelCellEditing();
  }

  @Override
  public void addCellEditorListener(CellEditorListener l) {
    synchronized (listeners) {
      if (listeners.isEmpty())
        delegate.addCellEditorListener(listener);
      listeners.add(l);
    }
  }

  @Override
  public void removeCellEditorListener(CellEditorListener l) {
    synchronized (listeners) {
      listeners.remove(l);
      if (listeners.isEmpty())
        delegate.removeCellEditorListener(listener);
    }
  }
}
