// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil.table;

import com.google.common.base.Preconditions;
import com.sieuferd.uiutil.api.TableColors;
import java.awt.Component;
import javax.annotation.Nullable;
import javax.swing.ButtonModel;
import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;
import org.openide.util.Utilities;

// TODO[cc]: Consider renaming now that I also do some magic with checkboxes.
public class ColorOverrideRenderer implements TableCellRenderer {
  private final TableCellRenderer delegate;
  private final TableColors colors;

  public ColorOverrideRenderer(TableCellRenderer delegate, TableColors colors) {
    Preconditions.checkNotNull(delegate);
    Preconditions.checkNotNull(colors);
    this.delegate = delegate;
    this.colors = colors;
  }

  @Override
  public Component getTableCellRendererComponent(JTable table, @Nullable Object value,
      boolean isSelected, boolean hasFocus, int row, int column)
  {
    /* Workaround for a bug on Aqua where OutlineView paints black backgrounds for "lead selection"
    cells that are focused but not selected; this state can be reached by deleting a row or by
    unselecting a row in a multiple selection. */
    final boolean useFocus = Utilities.isMac() ? isSelected && hasFocus : hasFocus;
    Component ret = delegate.getTableCellRendererComponent(
        table, value, isSelected, useFocus, row, column);
    ret.setForeground(colors.getForeground(table, isSelected));
    ret.setBackground(colors.getBackground(table, isSelected));
    if ((ret instanceof JCheckBox)) {
      // Properly gray out un-editable checkboxes.
      ret.setEnabled(table.getModel().isCellEditable(row, column));
      /* Don't paint the focus border (see JTable.BooleanRenderer). Not painting the border seems
      safer than removing the border altogether. I've confirmed that the checkmark can still be
      painted in the space taken by the unpainted border. */
      ((JCheckBox) ret).setBorderPainted(false);
      /* Support tri-state checkboxes. See
      o.n.swing.outline.DefaultOutlineCellRenderer.setUpCheckBox. */
      ButtonModel bm = ((JCheckBox) ret).getModel();
      bm.setSelected(!Boolean.FALSE.equals(value));
      bm.setArmed(value == null);
      bm.setPressed(value == null);
    }
    return ret;
  }
}
