// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.uiutil;

import static com.sieuferd.uiutil.UIUtil.blendOver;
import java.awt.Color;
import java.awt.font.TextAttribute;
import java.text.AttributedString;
import org.junit.Assert;
import org.junit.Test;

public class UIUtilTest {
  public UIUtilTest() {
  }

  @Test
  public void testBlendOver() {
    final Color nanColor = new Color(Float.NaN, Float.NaN, Float.NaN, 0.0f);
    Assert.assertEquals(Color.GRAY, blendOver(nanColor, Color.GRAY));
    Assert.assertEquals(Color.BLUE, blendOver(nanColor, Color.BLUE));
    Assert.assertEquals(Color.GRAY, blendOver(UIUtil.COLOR_NONE, Color.GRAY));
    Assert.assertEquals(Color.BLUE, blendOver(UIUtil.COLOR_NONE, Color.BLUE));
    Assert.assertEquals(Color.BLUE, blendOver(Color.BLUE, UIUtil.COLOR_NONE));
    Assert.assertEquals(Color.BLUE, blendOver(Color.BLUE, Color.RED));
    Assert.assertEquals(UIUtil.COLOR_NONE, blendOver(UIUtil.COLOR_NONE, UIUtil.COLOR_NONE));
    Assert.assertEquals(UIUtil.COLOR_NONE, blendOver(nanColor, nanColor));
    Assert.assertEquals(UIUtil.COLOR_NONE, blendOver(nanColor, UIUtil.COLOR_NONE));
    Assert.assertEquals(UIUtil.COLOR_NONE, blendOver(UIUtil.COLOR_NONE, nanColor));
  }

  private static String astos(AttributedString as) {
    return SwingString.attributed(as).getForConcatenation();
  }

  @Test
  public void testAttributedStringToSwingHTML() {
    Assert.assertEquals("", astos(new AttributedString("")));
    Assert.assertEquals("abc", astos(new AttributedString("abc")));
    Assert.assertEquals("&lt;abc&gt;", astos(new AttributedString("<abc>")));
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE);
      Assert.assertEquals("<i>abcd</i>", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 0, 4);
      Assert.assertEquals("<i>abcd</i>", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 0, 1);
      Assert.assertEquals("<i>a</i>bcd", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 1, 2);
      Assert.assertEquals("a<i>b</i>cd", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 3, 4);
      Assert.assertEquals("abc<i>d</i>", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 2, 4);
      Assert.assertEquals("ab<i>cd</i>", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 0, 3);
      Assert.assertEquals("<i>abc</i>d", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 1, 4);
      Assert.assertEquals("a<i>bcd</i>", astos(as));
    }
    {
      AttributedString as = new AttributedString("a<cd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 1, 4);
      Assert.assertEquals("a<i>&lt;cd</i>", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 0, 1);
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 2, 3);
      Assert.assertEquals("<i>a</i>b<i>c</i>d", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 0, 1);
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 1, 3);
      // AttributedString does not attempt to merge non-overlapping runs.
      Assert.assertEquals("<i>a</i><i>bc</i>d", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 0, 2);
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 1, 3);
      // AttributedString splits overlapping runs into individual adjacent runs.
      Assert.assertEquals("<i>a</i><i>b</i><i>c</i>d", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 1, 3);
      Assert.assertEquals("a<i>bc</i>d", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 1, 3);
      as.addAttribute(TextAttribute.FOREGROUND, Color.BLUE, 1, 3);
      Assert.assertEquals("a<font color='#0000FF'><i>bc</i></font>d", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 1, 3);
      as.addAttribute(TextAttribute.FOREGROUND, Color.BLUE, 1, 3);
      as.addAttribute(TextAttribute.BACKGROUND, Color.YELLOW, 1, 3);
      Assert.assertEquals("a<font color='#0000FF' style='background-color: #FFFF00'><i>bc</i></font>d", astos(as));
    }
    {
      AttributedString as = new AttributedString("abcd");
      as.addAttribute(TextAttribute.POSTURE, TextAttribute.POSTURE_OBLIQUE, 1, 3);
      as.addAttribute(TextAttribute.BACKGROUND, Color.YELLOW, 1, 3);
      Assert.assertEquals("a<font style='background-color: #FFFF00'><i>bc</i></font>d", astos(as));
    }
  }
}
