// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.widget;

import com.google.common.base.Joiner;
import com.google.common.base.Preconditions;
import com.google.common.collect.Lists;
import java.awt.event.ActionEvent;
import java.util.List;
import javax.annotation.Nullable;
import javax.swing.AbstractAction;
import javax.swing.Action;

// TODO[cc]: Get rid of this once all the actions have been transitioned to more specific actions.
final class CellWidgetAction extends AbstractAction {
  /** Positive dx always means right, regardless of component orientation. This means, for instance,
  that the "right" arrow key will move the cursor left if the component orientation is
  Right-to-Left. This is the actual behavior of the JTable in Right-to-Left mode, but I'm not sure
  if it's the best behavior. */
  private final int dx;
  private final int dy;
  private final boolean extend;
  private final boolean inSelection;
  /**
   * Following the convention from BasicTableUI, "horizontally, forwards always means right,
   * regardless of component orientation".
   */
  private final @Nullable Boolean forwards;
  private final @Nullable Boolean vertically;
  private final boolean toLimit;

  private CellWidgetAction(String name, int dx, int dy, boolean extend, boolean inSelection,
      @Nullable Boolean forwards, @Nullable Boolean vertically, boolean toLimit)
  {
    super(name);
    Preconditions.checkArgument(dx >= -1 && dx <= 1);
    Preconditions.checkArgument(dy >= -1 && dy <= 1);
    this.dx = dx;
    this.dy = dy;
    this.extend = extend;
    this.inSelection = inSelection;
    this.forwards = forwards;
    this.vertically = vertically;
    this.toLimit = toLimit;
  }

  CellWidgetAction(String name) {
    this(name, 0, 0, false, false, false, false, false);
  }

  CellWidgetAction(String name, boolean extend, boolean forwards, boolean vertically,
      boolean toLimit)
  {
    this(name, 0, 0, extend, false, forwards, vertically, toLimit);
  }

  CellWidgetAction(String name, int dx, int dy, boolean extend, boolean inSelection) {
    this(name, dx, dy, extend, inSelection, null, null, false);
    // Similar to the invariants maintained by BasicTableUI.
    Preconditions.checkArgument((dx == 0 || dy == 0) && !(dx == 0 && dy == 0));
  }

  private String getName() {
    return (String) getValue(Action.NAME);
  }

  @Override
  public String toString() {
    List<String> ret = Lists.newArrayList();
    if (dx != 0)
      ret.add("dx=" + dx);
    if (dy != 0)
      ret.add("dy=" + dy);
    if (extend)
      ret.add("extend");
    if (inSelection)
      ret.add("inSelection");
    if (forwards != null)
      ret.add("forwards=" + forwards);
    if (vertically != null)
      ret.add("vertically=" + vertically);
    if (toLimit)
      ret.add("toLimit");
    return getName() + ": " + Joiner.on(", ").join(ret);
  }

  @Override
  public void actionPerformed(ActionEvent e) {
    System.out.println(toString() + " from " + e.getSource());
  }
}
