// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.widget;

import com.google.common.base.Preconditions;
import com.google.common.collect.Maps;
import java.util.Map;
import com.sieuferd.cell.support.Cell;
import com.sieuferd.cell.support.Coordinates;
import com.sieuferd.cell.support.RenderContext;
import com.sieuferd.datamodel.ViewResult;
import javax.annotation.concurrent.Immutable;

// TODO[cc]: Rename this, since it's not a subclass of Cell.
/** Immutable except for a debugging field. */
@Immutable
public final class GeneratedCell {
  private final ViewResult result;
  private final Cell cell;
  private final RenderContext renderContext;
  private final Map<String,Coordinates> debugLabels;
  private final boolean tracksViewportWidth, tracksViewportHeight;

  private GeneratedCell(ViewResult result, Cell cell, RenderContext renderContext,
      boolean tracksViewportWidth, boolean tracksViewportHeight,
      Map<String,Coordinates> debugLabels)
  {
    Preconditions.checkNotNull(result);
    Preconditions.checkNotNull(cell);
    Preconditions.checkNotNull(renderContext);
    cell.checkRenderContext(renderContext);
    this.result = result;
    this.cell = cell;
    this.renderContext = renderContext;
    this.debugLabels = Maps.newLinkedHashMap(debugLabels);
    this.tracksViewportWidth = tracksViewportWidth;
    this.tracksViewportHeight = tracksViewportHeight;
  }

  public GeneratedCell(ViewResult result, Cell cell, RenderContext renderContext,
      boolean tracksViewportWidth, boolean tracksViewportHeight)
  {
    this(result, cell, renderContext, tracksViewportWidth, tracksViewportHeight,
        Maps.newLinkedHashMap());
  }

  public ViewResult getResult() {
    return result;
  }

  public Cell getCell() {
    return cell;
  }

  public RenderContext getRenderContext() {
    return renderContext;
  }

  public boolean getTracksViewportWidth() {
    return tracksViewportWidth;
  }

  public boolean getTracksViewportHeight() {
    return tracksViewportHeight;
  }

  Coordinates getActualDimensions() {
    return cell.getDimensionsWithOverprint(renderContext);
  }

  public Map<String,Coordinates> getDebugLabels() {
    return debugLabels;
  }
}
