// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.csv;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import com.sieuferd.csv.Output.PostponingOutput;
import com.sieuferd.csv.ProxyParser.Proxy;
import java.io.IOException;
import java.util.List;

public class BaseParser extends Parser {
  private final ImmutableList<String> currentRow;
  /** Includes quote characters. */
  private final String currentField;

  public BaseParser(List<String> currentRow, String currentField) {
    Preconditions.checkNotNull(currentRow);
    Preconditions.checkNotNull(currentField);
    this.currentRow = ImmutableList.copyOf(currentRow);
    this.currentField = currentField;
  }

  @Override
  public int getBranchCount() {
    return 1;
  }

  @Override
  public Parser advance(final Config config, final Output output, final char c)
      throws IOException, BadParseException
  {
    final boolean isNewline = (c == '\n' || c == '\r');
    final boolean isFieldSeparator = (c == config.getFieldSeparator());
    //final boolean isQuote = (c == config.getQuote());
    //final boolean isSpace = (c == ' ');

    if (isNewline || isFieldSeparator) {
      Proxy asFieldPartProxy =
          new Proxy(new BaseParser(currentRow, currentField + c), new PostponingOutput());

      // Parser branch that interprets the character as a separator.
      PostponingOutput asSeparatorOutput = new PostponingOutput();
      List<String> asSeparatorCurrentRow = Lists.newArrayList(currentRow);
      //if (!(currentField.startsWith("\"") && currentField.endsWith("\"")))
      //  throw new BadParseException("Not a quoted field");
      //System.out.println(currentField);
      if (!(isNewline && asSeparatorCurrentRow.isEmpty()))
        asSeparatorCurrentRow.add(currentField);
      if (isNewline && !asSeparatorCurrentRow.isEmpty()) {
        if (asSeparatorCurrentRow.size() < 10)
          throw new BadParseException("Bad size " + asSeparatorCurrentRow);
        //System.out.println(asSeparatorCurrentRow.size());
        asSeparatorOutput.emitRow(asSeparatorCurrentRow);
        asSeparatorCurrentRow.clear();
      }
      Proxy asSeparatorProxy =
          new Proxy(new BaseParser(asSeparatorCurrentRow, ""), asSeparatorOutput);
      //return new ProxyParser(ImmutableList.of(asFieldPartProxy, asSeparatorProxy));
      return new FallbackParser(asSeparatorProxy, asFieldPartProxy);
      //return asSeparatorProxy.parser;
    } else {
      // Part of a field (even if quote).
      return new BaseParser(currentRow, currentField + c);
    }
  }

  @Override
  public Parser flush(Output output) {
    // No action needed.
    return this;
  }
}
