// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.csv;

import com.google.common.base.Preconditions;
import com.sieuferd.csv.ProxyParser.Proxy;
import java.io.IOException;

public class FallbackParser extends Parser {
  private final int ROW_COUNT_THRESHOLD = 3;
  private final Proxy principalProxy;
  private final Proxy fallbackProxy;
  private final StringBuilder piledInput = new StringBuilder();

  public FallbackParser(Proxy principalProxy, Proxy fallbackProxy) {
    Preconditions.checkNotNull(principalProxy);
    Preconditions.checkNotNull(fallbackProxy);
    this.principalProxy = principalProxy;
    this.fallbackProxy = fallbackProxy;
  }

  @Override
  public int getBranchCount() {
    return principalProxy.parser.getBranchCount();
  }

  @Override
  public Parser advance(Config config, Output output, char c) throws IOException, BadParseException
  {
    try {
      principalProxy.parser.advance(config, principalProxy.output, c);
    } catch (BadParseException e) {
      fallbackProxy.parser.flush(fallbackProxy.output);
      fallbackProxy.output.flushToOutput(output);
      for (int i = 0; i < piledInput.length(); i++)
        fallbackProxy.parser.advance(config, output, piledInput.charAt(i));
      return fallbackProxy.parser;
    }
    // TODO[pd]: Need to flush.
    if (principalProxy.output.getRowCount() >= ROW_COUNT_THRESHOLD) {
      return flush(output);
    } else {
      return this;
    }
  }

  @Override
  public Parser flush(Output output) {
    principalProxy.parser.flush(principalProxy.output);
    principalProxy.output.flushToOutput(output);
    return principalProxy.parser;
  }
}
