// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.csv;

import com.google.common.base.Preconditions;
import com.google.common.collect.Lists;
import com.sieuferd.csv.Output.PostponingOutput;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

public class ProxyParser extends Parser {
  private final List<Proxy> proxies;

  public ProxyParser(List<Proxy> proxies) {
    this.proxies = Lists.newArrayList(proxies);
  }

  @Override
  public int getBranchCount() {
    int ret = 0;
    for (Proxy proxy : proxies)
      ret += proxy.parser.getBranchCount();
    return ret;
  }

  @Override
  public Parser flush(Output output) {
    throw new UnsupportedOperationException();
  }

  public static final class Proxy {
    Parser parser;
    final PostponingOutput output;

    public Proxy(Parser parser, PostponingOutput output) {
      Preconditions.checkNotNull(parser);
      Preconditions.checkNotNull(output);
      this.parser = parser;
      this.output = output;
    }
  }

  @Override
  public Parser advance(final Config config, final Output output, final char c)
      throws IOException, BadParseException
  {
    final Iterator<Proxy> iter = proxies.iterator();
    while (iter.hasNext()) {
      final Proxy proxy = iter.next();
      try {
        proxy.parser = proxy.parser.advance(config, proxy.output, c);
        // TODO[cc]: Consider using a return value instead of an exception.
      } catch (BadParseException e) {
        iter.remove();
      }
    }
    int proxyParserCount = proxies.size();
    if (proxyParserCount == 0) {
      throw new BadParseException("No proxy parsers remaining");
    } else if (proxyParserCount == 1) {
      final Proxy lastProxyParser = proxies.get(0);
      lastProxyParser.output.flushToOutput(output);
      // Ensure an error message if this parser is used again.
      proxies.clear();
      return lastProxyParser.parser;
    }
    return this;
  }
}
