// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.harness;

import com.sieuferd.database.SQLConnection;
import com.sieuferd.importer.excel.ExcelImporter;
import com.sieuferd.project.util.ExcelClipboardUtil;
import com.sieuferd.query.SQLDatabase;
import com.sieuferd.util.ObjectReference;
import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.sql.DriverManager;
import java.sql.SQLException;
import javax.swing.SwingUtilities;
import org.openide.util.Utilities;

public final class ExcelClipboardImportDriver {
  public static final void main(String args[])
      throws SQLException, IOException, UnsupportedFlavorException, InterruptedException,
      InvocationTargetException
  {
    ObjectReference<InputStream> isRef = ObjectReference.create();
    SwingUtilities.invokeAndWait(() -> {
      final DataFlavor flavor = ExcelClipboardUtil.getExcelDataFlavor();
      try {
        isRef.setValue(
            (InputStream) Toolkit.getDefaultToolkit().getSystemClipboard().getData(flavor));
      } catch (UnsupportedFlavorException | IOException e) {
        throw new RuntimeException(e);
      }
    });
    PrintWriter pw = new PrintWriter(System.out, true);
    try (SQLConnection output = new SQLConnection(DriverManager.getConnection(
          "jdbc:postgresql://127.0.0.1:5432/managed", "duser", "dpass")))
    {
      (Utilities.isMac()
          ? ExcelImporter.MAC_EXCEL_CLIPBOARD_INSTANCE
          : ExcelImporter.WIN_EXCEL_CLIPBOARD_INSTANCE)
          .importClipboardStream(isRef.getValue(), output, new SQLDatabase("managed", ""), pw);
    } catch (SQLException e) {
      SQLException chain = e;
      while ((chain = chain.getNextException()) != null) {
        System.err.println("Chained SQL exception: " + chain.getMessage());
      }
      throw e;
    } finally {
      pw.flush();
    }
  }
}
