// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.harness;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.sql.DriverManager;
import java.sql.SQLException;
import com.sieuferd.datamodel.Data;
import com.sieuferd.viewset.SieuferdProjectXML;
import com.sieuferd.database.SQLConnection;
import com.sieuferd.datamodel.CProp;
import com.sieuferd.datamodel.MutableView;
import com.sieuferd.datamodel.PrettyPrinter;
import com.sieuferd.datamodel.ViewDefaults;
import com.sieuferd.datamodel.Views;
import com.sieuferd.evaluator.QueryUtil;
import com.sieuferd.evaluator.CancelHandler;
import com.sieuferd.evaluator.DataProperties;
import com.sieuferd.evaluator.QueryEvaluator;
import com.sieuferd.layout.DefaultStyleSheets;
import com.sieuferd.layout.LAProp;
import com.sieuferd.query.QProp;
import com.sieuferd.util.SieuferdXMLException;
import org.openide.util.Utilities;

public class LoadedXMLDriver {
  private static final boolean TEST_ALL = false;
  private static final boolean USE_POSTGRES = true;
  public static final ViewDefaults VIEW_DEFAULTS = ViewDefaults.builder()
      .set(CProp.VISIBLE, true)
      .set(CProp.LABEL, "")
      .setAll(QProp.DEFAULTS)
      .setAll(LAProp.DEFAULTS)
      .setAll(DefaultStyleSheets.GRAYSCALE)
      .build();
  public static final File QUERY_XML_DIR = new File(".");
  public static final File QUERY_XML_FILE = new File(QUERY_XML_DIR, "offerings.xml");

  private static void loadOneViewFile(File file, boolean printOutput)
      throws IOException, SieuferdXMLException, SQLException
  {
    MutableView view = Views.newMutable(SieuferdProjectXML.readSieuferdViewXML(
        new FileInputStream(file), Utilities.toURI(file).toString(),
        VIEW_DEFAULTS.getProperties()), VIEW_DEFAULTS);

    try (SQLConnection connection = new SQLConnection(
        DriverManager.getConnection(USE_POSTGRES
            ? "jdbc:postgresql://localhost:5432/princeton"
            : "jdbc:mysql://127.0.0.1:3306/", "duser", "dpass")))
    {
      Data data;
      QueryUtil.configurePrimaryKeys(connection, view);
      final QueryEvaluator evaluator =
          QueryEvaluator.create(view, connection, null, new CancelHandler());
      DataProperties.configureDataProperties(view);
      try {
        data = WidgetMain.evaluate(view, evaluator, Integer.MAX_VALUE);
      } finally {
        evaluator.close();
      }
      String prettyPrinted = PrettyPrinter.prettyPrint(view, data);
      if (printOutput) {
        //System.out.println(prettyPrinted);
        System.out.println("===== JSON =====");
        System.out.println(JSONPrinter.generateJSON(view, data, true));
        System.out.println("================");
        //System.out.println(Views.toLongString(view));
      }
    }
  }

  public static void main(String args[])
      throws IOException, SieuferdXMLException, SQLException
  {
    if (TEST_ALL) {
      for (File file : QUERY_XML_DIR.listFiles()) {
        System.out.println(file.getName());
        loadOneViewFile(file, true);
      }
    } else {
      loadOneViewFile(QUERY_XML_FILE, true);
    }
  }
}
