// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.harness;

import com.sieuferd.uiutil.UIUtil;
import com.sieuferd.uiutil.api.TableColors;
import com.sieuferd.uiutil.table.ColorOverrideRenderer;
import com.sieuferd.uiutil.table.SpanningTableCellRenderer;
import com.sieuferd.uiutil.table.ColorOverrideEditor;
import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import org.openide.util.Exceptions;

/* Based on
https://code.google.com/p/java-swing-tips/source/browse/trunk/ColumnSpanningCellRenderer/src/java/example/MainPanel.java?r=806 */
public class SpanningTableCellRendererTest extends JPanel {
  private static final String[] COLUMN_NAMES = {"Column A", "Column B", "Column C"};
  private static final Object[][] DATA = {
    {true , "Blah blah blahblahblah blah 1A" , "Blah blasdfasdfah blah 1B", "Blah blahasdfa blah 1C"       },
    {false, "Blah blah blah 2A"              , ""                         , "Blah blah sadfasdfdsafblah 2C"},
    {true , "Blah blah dasfas dfasdf blah 3A", ""                         , ""                             },
    {true , ""                               , "Blah basdfasdflah blah 4B", "Blah blasdfasdfah blah 4C"    },
    {false, "Blahsad fjaksldf jaklsdf jas 4A", ""                         , ""                             }
  };

  private final DefaultTableModel model = new DefaultTableModel(DATA, COLUMN_NAMES) {
    @Override
    public boolean isCellEditable(int row, int column) {
      return row % 2 == 0;
    }
  };

  private final JTable table = new JTable(model);

  public SpanningTableCellRendererTest() {
    super(new BorderLayout());
    table.setAutoCreateRowSorter(true);
    table.setFillsViewportHeight(true);
    table.setShowVerticalLines(false);
    table.setIntercellSpacing(new Dimension(0, 1));
    TableColors.registerFocusRepaint(table);
    TableColors tableColors = TableColors.getInstance();
    table.getColumnModel().getColumn(0).setCellRenderer(new ColorOverrideRenderer(
        table.getDefaultRenderer(Boolean.class), tableColors));
    table.getColumnModel().getColumn(0).setCellEditor(new ColorOverrideEditor(
        table.getDefaultEditor(Boolean.class), tableColors));
    //table.getColumnModel().getColumn(0).setCellEditor(table.getDefaultEditor(Boolean.class));
    UIUtil.configureBooleanTableValueShortcuts(table, 0);
    //table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    //table.setCellSelectionEnabled(true);
    for (int i = 0; i < table.getColumnModel().getColumnCount(); i++) {
      TableColumn c = table.getColumnModel().getColumn(i);
      if (i > 0)
        c.setCellRenderer(new SpanningTableCellRenderer(true, TableColors.getInstance()));
      c.setMinWidth(50);
    }
    setLayout(new BorderLayout());
    add(new JTextField(), BorderLayout.NORTH);
    add(new JScrollPane(table), BorderLayout.CENTER);
  }

  public static void main(String[] args) {
    SwingUtilities.invokeLater(() -> {
      try {
        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      } catch (Exception e) {
        Exceptions.printStackTrace(e);
      }
      JFrame frame = new JFrame("SpanningTableCellRendererTest");
      frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
      frame.getContentPane().add(new SpanningTableCellRendererTest());
      frame.setLocationRelativeTo(null); // Center frame.
      frame.pack();
      frame.setVisible(true);
    });
  }
}
