// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.harness;

import com.sieuferd.uiutil.ThreadUtil;
import com.sieuferd.uiutil.UIUtil;
import com.sieuferd.upstream.localeditor.LocalEditor;
import com.sieuferd.widget.InplaceEditor;
import java.awt.event.KeyEvent;
import javax.annotation.Nullable;
import javax.swing.KeyStroke;
import javax.swing.border.EmptyBorder;
import javax.swing.text.EditorKit;
import javax.swing.text.JTextComponent;

final class TestInplaceEditor implements InplaceEditor {
  private static final boolean USE_NETBEANS_EDITOR_KIT = true;
  private final LocalEditor editor = new LocalEditor(new EmptyBorder(0, 0, 0, 0));
  private boolean active = false;

  public TestInplaceEditor() {
    setEditorKit();
    editor.setWrapping(false);
    editor.setVisibleRows(null);
  }

  @Override
  public LocalEditor getComponent() {
    return editor;
  }

  @Override
  public JTextComponent getFocusComponent() {
    return editor.getEditorPane();
  }

  @Override
  public boolean forwardUnhandledKeyStroke(KeyStroke ks, KeyEvent evt, boolean pressed) {
    return editor.forwardUnhandledKeyStroke(ks, evt, pressed);
  }

  @Override
  public boolean shouldStartEditing(@Nullable KeyEvent evt) {
    return evt == null || UIUtil.isCharacterInsertionEvent(evt);
  }

  private void setEditorKit() {
    if (USE_NETBEANS_EDITOR_KIT) {
      try {
        /* The right way to get a PlainKit is by calling
        CloneableEditorSupport.getEditorKit("text/plain"), but I could not get it working when used
        from a main class. Since this is for testing only, I think it's OK to instantiate it
        directly. */
        Class<?> kitClass = Class.forName("org.netbeans.modules.editor.plain.PlainKit");
        editor.getEditorPane().setEditorKit((EditorKit) kitClass.newInstance());
      } catch (Exception e) {
        throw new AssertionError(e);
      }
    }
  }

  @Override
  public void prepare(boolean clear) {
    if (clear) {
      UIUtil.setDocumentText(editor.getEditorPane().getDocument(), "");
    } else {
      UIUtil.setDocumentText(editor.getEditorPane().getDocument(), "Some Existing Content");
      editor.getEditorPane().setCaretPosition(editor.getEditorPane().getDocument().getLength());
    }
  }

  public String getText() {
    return editor.getEditorPane().getText();
  }

  @Override
  public void setActive(boolean active) {
    ThreadUtil.checkOnEDT();
    System.out.println("Inplace editor " + (active ? "active"  : "inactive"));
    this.active = active;
  }

  public boolean isActive() {
    return active;
  }
}
