// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.harness;

import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.LinkedListMultimap;
import com.google.common.collect.Multimap;
import java.awt.EventQueue;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.logging.Logger;
import com.sieuferd.database.SQLConnection;
import com.sieuferd.datamodel.ImmutableData;
import com.sieuferd.datamodel.ImmutableView;
import com.sieuferd.datamodel.MutableView;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.ResultLimit;
import com.sieuferd.datamodel.View;
import com.sieuferd.util.SieuferdXMLException;
import com.sieuferd.viewset.SieuferdProjectXML;
import com.sieuferd.datamodel.ViewResult;
import com.sieuferd.datamodel.Views;
import com.sieuferd.datamodel.proptypes.Warning;
import com.sieuferd.evaluator.QueryUtil;
import com.sieuferd.evaluator.CancelHandler;
import com.sieuferd.evaluator.DataProperties;
import com.sieuferd.evaluator.QueryEvaluator;
import com.sieuferd.uiutil.UIUtil;
import java.util.Map.Entry;
import org.openide.util.Utilities;

public class WidgetMain {
  private static final Logger LOG = Logger.getLogger(WidgetMain.class.getName());
  private WidgetFrame frame;
  private Connection connection;

  public WidgetMain() throws SQLException {
    // TODO[cc]: Arrange to use try-with-resources.
    //connection = DriverManager.getConnection("jdbc:derby://127.0.0.1:1527/sample", "app", "app");
    connection = DriverManager.getConnection("jdbc:postgresql://127.0.0.1:5432/princeton", "duser", "dpass");
    //connection = DriverManager.getConnection("jdbc:mysql://127.0.0.1:3306/", "duser", "dpass");
    //connection = DriverManager.getConnection("jdbc:postgresql://localhost:5432/pivotpaths", "duser", "dpass");
    try {
      EventQueue.invokeAndWait(() -> {
        frame = new WidgetFrame();
        frame.setVisible(true);
      });
    } catch (InterruptedException | InvocationTargetException e) {
      throw new AssertionError(e);
    }
    frame.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosed(WindowEvent event) {
        try {
          connection.close();
        } catch (SQLException e) {
          throw new RuntimeException(e);
        }
      }
    });
  }

  public Connection getConnection() {
    return connection;
  }

  public void setResult(final ViewResult result) {
    EventQueue.invokeLater(() -> {
      frame.setResult(result);
    });
  }

  public static void main(String args[])
      throws SQLException, IOException, SieuferdXMLException
  {
    System.setProperty("com.apple.mrj.application.apple.menu.about.name", "Nested Table Flow");
    //System.setProperty("org.netbeans.editor.disable.end.virtual.space", "true");
    UIUtil.setDefaultLookAndFeel(true);
    WidgetMain wm = new WidgetMain();

    ViewResult result = null;
    LOG.info("Connecting...");
    try (SQLConnection connection = new SQLConnection(wm.getConnection())) {
      MutableView view = Views.newMutable(SieuferdProjectXML.readSieuferdViewXML(
          new FileInputStream(LoadedXMLDriver.QUERY_XML_FILE),
          Utilities.toURI(LoadedXMLDriver.QUERY_XML_FILE).toString(),
          LoadedXMLDriver.VIEW_DEFAULTS.getProperties()), LoadedXMLDriver.VIEW_DEFAULTS);
      QueryUtil.configurePrimaryKeys(connection, view);
      QueryEvaluator evaluator = QueryEvaluator.create(view, connection, null, new CancelHandler());
      DataProperties.configureDataProperties(view);
      try {
        final int limit = Integer.MAX_VALUE;
        final ImmutableView immutableView = ImmutableView.copyOf(view);
        final ImmutableData data = evaluate(immutableView, evaluator, limit);
        result = new ViewResult(ResultLimit.UNDEFINED, immutableView, data, ImmutableMultimap.of());
      } finally {
        evaluator.close();
      }
    }

    LOG.info("Displaying...");
    wm.setResult(result);
  }

  public static ImmutableData evaluate(View view, QueryEvaluator evaluator, int limit) {
    Multimap<Path,Warning> toWarnings = LinkedListMultimap.create();
    ImmutableData ret = evaluator.evaluate(limit, toWarnings);
    if (!toWarnings.isEmpty()) {
      StringBuilder sb = new StringBuilder();
      sb.append("Evaluation errors:\n");
      for (Entry<Path,Warning> entry : toWarnings.entries()) {
        sb.append("  ")
            .append(entry.getKey().toString(view))
            .append(": ")
            .append(entry.getValue().getDisplayMessage())
            .append("\n");
      }
      throw new RuntimeException(sb.toString());
    }
    return ret;
  }
}
