// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.legacy.driver;

import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import java.io.PrintStream;
import com.sieuferd.datamodel.Data;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.Tuple;
import com.sieuferd.cell.support.Coordinates;
import com.sieuferd.cell.support.Pixels;
import com.sieuferd.cell.support.RenderContext;
import com.sieuferd.datamodel.CProp;
import com.sieuferd.datamodel.MutableView;
import com.sieuferd.datamodel.View;
import com.sieuferd.datamodel.ViewResult;
import com.sieuferd.layout.YesNoAuto;
import com.sieuferd.layout.LAProp;
import org.apache.commons.lang.mutable.MutableInt;
import org.apache.commons.math.stat.descriptive.SummaryStatistics;

public class DataCollection {
  private static final double PAPER_WIDTH_IN = 8.5, PAPER_HEIGHT_IN = 11.0;
  private static final double PAPER_MARGINS_IN = 0.25;

  private static void gatherValueStats(ViewResult result, Path path, Tuple parentTuple,
      SummaryStatistics characterStats, SummaryStatistics depthStats,
      SummaryStatistics pluralDepthStats, MutableInt numPrimitives, MutableInt numTuples,
      int atDepth, int atPluralDepth)
  {
    final View view = result.getView();
    final Data data = result.getData();
    if (path.isRelation()) {
      for (Tuple tuple : data.getRelationTuples(path.getRelation(), parentTuple)) {
        numTuples.increment();
        for (Path childPath : view.getChildPaths(path.getRelation())) {
          gatherValueStats(result, childPath, tuple, characterStats, depthStats, pluralDepthStats,
              numPrimitives, numTuples, atDepth + 1, view.get(CProp.PLURAL,
              path.getRelation()) ? atPluralDepth + 1 : atPluralDepth);
        }
      }
    } else {
      // TODO[pd]: Can I do this, or do I have to inside-out this method?
      Object content = data.getTupleField(parentTuple, path.getPrimitive());
      characterStats.addValue(content == null ? 0 : content.toString().length());
      depthStats.addValue(atDepth);
      pluralDepthStats.addValue(atPluralDepth);
      numPrimitives.increment();
    }
  }

  private static void gatherValueStats(String corpusKey, ViewResult result, PrintStream output) {
    if (corpusKey == null) {
      output.println("corpusKey\ttotalChars\tnumPrimitives\tnumTuples\tmeanDepth\tmaxDepth\tmeanPluralDepth\tmaxPluralDepth");
      return;
    }
    SummaryStatistics characterStats   = new SummaryStatistics();
    SummaryStatistics depthStats       = new SummaryStatistics();
    SummaryStatistics pluralDepthStats = new SummaryStatistics();
    MutableInt numPrimitives = new MutableInt();
    MutableInt numTuples = new MutableInt();
    gatherValueStats(result, Path.ROOT, Tuple.createRootParent(result.getData()),
        characterStats, depthStats, pluralDepthStats, numPrimitives, numTuples, 0, 0);
    output.println(
        corpusKey + "\t" +
        characterStats.getSum() + "\t" +
        numPrimitives.intValue() + "\t" +
        numTuples.intValue() + "\t" +
        depthStats.getMean() + "\t" +
        depthStats.getMax() + "\t" +
        pluralDepthStats.getMean() + "\t" +
        pluralDepthStats.getMax());
  }

  public static RenderContext getStandardizedRenderContext() {
    return getStandardizedRenderContext(1.0);
  }

  private static RenderContext getStandardizedRenderContext(double scaling) {
    FontRenderContext frc = new FontRenderContext(
        new AffineTransform(
        scaling * 50.0 / 3, 0.0, 0.0,
        scaling * 50.0 / 3, 0.0, 0.0), false, true);
    return new RenderContext(frc, 1.0, null, false, false);
  }

  public static Coordinates getStandardizedPageSize() {
    RenderContext rcx = getStandardizedRenderContext(1.0);
    return new Coordinates(
        Pixels.create(rcx, (PAPER_WIDTH_IN - 2.0 * PAPER_MARGINS_IN) * 72.0),
        Pixels.create(rcx, (PAPER_HEIGHT_IN - 2.0 * PAPER_MARGINS_IN) * 72.0));
  }

  public static double squareInchesAfterScaling(Coordinates dimensions) {
    return dimensions.getX().toInchesAfterScaling() * dimensions.getY().toInchesAfterScaling();
  }

  public static void setNaiveStyleSheet(Path path, MutableView view,
      boolean tabular)
  {
    view.set(LAProp.START_NEW_COLUMN    , path, YesNoAuto.NO);
    view.set(LAProp.START_NEW_COLUMN_SET, path, YesNoAuto.NO);
    if (path.isRelation()) {
      view.set(LAProp.USE_TABLE, path, tabular ? YesNoAuto.YES : YesNoAuto.NO);
      for (Path childPath : view.getChildPaths(path.getRelation()))
        setNaiveStyleSheet(childPath, view, tabular);
    }
  }
}
