// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.legacy.figures;

import com.google.common.collect.ImmutableSet;
import com.sieuferd.datamodel.MutableView;
import java.awt.print.PageFormat;
import java.awt.print.Pageable;
import java.awt.print.Paper;
import java.awt.print.Printable;
import java.awt.print.PrinterJob;
import com.sieuferd.datamodel.Path;
import com.sieuferd.layout.YesNoAuto;
import com.sieuferd.layout.LAProp;

public final class FigureGeneration {
  // Paper width 8.5 for most figures; 4.5 for narrower. (Margins 0.5.)
  // Paper width 7 for single version (course 3416 in dept. 27).
  // For single table, also set TABLE_MAX_PRIMITIVE_WIDTH to 200pts.
  private static final double PAPER_WIDTH_IN = 4.5, PAPER_HEIGHT_IN = 40.0;
  private static final double PAPER_MARGINS_IN = 0.5;
  public static ImmutableSet<String> variableLengthPaths = ImmutableSet.of(
    "#/Title",
    "#/Description",
    "#/Website",
    "#/Grading/Grading Type/Title",
    "#/Sample Reading List/Author Name",
    "#/Sample Reading List/Title",
    "#/Sections/Instructors/Instructor/First",
    "#/Sections/Instructors/Instructor/Middle",
    "#/Sections/Instructors/Instructor/Last",
    "#/Title",
    "#/Description",
    "#/Website",
    "#/Grading/Grading Type/Title",
    "#/Sample Reading List/Author Name",
    "#/Sample Reading List/Title",
    "#/Sections/Instructors/Instructor/First",
    "#/Sections/Instructors/Instructor/Middle",
    "#/Sections/Instructors/Instructor/Last");

  private FigureGeneration() { }

  public static void setTableStyleSheet(Path path, MutableView view) {
    view.set(LAProp.START_NEW_COLUMN    , path, YesNoAuto.NO);
    view.set(LAProp.START_NEW_COLUMN_SET, path, YesNoAuto.NO);
    if (path.isRelation()) {
      view.set(LAProp.USE_TABLE, path, YesNoAuto.YES);
      for (Path childPath : view.getChildPaths(path.getRelation()))
        setTableStyleSheet(childPath, view);
    }
  }

  public static void setOutlineStyleSheet(Path path, MutableView view) {
    view.set(LAProp.START_NEW_COLUMN    , path, YesNoAuto.NO);
    view.set(LAProp.START_NEW_COLUMN_SET, path, YesNoAuto.NO);
    if (path.isRelation()) {
      view.set(LAProp.USE_TABLE, path, YesNoAuto.NO);
      for (Path childPath : view.getChildPaths(path.getRelation()))
        setOutlineStyleSheet(childPath, view);
    }
  }

  public static void setNoColumnStylesheet(Path path, MutableView view) {
    view.set(LAProp.START_NEW_COLUMN    , path, YesNoAuto.NO);
    view.set(LAProp.START_NEW_COLUMN_SET, path, YesNoAuto.NO);
    if (path.isRelation()) {
      for (Path childPath : view.getChildPaths(path.getRelation()))
        setNoColumnStylesheet(childPath, view);
    }
  }

  public static void configurePrinterJob(PrinterJob job, final Printable printer) {
    Paper paper = new Paper();
    paper.setSize(PAPER_WIDTH_IN * 72.0, PAPER_HEIGHT_IN * 72.0);
    paper.setImageableArea(
        PAPER_MARGINS_IN * 72.0, PAPER_MARGINS_IN * 72.0,
        (PAPER_WIDTH_IN - 2.0 * PAPER_MARGINS_IN) * 72.0,
        (PAPER_HEIGHT_IN - 2.0 * PAPER_MARGINS_IN) * 72.0);
    final PageFormat pageFormat = new PageFormat();
    pageFormat.setPaper(paper);
    pageFormat.setOrientation(PageFormat.PORTRAIT);
    job.setPageable(new Pageable() {
      @Override
      public int getNumberOfPages() {
        return Pageable.UNKNOWN_NUMBER_OF_PAGES;
      }

      @Override
      public PageFormat getPageFormat(int pageIndex) throws IndexOutOfBoundsException {
        return pageFormat;
      }

      @Override
      public Printable getPrintable(int pageIndex) throws IndexOutOfBoundsException {
        return printer;
      }
    });
  }
}
