// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.legacy.xmlimport;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.logging.Logger;
import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import com.sieuferd.datamodel.ImmutableData;
import org.w3c.dom.Document;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

// TODO[pd]: Deal with attributes.
/* TODO[pd]: Store XML primitives as verbatim XML. An example can be found in the NASA dataset in
             the "definition" field. */
public final class XMLImport {
  private static final Logger LOG = Logger.getLogger(XMLImport.class.getName());
  private XMLImport() { }

  /** Will close the input stream after parsing. */
  public static ImmutableData convertXML(InputStream xmlStream) throws XMLException, IOException {
    try {
      DocumentBuilder documentBuilder;
      try {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        dbf.setValidating(false);
        dbf.setExpandEntityReferences(false);
        /* Avoid HTTP operations in the middle of XML parsing. See
        http://stackoverflow.com/questions/4002885/unknown-host-exception-while-parsing-an-xml-file
        http://stackoverflow.com/questions/155101/make-documentbuilder-parse-ignore-dtd-references
        */
        dbf.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
        dbf.setFeature("http://apache.org/xml/features/nonvalidating/load-dtd-grammar", false);
        dbf.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
        documentBuilder = dbf.newDocumentBuilder();
      } catch (ParserConfigurationException e) {
        throw new XMLException("Error initializing XML parser", e);
      }
      // If all else fails, intercept all attempts to resolve external XML entities.
      documentBuilder.setEntityResolver(new EntityResolver() {
        @Override
        public InputSource resolveEntity(String publicId, String systemId) throws IOException {
          LOG.warning("Unexpected attempt to resolve an external XML entity");
          return new InputSource(new StringReader(""));
        }
      });
      Document xmlDocument;
      try {
        xmlDocument = documentBuilder.parse(xmlStream);
      } catch (SAXException e) {
        throw new XMLException("Error parsing XML file", e);
      }
      throw new UnsupportedOperationException();
    } finally {
      try {
        xmlStream.close();
      } catch (IOException e) {
        // Ignore; I can't know if DocumentBuilder.parse() closed the stream already or not.
      }
    }
  }

  public static ImmutableData convertXML(File f) throws XMLException, IOException {
    return convertXML(new BufferedInputStream(new FileInputStream(f)));
  }
}
