// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.legacy.xmlimport;

import com.google.common.base.Objects;
import com.google.common.base.Preconditions;
import javax.annotation.Nullable;
import org.apache.commons.lang.WordUtils;

public final class XMLLabel {
  // TODO[pd]: See if this can be cleaned up.
  public static final XMLLabel SURROGATE_KEY_LABEL =
      new XMLLabel("", "id", XMLLabelType.SURROGATE_KEY);

  public static enum XMLLabelType {
    ELEMENT("E"),
    ATTRIBUTE("A"),
    TEXTELEMENT("T"),
    SURROGATE_KEY("K");
    private final String denote;

    private XMLLabelType(String denote) {
      this.denote = denote;
    }

    @Override
    public String toString() {
      return denote;
    }
  }
  final XMLLabelType type;
  final String name;
  final String namespaceURI;

  public XMLLabel(@Nullable String namespaceURI, String name, XMLLabelType type) {
    Preconditions.checkNotNull(name);
    Preconditions.checkNotNull(type);
    this.type = type;
    this.name = name;
    // TODO[pd]: Ensure that an empty namespace is equivalent to a null namespace.
    this.namespaceURI = (namespaceURI == null || namespaceURI.isEmpty()) ? null : namespaceURI;
  }

  public String getPresentationString() {
    // TODO[cc]: Move this presentation logic elsewhere.
    return WordUtils.capitalizeFully(name.replace("_", " "));
  }

  public XMLLabelType getType() {
    return type;
  }

  public String getName() {
    return name;
  }

  public @Nullable String getNamespaceURI() {
    return namespaceURI;
  }

  @Override
  public String toString() {
    return (namespaceURI == null ? "" : (namespaceURI + ":")) +
        name + "(" + type + ")";
  }

  @Override
  public int hashCode() {
    return Objects.hashCode(name, namespaceURI, type);
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (obj == null || obj.getClass() != XMLLabel.class)
      return false;
    XMLLabel rhs = (XMLLabel) obj;
    return Objects.equal(name        , rhs.name)
        && Objects.equal(namespaceURI, rhs.namespaceURI)
        && Objects.equal(type        , rhs.type);
  }
}
