// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.database;

import com.google.common.collect.Lists;
import java.util.List;

public class SQLFragmentBuilder {
  private final StringBuilder jdbcStringBuilder = new StringBuilder();
  private final List<Object> queryParameters = Lists.newArrayList();

  SQLFragmentBuilder() { }

  public SQLFragmentBuilder appendSQLFragment(SQLFragment sqlFragment) {
    jdbcStringBuilder.append(sqlFragment.getJDBCString());
    queryParameters.addAll(sqlFragment.getQueryParameters());
    return this;
  }

  public SQLFragmentBuilder appendJDBCString(String jdbcString) {
    jdbcStringBuilder.append(jdbcString);
    return this;
  }

  public SQLFragmentBuilder appendQueryParameter(Object queryParameter) {
    appendSQLFragment(SQLFragment.createParameter(queryParameter));
    return this;
  }

  public SQLFragment build() {
    return SQLFragment.create(jdbcStringBuilder.toString(), queryParameters);
  }
}
