// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.database;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Maps;
import com.sieuferd.query.SQLTable;
import com.sieuferd.util.MapUtil;
import java.util.Collection;
import java.util.Map;
import java.util.Set;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class SQLTableInfo {
  private final SQLTable table;
  private final ImmutableMap<String,SQLColumnInfo> columns;
  private final ImmutableSet<String> primaryKeyColumnNames;

  public SQLTableInfo(
      SQLTable table, Collection<SQLColumnInfo> columns, Set<String> primaryKeyColumnNames)
  {
    Preconditions.checkNotNull(table);
    Preconditions.checkNotNull(columns);
    Map<String,SQLColumnInfo> toColumns = Maps.newLinkedHashMap();
    for (SQLColumnInfo columnInfo : columns)
      MapUtil.putChecked(toColumns, columnInfo.getName(), columnInfo);
    Preconditions.checkArgument(toColumns.keySet().containsAll(primaryKeyColumnNames));
    this.table = table;
    this.columns = ImmutableMap.copyOf(toColumns);
    this.primaryKeyColumnNames = ImmutableSet.copyOf(primaryKeyColumnNames);
  }

  public SQLTable getTable() {
    return table;
  }

  public Collection<SQLColumnInfo> getColumns() {
    return columns.values();
  }

  public Set<String> getColumnNames() {
    return columns.keySet();
  }

  public @Nullable SQLColumnInfo getColumnInfo(String columnName) {
    return columns.get(columnName);
  }

  public boolean isPrimaryKey(String columnName) {
    Preconditions.checkArgument(columns.keySet().contains(columnName));
    return primaryKeyColumnNames.contains(columnName);
  }
}
