// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.database;

import com.google.common.base.Preconditions;
import java.sql.SQLException;
import javax.annotation.Nonnull;

public abstract class SQLTask<R> {
  private final String name;

  /** @param name name of the task, used for logging purposes */
  public SQLTask(String name) {
    Preconditions.checkNotNull(name);
    this.name = name;
  }

  @Override
  public String toString() {
    return name;
  }

  /**
   * Execute the task on the supplied connection. {@code SQLException}s thrown by the underlying
   * JDBC driver should be passed up the stack, or reported via
   * {@link SQLConnection#reportSQLException(java.sql.SQLException)}. No guarantees are made as to
   * what thread the task will be executed on, and the task may be executed more than once, for
   * instance in case it is necessary to retry the operation.
   */
  public abstract @Nonnull R run(SQLConnection sqlConnection) throws SQLException;

  /**
   * Exception indicating that the task failed due to a connection error that has already been
   * communicated to the user.
   */
  public static final class UserInformedConnectionException extends Exception {
    public UserInformedConnectionException(String message) {
      super(message);
    }

    public UserInformedConnectionException(Throwable cause) {
      super(cause);
    }
  }
}
