// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.database;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Maps;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.sql.Types;
import java.util.Map;
import javax.annotation.concurrent.Immutable;

// TODO[cc]: Replace this with java.sql.JDBCType.
/**
 * Wrapper for a JDBC type code as per the constants defined in {@link java.sql.Types}.
 */
@Immutable
public final class SQLTypeCode {
  public static final SQLTypeCode NULL;
  private static final ImmutableMap<Integer,String> NAMES;
  private final int code;

  static {
    if (Types.NULL != 0)
      throw new AssertionError();
    NULL = new SQLTypeCode(0);
    final Map<Integer,String> toNames = Maps.newLinkedHashMap();
    // Get the names of the constants in java.sql.Types by reflection, for use by toString().
    try {
      for (Field field : Types.class.getDeclaredFields()) {
        final int modifiers = field.getModifiers();
        if (Modifier.isStatic(modifiers) && Modifier.isPublic(modifiers) &&
            field.getType().equals(Integer.TYPE))
        {
          toNames.put(field.getInt(null), field.getName());
        }
      }
    } catch (IllegalAccessException e) {
      throw new RuntimeException("Failed to load constant list from java.sql.Types", e);
    }
    NAMES = ImmutableMap.copyOf(toNames);
  }

  private SQLTypeCode(int code) {
    // Allow any code, in case individual JDBC drivers use their own extensions.
    this.code = code;
  }

  public static SQLTypeCode create(int code) {
    if (code == 0)
      return NULL;
    return new SQLTypeCode(code);
  }

  public int getCode() {
    return code;
  }

  @Override
  public String toString() {
    String ret = NAMES.get(code);
    return ret == null ? ("UNKNOWN(" + code + ")") : ret;
  }

  @Override
  public int hashCode() {
    return code;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof SQLTypeCode))
      return false;
    SQLTypeCode other = (SQLTypeCode) obj;
    return this.code == other.code;
  }
}
