// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.database.dialect;

import java.sql.Connection;
import java.sql.SQLException;
import javax.annotation.Nullable;
import org.hibernate.boot.registry.StandardServiceRegistryBuilder;
import org.hibernate.dialect.Dialect;
import org.hibernate.dialect.MySQLDialect;
import org.hibernate.dialect.Oracle8iDialect;
import org.hibernate.dialect.PostgreSQL81Dialect;
import org.hibernate.engine.jdbc.dialect.spi.DatabaseMetaDataDialectResolutionInfoAdapter;
import org.hibernate.engine.jdbc.dialect.spi.DialectResolver;
import org.hibernate.service.ServiceRegistry;

/* TODO[cc]: De-centralize this by making individual dialects decide whether or not they would like
             to handle a particular Connection object. */
public final class BackendDialectResolver {
  public static final BackendDialectResolver INSTANCE = new BackendDialectResolver();
  private static final DialectResolver HIBERNATE_DIALECT_RESOLVER =
      lookupHibernateDialectResolver();

  private static DialectResolver lookupHibernateDialectResolver() {
    final ServiceRegistry hibernateServiceRegistry = new StandardServiceRegistryBuilder().build();
    try {
      return hibernateServiceRegistry.getService(DialectResolver.class);
    } finally {
      StandardServiceRegistryBuilder.destroy(hibernateServiceRegistry);
    }
  }

  public @Nullable BackendDialect resolveDialect(Connection connection) throws SQLException {
    final String databaseProductName = connection.getMetaData().getDatabaseProductName();

    if ("NuoDB".equals(databaseProductName))
      return new NuoDBBackendDialect(connection);

    // Hibernate-based dialects.
    Dialect hibernateDialect = HIBERNATE_DIALECT_RESOLVER.resolveDialect(
        new DatabaseMetaDataDialectResolutionInfoAdapter(connection.getMetaData()));
    if (hibernateDialect == null) {
      return new GenericJDBCBackendDialect(connection);
    } else {
      if (hibernateDialect instanceof MySQLDialect) {
        return new MySQLBackendDialect(connection, hibernateDialect);
      } else if (hibernateDialect instanceof PostgreSQL81Dialect) {
        /* PostgreSQL81Dialect is the base class of all of Hibernate's PostgreSQL dialects,
        including PostgreSQLDialect. */
        return new PostgreSQLBackendDialect(connection, hibernateDialect);
      } else if (hibernateDialect instanceof Oracle8iDialect) {
        // Oracle8iDialect is the base class of all of Hibernate's Oracle dialects.
        return new OracleBackendDialect(connection, hibernateDialect);
      } else {
        return new HibernateBackendDialect(connection, hibernateDialect);
      }
    }
  }
}
