// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.database.dialect;

import com.sieuferd.database.SQLFragment;
import com.sieuferd.database.SQLFragmentBuilder;
import com.sieuferd.database.dialect.FunctionImplementation.SimpleFunctionImplementation;
import com.sieuferd.datamodel.expressions.Function;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;
import org.hibernate.dialect.Dialect;

public class MySQLBackendDialect extends HibernateBackendDialect {
  public MySQLBackendDialect(Connection connection, Dialect hibernateDialect) throws SQLException {
    super(connection, hibernateDialect);

    registerFunction(Function.create("current_time"), "current_time");
    registerFunction(Function.TOTALLY_ORDERED_COLLATION,
        "CAST([1] AS CHAR CHARACTER SET utf8) COLLATE utf8_bin");
    registerFunction(Function.GROUP_CONCAT, "GROUP_CONCAT([2] SEPARATOR [1])");

    // See http://dev.mysql.com/doc/refman/5.0/en/comparison-operators.html
    registerFunction(Function.IS, "[1] <=> [2]");
    registerFunction(Function.IS_NOT, "NOT ([1] <=> [2])");
    registerFunction(Function.STRING_CONCAT, new SimpleFunctionImplementation(null) {
      @Override
      protected SQLFragment renderFragment(List<SQLFragment> arguments) {
        if (arguments.isEmpty())
          return SQLFragment.createParameter("");
        SQLFragmentBuilder ret = SQLFragment.builder();
        int i = 0;
        for (SQLFragment argument : arguments) {
          if (i++ > 0)
            ret.appendJDBCString(", ");
          ret.appendSQLFragment(argument.enclose("CAST((", ") AS char)"));
        }
        /* Use concat_ws with an empty separator string rather than concat in order to correctly
        treat NULL arguments as empty strings. */
        return ret.build().enclose("concat_ws('', ", ")");
      }
    });
    /* TODO[pd]: This is ugly and not entirely functional; try to do better. But as far as I know,
                 MySQL does not have a proper regex replace function. */
    registerFunction(Function.STRING_NORMALIZE_WHITESPACE, new SimpleFunctionImplementation(1) {
      @Override
      protected SQLFragment renderFragment(List<SQLFragment> arguments) {
        return arguments.get(0)
            .enclose("trim(", ")")
            .enclose("replace(", ", '\n', ' ')")
            .enclose("replace(", ", '\r', ' ')")
            .enclose("replace(", ", '\t', ' ')")
            .enclose("replace(", ", '  ', ' ')")
            .enclose("replace(", ", '  ', ' ')")
            .enclose("replace(", ", '  ', ' ')")
            .enclose("replace(", ", '  ', ' ')");
      }
    });
  }

  @Override
  public String getDisplayName() {
    return "MySQL";
  }
}
