// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.database.dialect;

import com.google.common.base.Preconditions;
import com.sieuferd.database.SQLFragment;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class RenderedExpression {
  private final SQLFragment fragment;
  private final RenderVariant variant;

  public RenderedExpression(SQLFragment fragment, RenderVariant variant) {
    Preconditions.checkNotNull(fragment);
    Preconditions.checkNotNull(variant);
    this.fragment = fragment;
    this.variant = variant;
  }

  @Override
  public String toString() {
    return fragment.toString();
  }

  public static RenderedExpression simple(SQLFragment fragment) {
    return new RenderedExpression(fragment, RenderVariant.DEFAULT);
  }

  /* TODO[cc]: Consider replacing getFragment with methods for individual contexts (preferred,
               selectable, conditional, orderable), and getting rid of all the context casts other
               than selectable. */

  public SQLFragment getFragment() {
    return fragment;
  }

  public RenderVariant getVariant() {
    return variant;
  }

  public RenderedExpression toPreferred() {
    return variant.toPreferred(fragment);
  }

  public RenderedExpression toSelectable() {
    return variant.toSelectable(fragment);
  }

  public RenderedExpression toConditional() {
    return variant.toConditional(fragment);
  }

  public RenderedExpression toOrderable() {
    return variant.toOrderable(fragment);
  }
}
