// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.database.dialect;

import org.junit.Assert;
import org.junit.Test;

public class TemplateFunctionImplementationTest {
  private void testTemplate(String template, String expectedParts) {
    RenderVariant outputVariant = RenderVariant.DEFAULT;
    FunctionImplementation fun = new TemplateFunctionImplementation(null, template, outputVariant);
    String expected = "TemplateFunctionImplementation(" +
        outputVariant.getClass().getSimpleName() + ", [" + expectedParts + "])";
    String actual = fun.toString();
    System.out.println(actual);
    Assert.assertEquals(expected, fun.toString());
  }

  @Test
  public void testBasicStrings() {
    testTemplate("", "");
    testTemplate(" ", "S{ }");
    testTemplate("this is a test", "S{this is a test}");
  }

  @Test
  public void testOpenEscapes() {
    testTemplate("[[", "S{[}");
    testTemplate("[[[[", "S{[[}");
    testTemplate("this is a[[ test", "S{this is a[ test}");
    testTemplate("[[this is a[[ test[[", "S{[this is a[ test[}");
    testTemplate("[[this is a[[ test[[[[", "S{[this is a[ test[[}");
    testTemplate("[[this is a[[ test[[[[z", "S{[this is a[ test[[z}");
    testTemplate("q[[this is a[[ test[[z", "S{q[this is a[ test[z}");
  }

  @Test
  public void testCloseEscapes() {
    testTemplate("]]", "S{]}");
    testTemplate("]]]]", "S{]]}");
    testTemplate("this is a]] test", "S{this is a] test}");
    testTemplate("]]this is a]] test]]", "S{]this is a] test]}");
    testTemplate("]]this is a]] test]]]]", "S{]this is a] test]]}");
    testTemplate("]]this is a]] test]]]]z", "S{]this is a] test]]z}");
    testTemplate("q]]this is a]] test]]z", "S{q]this is a] test]z}");
  }

  @Test
  public void testMixedEscapes() {
    testTemplate("[[]]", "S{[]}");
    testTemplate("]][[", "S{][}");
    testTemplate("[[[[", "S{[[}");
    testTemplate("[[[[]]", "S{[[]}");
    testTemplate("this]] is a[[ test", "S{this] is a[ test}");
    testTemplate("[[this is a[[]] test[[", "S{[this is a[] test[}");
    testTemplate("]][[this is a[[ test[[[[", "S{][this is a[ test[[}");
    testTemplate("[[this is a[[ test[[[[]]z", "S{[this is a[ test[[]z}");
    testTemplate("q[[this is a[[ t]]est[[z", "S{q[this is a[ t]est[z}");
  }

  @Test
  public void testBasicArguments() {
    testTemplate("[1] oper [2]", "A{p1}, S{ oper }, A{p2}");
    testTemplate("[1] oper [2] foo [3]", "A{p1}, S{ oper }, A{p2}, S{ foo }, A{p3}");
    testTemplate("[1] oper [2] foo [1]", "A{p1}, S{ oper }, A{p2}, S{ foo }, A{p1}");
    testTemplate("[1] oper [2] foo [1]a", "A{p1}, S{ oper }, A{p2}, S{ foo }, A{p1}, S{a}");
    testTemplate("asdf[1][2] foo [1]a", "S{asdf}, A{p1}, A{p2}, S{ foo }, A{p1}, S{a}");
    testTemplate("asdf[1][12] foo [10]a", "S{asdf}, A{p1}, A{p12}, S{ foo }, A{p10}, S{a}");
    testTemplate("[2][1]", "A{p2}, A{p1}");
  }

  @Test
  public void testVariantArgumentsChar() {
    testTemplate("[p1] oper [s2]", "A{p1}, S{ oper }, A{s2}");
    testTemplate("[c1] oper [o2] foo [o3]", "A{c1}, S{ oper }, A{o2}, S{ foo }, A{o3}");
    testTemplate("[s1] oper [s2] foo [s1]", "A{s1}, S{ oper }, A{s2}, S{ foo }, A{s1}");
    testTemplate("[p1] oper [p2] foo [p1]a", "A{p1}, S{ oper }, A{p2}, S{ foo }, A{p1}, S{a}");
    testTemplate("asdf[s1][c2] foo [c1]a", "S{asdf}, A{s1}, A{c2}, S{ foo }, A{c1}, S{a}");
    testTemplate("asdf[s1][s12] foo [s10]a", "S{asdf}, A{s1}, A{s12}, S{ foo }, A{s10}, S{a}");
    testTemplate("[s2][p1]", "A{s2}, A{p1}");
  }

  @Test
  public void testVariantArgumentsCharAndEscaping() {
    testTemplate("[p1] op[[er [s2]", "A{p1}, S{ op[er }, A{s2}");
    testTemplate("[c1] op[[er [o2] f]]oo [o3]", "A{c1}, S{ op[er }, A{o2}, S{ f]oo }, A{o3}");
    testTemplate("[s1] oper [s2] foo [[[s1]", "A{s1}, S{ oper }, A{s2}, S{ foo [}, A{s1}");
    testTemplate("[p1] oper [p2] foo [p1]a]]", "A{p1}, S{ oper }, A{p2}, S{ foo }, A{p1}, S{a]}");
    testTemplate("asdf[s1][c2] foo [c1]]]", "S{asdf}, A{s1}, A{c2}, S{ foo }, A{c1}, S{]}");
    testTemplate("asdf[s1][s12] foo [s10][[", "S{asdf}, A{s1}, A{s12}, S{ foo }, A{s10}, S{[}");
    testTemplate("[s2][[[p1]", "A{s2}, S{[}, A{p1}");
  }
}
