// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator;

import com.google.common.base.Preconditions;
import java.sql.SQLException;
import java.sql.Statement;
import javax.annotation.Nullable;
import javax.annotation.concurrent.ThreadSafe;

@ThreadSafe
public final class CancelHandler {
  private @Nullable Statement statement;
  private boolean canceled;

  // TODO[cc]: Consider returning an AutoCloseable.
  /**
   * @return false iff already canceled
   */
  public synchronized boolean beginStatement(Statement statement) {
    Preconditions.checkNotNull(statement);
    if (isCanceled()) {
      return false;
    } else {
      this.statement = statement;
      return true;
    }
  }

  public synchronized void endStatement() {
    statement = null;
  }

  public synchronized boolean isCanceled() {
    return canceled;
  }

  public synchronized void cancel() throws SQLException {
    if (!canceled) {
      canceled = true;
      if (statement != null) {
        statement.cancel();
        statement = null;
      }
    }
  }
}
