// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator.impl;

import com.google.common.base.Preconditions;
import com.sieuferd.database.SQLFragment;
import com.sieuferd.database.dialect.RenderVariant;
import com.sieuferd.database.dialect.RenderedExpression;
import com.sieuferd.evaluator.impl.SQLIdentifier.ColumnID;
import com.sieuferd.evaluator.impl.SQLIdentifier.InstanceID;
import java.util.Objects;
import javax.annotation.concurrent.Immutable;

@Immutable
final class Column {
  private final ColumnID id;
  private final RenderVariant variant;

  Column(ColumnID id, RenderVariant type) {
    Preconditions.checkNotNull(id);
    Preconditions.checkNotNull(type);
    this.id = id;
    this.variant = type;
  }

  /**
   * Get the column name as a fragment to be used in an AS or GROUP BY clause.
   */
  public SQLFragment getIDFragment() {
    return id.toSQLFragment();
  }

  /**
   * Get a reference expression suitable for use where the column must be accessed via the instance
   * identified by {@code instanceID}.
   */
  public RenderedExpression getRenderedExpression(InstanceID instanceID) {
    return new RenderedExpression(
        instanceID.createColumnReferenceFragment(id.toSQLFragment()), variant);
  }

  /**
   * Get a reference expression suitable for use in contexts where the column is directly in scope.
   */
  public RenderedExpression getRenderedExpression() {
    return new RenderedExpression(id.toSQLFragment(), variant);
  }

  @Override
  public String toString() {
    return id.toString();
  }

  @Override
  public int hashCode() {
    return Objects.hash(id, variant);
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof Column))
      return false;
    Column other = (Column) obj;
    return this.id.equals(other.id)
        && this.variant.equals(other.variant);
  }
}
