// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator.impl;

import com.google.common.base.Preconditions;
import com.sieuferd.datamodel.Path.PrimitivePath;
import java.util.Objects;
import javax.annotation.concurrent.Immutable;

@Immutable
final class CorrelationParameter {
  private final PrimitivePath path;
  private final Column column;
  private final boolean useIs;

  CorrelationParameter(PrimitivePath path, Column column, boolean useIs) {
    Preconditions.checkNotNull(path);
    Preconditions.checkNotNull(column);
    this.path = path;
    this.column = column;
    this.useIs = useIs;
  }

  public PrimitivePath getPath() {
    return path;
  }

  public Column getColumn() {
    return column;
  }

  public boolean getUseIs() {
    return useIs;
  }

  @Override
  public int hashCode() {
    return Objects.hash(path, column, useIs);
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == this)
      return true;
    if (!(obj instanceof CorrelationParameter))
      return false;
    CorrelationParameter other = (CorrelationParameter) obj;
    return this.path.equals(other.path)
        && this.column.equals(other.column)
        && this.useIs == other.useIs;
  }

  @Override
  public String toString() {
    return path.toString() + "=" + column;
  }
}
