// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator.impl;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableList;
import com.sieuferd.database.dialect.RenderedExpression;
import com.sieuferd.datamodel.proptypes.Warning;
import java.util.List;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

// TODO[cc]: Find a better name.

/** An evaluation which may be associated with yet-unreported warnings. */
@Immutable
public class Evaluation {
  private final @Nullable RenderedExpression renderedExpression;
  private final ImmutableList<Warning> warnings;

  public Evaluation(@Nullable RenderedExpression renderedExpression, List<Warning> warnings) {
    this.renderedExpression = renderedExpression;
    this.warnings = ImmutableList.copyOf(warnings);
  }

  public static final Evaluation warning(Warning warning) {
    return new Evaluation(null, ImmutableList.of(warning));
  }

  public static final Evaluation warnings(List<Warning> warnings) {
    return new Evaluation(null, warnings);
  }

  // TODO[cc]: Clean up logic related to the error state.
  public boolean isError() {
    return renderedExpression == null;
  }

  public RenderedExpression getRenderedExpression() {
    Preconditions.checkNotNull(renderedExpression);
    return renderedExpression;
  }

  public ImmutableList<Warning> getWarnings() {
    return warnings;
  }

  // TODO[cc]: Consider getting rid of this.

  /** An evaluation with no unreported warnings. */
  @Immutable
  public static class NWEvaluation extends Evaluation {
    private NWEvaluation(@Nullable RenderedExpression renderedExpression) {
      super(renderedExpression, ImmutableList.of());
    }

    public final static NWEvaluation error() {
      return new NWEvaluation(null);
    }

    public final static NWEvaluation simple(RenderedExpression renderedExpression) {
      Preconditions.checkNotNull(renderedExpression);
      return new NWEvaluation(renderedExpression);
    }
  }
}
