// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator.impl;

import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import java.util.Collections;
import java.util.Set;

public class EvaluationSettings {
  public static enum SettingsFlag {
    CORRELATION_JOIN_OPTIMIZATION,
    REFINE_CORRELATION_KEYS_OPTIMIZATION,
    LEFT_JOIN_ELIMINATION_OPTIMIZATION,
    DISTINCT_ELIMINATION_OPTIMIZATION
  }

  private final ImmutableSet<SettingsFlag> activeFlags;
  private final Set<SettingsFlag> queriedFlags = Sets.newLinkedHashSet();

  // TODO[pd]: Get the evaluator working with this set to false, then make the setting a flag.
  private final boolean usingRootBasedBranches;

  public EvaluationSettings(boolean usingRootBasedBranches, Set<SettingsFlag> activeFlags) {
    this.usingRootBasedBranches = usingRootBasedBranches;
    this.activeFlags = ImmutableSet.copyOf(activeFlags);
  }

  public boolean isUsingRootBasedBranches() {
    return usingRootBasedBranches;
  }

  public static EvaluationSettings create(boolean optimizations) {
    return new EvaluationSettings(true, optimizations
        ? ImmutableSet.copyOf(SettingsFlag.values())
        : ImmutableSet.of());
  }

  public ImmutableSet<SettingsFlag> getActiveFlags() {
    return activeFlags;
  }

  public Set<SettingsFlag> getQueriedFlags() {
    return Collections.unmodifiableSet(queriedFlags);
  }

  public boolean queryFlag(SettingsFlag flag) {
    queriedFlags.add(flag);
    return activeFlags.contains(flag);
  }

  @Override
  public String toString() {
    return "EvaluationSettings(uRBB=" + usingRootBasedBranches + ", flags=" + activeFlags + ")";
  }
}
