// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator.impl;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Maps;
import com.sieuferd.datamodel.Data;
import com.sieuferd.datamodel.Path.PrimitivePath;
import com.sieuferd.datamodel.PrimitiveValue;
import com.sieuferd.datamodel.Tuple;
import com.sieuferd.evaluator.impl.RowFetcher.Row;
import com.sieuferd.util.MapUtil;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import javax.annotation.Nullable;
import javax.annotation.concurrent.NotThreadSafe;

@NotThreadSafe
public final class KeyMatcher {
  private final Data data;
  private final ImmutableList<PrimitivePath> keyPaths;
  // TODO[pd]: Read keys sequentially from data as ordered rows come in.
  private final Map<ImmutableList<PrimitiveValue>,Tuple> content = Maps.newLinkedHashMap();

  public KeyMatcher(Data data, Collection<PrimitivePath> keyPaths, boolean isRoot) {
    this.data = data;
    this.keyPaths = ImmutableList.copyOf(keyPaths);
    // Add the unit tuple for the root relation.
    if (isRoot)
      MapUtil.putChecked(content, ImmutableList.of(), Tuple.createRootParent(data));
  }

  public Collection<PrimitivePath> getKeyPaths() {
    return keyPaths;
  }

  public void addTuple(Tuple tuple, Row rowWithKey) {
    // TODO[pd]: Consider pulling the key from data rather than from rowWithKey.
    MapUtil.putChecked(content, getKey(rowWithKey), tuple);
  }

  public @Nullable Tuple findTupleNullable(Row rowWithKey) {
    return content.get(getKey(rowWithKey));
  }

  public Tuple findTuple(Row rowWithKey) {
    return MapUtil.getChecked(content, getKey(rowWithKey));
  }

  private ImmutableList<PrimitiveValue> getKey(Row row) {
    Preconditions.checkNotNull(row);
    final List<PrimitiveValue> ret = new ArrayList<>(keyPaths.size());
    for (PrimitivePath keyPath : keyPaths)
      ret.add(row.getValue(keyPath));
    return ImmutableList.copyOf(ret);
  }
}
