// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator.impl;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableSet;
import com.sieuferd.database.dialect.RenderedExpression;
import com.sieuferd.datamodel.Path;
import com.sieuferd.datamodel.Path.PrimitivePath;
import com.sieuferd.evaluator.impl.SQLIdentifier.InstanceID;

final class RelationInstance {
  private final InstanceID instanceID;
  private final boolean leftJoined;
  private final QueryBlock queryBlock;

  RelationInstance(InstanceID instanceID, boolean leftJoined, QueryBlock queryBlock) {
    Preconditions.checkNotNull(instanceID);
    Preconditions.checkNotNull(queryBlock);
    this.instanceID = instanceID;
    this.leftJoined = leftJoined;
    this.queryBlock = queryBlock;
  }

  public InstanceID getInstanceID() {
    return instanceID;
  }

  public boolean isLeftJoined() {
    return leftJoined;
  }

  public QueryBlock getQueryBlock() {
    return queryBlock;
  }

  @Override
  public String toString() {
    return instanceID.toString() + " (" + (leftJoined ? "left" : "inner") + ")";
  }

  public RenderedExpression requestFromSelectList(
      PrimitivePath requestedPath, ImmutableSet<Path> alreadyVisitedPaths)
  {
    final Column column = queryBlock.requestFromSelectList(requestedPath, alreadyVisitedPaths);
    return column.getRenderedExpression(instanceID);
  }

  public Evaluation requestAggregation() {
    final Column column = queryBlock.getAggregationColumn();
    return new Evaluation(
        column.getRenderedExpression(instanceID), queryBlock.getAggregationWarnings());
  }
}
