// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator.impl;

import com.google.common.base.Preconditions;
import com.sieuferd.database.SQLType;
import com.sieuferd.database.dialect.BackendDialect;
import com.sieuferd.database.dialect.RenderVariant;
import java.sql.ResultSet;
import java.sql.SQLException;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class ResultColumn {
  private final int columnIndex;
  private final SQLType jdbcType;
  private final RenderVariant renderVariant;

  public ResultColumn(int columnIndex, SQLType jdbcType, RenderVariant renderVariant) {
    Preconditions.checkNotNull(jdbcType);
    Preconditions.checkArgument(columnIndex >= 1);
    Preconditions.checkNotNull(renderVariant);
    this.columnIndex = columnIndex;
    this.jdbcType = jdbcType;
    this.renderVariant = renderVariant;
  }

  public @Nullable Object fetchValue(BackendDialect dialect, ResultSet resultSet)
      throws SQLException
  {
    Object ret = dialect.fetchResultValue(resultSet, columnIndex, jdbcType, renderVariant);
    if (ret instanceof java.sql.Date ||
        ret instanceof java.sql.Time ||
        ret instanceof java.sql.Timestamp)
    {
      throw new SQLException("Backend dialect did not properly convert " +
          ret.getClass().getCanonicalName() + " JDBC type to corresponding java.time type");
    }
    return ret;
  }
}
