// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator;

import com.google.common.base.Joiner;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.sieuferd.datamodel.expressions.Function;
import com.sieuferd.datamodel.expressions.Expression;
import com.sieuferd.datamodel.expressions.Expression.ConstantExpression;
import com.sieuferd.datamodel.expressions.Expression.FunctionExpression;
import java.util.Collections;
import java.util.List;
import java.util.Set;

public final class FunctionTestRequirements {
  private final Set<String> remainingRequirements = Sets.newLinkedHashSet();

  public FunctionTestRequirements() {
  }

  public void addRequirements(Function function) {
    if (Function.AGGREGATE_FUNCTIONS.contains(function) || !function.isBuiltin())
      return;
    String fname = function.toString();
    Integer argCount = function.getArgumentCount();
    if (argCount == null) {
      remainingRequirements.add(fname + "()");
      remainingRequirements.add(fname + "(x)");
      remainingRequirements.add(fname + "(x, x)");
      remainingRequirements.add(fname + "(x, x, x)");
    } else {
      List<String> args = Lists.newArrayList();
      for (int i = 0; i < argCount; i++)
        args.add("x");
      remainingRequirements.add(fname + "(" + Joiner.on(", ").join(args) + ")");
    }
  }

  public void fulfillRequirements(Expression expression) {
    if (!(expression instanceof FunctionExpression))
      return;
    FunctionExpression fexp = (FunctionExpression) expression;
    String fname = fexp.getFunction().toString();
    List<String> args = Lists.newArrayList();
    for (Expression argument : fexp.getArguments())
      args.add(argument.equals(ConstantExpression.NULL) ? "_" : "x");
    remainingRequirements.remove(fname + "(" + Joiner.on(", ").join(args) + ")");
  }

  public Set<String> getRemainingRequirements() {
    return Collections.unmodifiableSet(remainingRequirements);
  }
}
