// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator;

import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import com.sieuferd.evaluator.impl.EvaluationSettings;
import com.sieuferd.evaluator.impl.EvaluationSettings.SettingsFlag;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Set;
import javax.annotation.Nullable;

public final class SettingsIterable implements Iterable<EvaluationSettings> {
  public static final SettingsIterable INSTANCE = new SettingsIterable();
  private SettingsIterable() { }

  @Override
  public Iterator<EvaluationSettings> iterator() {
    return new Iterator<EvaluationSettings>() {
      private final Set<Set<SettingsFlag>> remainingCombinations =
          Sets.newLinkedHashSet(Sets.powerSet(ImmutableSet.copyOf(SettingsFlag.values())));
      private @Nullable EvaluationSettings previous = null;
      private @Nullable EvaluationSettings next = null;

      private void updateNext() {
        if (next != null)
          return;
        if (previous == null) {
          next = EvaluationSettings.create(false);
        } else {
          final Set<SettingsFlag> unqueriedFlags = Sets.difference(
              ImmutableSet.copyOf(SettingsFlag.values()), previous.getQueriedFlags());
          final Set<SettingsFlag> activeQueriedFlags =
              Sets.difference(previous.getActiveFlags(), unqueriedFlags);
          for (Set<SettingsFlag> unqueriedFlagCombination : Sets.powerSet(unqueriedFlags))
            remainingCombinations.remove(Sets.union(unqueriedFlagCombination, activeQueriedFlags));
          if (remainingCombinations.isEmpty()) {
            next = null;
          } else {
            next = new EvaluationSettings(
                previous.isUsingRootBasedBranches(), remainingCombinations.iterator().next());
          }
        }
      }

      @Override
      public boolean hasNext() {
        updateNext();
        return next != null;
      }

      @Override
      public EvaluationSettings next() {
        updateNext();
        if (next == null)
          throw new NoSuchElementException();
        previous = next;
        next = null;
        return previous;
      }

      @Override
      public void remove() {
        throw new UnsupportedOperationException();
      }
    };
  }
}
