// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableMap;
import com.sieuferd.database.SQLType;
import com.sieuferd.database.SQLTypeCode;
import com.sieuferd.query.SQLDatabase;
import java.sql.Types;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.Map;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class TestConfiguration {
  public static final Map<Class<?>,SQLType> SQL_TYPES = ImmutableMap.<Class<?>,SQLType>builder()
      .put(String.class, new SQLType(SQLTypeCode.create(Types.VARCHAR), "", 0, 0, 255, false))
      .put(Integer.class, new SQLType(SQLTypeCode.create(Types.BIGINT), "", 0, 0, 0, false))
      .put(LocalTime.class, new SQLType(SQLTypeCode.create(Types.TIME), "", 0, 0, 0, false))
      .put(LocalDate.class, new SQLType(SQLTypeCode.create(Types.DATE), "", 0, 0, 0, false))
      .put(Instant.class, new SQLType(SQLTypeCode.create(Types.TIMESTAMP), "", 0, 0, 0, false))
      .put(Double.class, new SQLType(SQLTypeCode.create(Types.DOUBLE), "", 0, 0, 0, false))
      .build();
  public static final TestConfiguration MYSQL = new TestConfiguration(
      "jdbc:mysql://127.0.0.1:3306/evaluatortest",
      "duser", "dpass", new SQLDatabase("", ""),
      false, false);
  public static final TestConfiguration POSTGRES = new TestConfiguration(
      "jdbc:postgresql://127.0.0.1:5432/evaluatortest",
      "duser", "dpass", new SQLDatabase("", ""),
      true, true);
  public static final TestConfiguration ORACLE = new TestConfiguration(
      "jdbc:oracle:thin:@localhost:1521:XE",
      "HR", "oracle", new SQLDatabase("HR", ""),
      // Could use CREATE GLOBAL TEMPORARY TABLE ( ... ) ON COMMIT PRESERVE ROWS
      false, false);

  public final String jdbcString;
  public final String user;
  public final String pass;
  public final SQLDatabase database;
  public final boolean temporaryTables;
  public final boolean includePseudocolumnInPassthroughView;

  public TestConfiguration(
      String jdbcString, String user, String pass, SQLDatabase database, boolean temporaryTables,
      boolean includePseudocolumnInPassthroughView)
  {
    Preconditions.checkNotNull(jdbcString);
    Preconditions.checkNotNull(user);
    Preconditions.checkNotNull(pass);
    Preconditions.checkNotNull(database);
    this.jdbcString = jdbcString;
    this.user = user;
    this.pass = pass;
    this.database = database;
    this.temporaryTables = temporaryTables;
    this.includePseudocolumnInPassthroughView = includePseudocolumnInPassthroughView;
  }
}
