// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.evaluator;

import com.google.common.base.Objects;
import com.google.common.base.Preconditions;
import java.util.Arrays;
import javax.annotation.concurrent.Immutable;

@Immutable
public final class TestTableContents {
  private final String columnNames[];
  private final Class<?> columnTypes[];
  private final Object[][] data;

  public TestTableContents(String[] columnNames, Class<?>[] columnTypes, Object[][] data) {
    this.columnNames = Arrays.copyOf(columnNames, columnNames.length);
    this.columnTypes = Arrays.copyOf(columnTypes, columnTypes.length);
    Preconditions.checkArgument(columnNames.length == columnTypes.length);
    Preconditions.checkArgument(!Arrays.asList(this.columnNames).contains(null));
    Preconditions.checkArgument(!Arrays.asList(this.columnTypes).contains(null));
    this.data = copyData(data, columnNames.length);
  }

  private static Object[][] copyData(Object[][] data, int expectedColumns) {
    Object[][] ret = new Object[data.length][];
    for (int i = 0; i < data.length; i++) {
      Preconditions.checkArgument(data[i].length == expectedColumns);
      ret[i] = Arrays.copyOf(data[i], expectedColumns);
    }
    return ret;
  }

  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof TestTableContents))
      return false;
    final TestTableContents other = (TestTableContents) obj;
    return Arrays.equals(this.columnNames, other.columnNames)
        && Arrays.equals(this.columnTypes, other.columnTypes)
        && Arrays.deepEquals(this.data, other.data);
  }

  @Override
  public int hashCode() {
    return Objects.hashCode(
        Arrays.hashCode(this.columnNames),
        Arrays.hashCode(this.columnTypes),
        Arrays.deepHashCode(this.data)
    );
  }

  public String[] getColumnNames() {
    return Arrays.copyOf(columnNames, columnNames.length);
  }

  public Class<?>[] getColumnTypes() {
    return Arrays.copyOf(columnTypes, columnTypes.length);
  }

  public Object[][] getData() {
    return copyData(data, columnNames.length);
  }
}
