// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.importer.excel;

import com.google.common.base.Preconditions;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.sieuferd.importer.ImportException;
import com.sieuferd.importer.excel.Spreadsheet.DataHandler;
import com.sieuferd.importer.excel.Spreadsheet.ParseWarning;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.List;
import java.util.Set;

final class TSVGenerator implements DataHandler {
  private final Set<ParseWarning> warnings = Sets.newLinkedHashSet();
  private final List<Object> outputRow = Lists.newArrayList();
  private final PrintWriter out;
  private int colNum;

  /** @param out will not be closed */
  public TSVGenerator(PrintWriter out) {
    Preconditions.checkNotNull(out);
    this.out = out;
  }

  private void outputValue(String s) {
    if (colNum > 0)
      out.print("\t");
    out.print("\"");
    out.print(s.replace("\"", "\"\""));
    out.print("\"");
    colNum++;
  }

  @Override
  public void rowComplete() throws ImportException {
    for (Object val : outputRow) {
      outputValue(val == null ? "" : val.getClass().getSimpleName());
      outputValue(val == null ? "" : val.toString());
    }
    outputRow.clear();
    colNum = 0;
    // Don't use println; it uses a platform-dependent line ending.
    out.print("\n");
  }

  @Override
  public void cell(int columnIndex, Object data) {
    while (columnIndex > outputRow.size())
      outputRow.add(null);
    outputRow.add(data);
  }

  @Override
  public void reportWarning(ParseWarning warning) {
    warnings.add(warning);
  }

  public Set<ParseWarning> getWarnings() {
    return Collections.unmodifiableSet(warnings);
  }
}
