// The SIEUFERD project by Eirik Bakke 2010-2016, ebakke@alum.mit.edu.
package com.sieuferd.web.main;

import com.google.common.collect.ImmutableList;
import com.google.gwt.animation.client.AnimationScheduler;
import com.google.gwt.canvas.client.Canvas;
import com.google.gwt.canvas.dom.client.Context2d;
import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.logical.shared.ResizeEvent;
import com.google.gwt.event.logical.shared.ResizeHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.RootLayoutPanel;
import java.util.List;
import com.sieuferd.datamodel.gwttest.ExternalTestClass;

public class WebSieuferd implements EntryPoint {
  private final Messages messages = GWT.create(Messages.class);
  private Canvas canvas;
  private int canvasWidth;
  private int canvasHeight;

  @Override
  public void onModuleLoad() {
    canvas = Canvas.createIfSupported();
    if (canvas == null)
      return;
    RootLayoutPanel.get().add(canvas);
    // Test Guava availability.
    List<String> testList = ImmutableList.of("test", "test2");
    GWT.log(testList.toString());
    doOnResize();
    Window.addResizeHandler(new ResizeHandler() {
      @Override
      public void onResize(ResizeEvent event) {
        doOnResize();
      }
    });

    final int cellWidth = 120;
    final int cellHeight = 17;

    AnimationScheduler.get().requestAnimationFrame(new AnimationScheduler.AnimationCallback() {
      @Override
      public void execute(double timestamp) {
        long counter = 0;
        long scroll = Math.round(timestamp / 10);
        canvas.getContext2d().setFillStyle("gray");
        canvas.getContext2d().fillRect(0, 0, canvasWidth, canvasHeight);
        canvas.getContext2d().setFillStyle("black");
        ExternalTestClass testObject = new ExternalTestClass("Yes!");
        canvas.getContext2d().fillText(messages.helloWorld() + " " + testObject.getFoo(), 10, 10);
        for (int y = 0; y < canvasHeight; y+= cellHeight) {
          for (int x = 0; x < canvasWidth; x+= cellWidth) {
            drawCell(canvas.getContext2d(), x, y +
                (Math.abs((scroll % 400) - 200)), cellWidth, cellHeight, counter++);
          }
        }
        AnimationScheduler.get().requestAnimationFrame(this, null);
      }
    }, canvas.getCanvasElement());
  }

  public static void drawCell(
      Context2d context, double x, double y, double w, double h, long counter)
  {
    context.fillText(counter + "", x, y);
    context.beginPath();
    context.moveTo(x, y);
    context.lineTo(x + w, y);
    context.stroke();
  }

  private void doOnResize() {
    // TODO[pd]: Handle null return values.
    canvasWidth = canvas.getParent().getOffsetWidth() + 10;
    canvasHeight = canvas.getParent().getOffsetHeight() + 10;
    GWT.log("W " + canvasWidth + " H " + canvasHeight);
    canvas.setWidth(canvasWidth + "px");
    canvas.setHeight(canvasHeight + "px");
    canvas.setCoordinateSpaceWidth(canvasWidth);
    canvas.setCoordinateSpaceHeight(canvasHeight);
  }
}
